@@ -25,19 +25,16 @@
 #include <time.h>
 #ifdef _WIN32
 #include <malloc.h>
-#define NTDDI_VERSION 0x0501
-#define _WIN32_WINNT 0x0501
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #include <windows.h>
 #include <io.h>
+#include <direct.h>
 #endif
 #include "types.h"
 #include "config.h"
-#ifdef	PARAMH
-#include <sys/param.h>
-#endif
 #ifndef _WIN32
+#include <sys/param.h>
 #include <stdbool.h>
 #else
 typedef int bool;
@@ -60,6 +57,12 @@ typedef int bool;
 
 #define BMAGIC 0x4675636B596F754661736369737473
 
+#ifdef _WIN32
+#define DEADBEEF_ADDR 0xDEADBEEFDEADBEEF
+#else
+#define DEADBEEF_ADDR 0xDEADBEEF
+#endif
+
 #define BASE_VERSION "UnrealIRCd"
 #ifndef _WIN32
 #define FDwrite(x,y,z) write(x, y, z)
@@ -85,19 +88,6 @@ typedef int bool;
 #define UNSURE (2)
 #endif
 
-#if 0
-#ifndef	MALLOCH
-char *malloc(), *calloc();
-void free();
-#else
-#include MALLOCH
-#endif
-#endif
-
-#define TS time_t
-
-
-extern int match(const char *, const char *);
 #define mycmp(a,b) \
  ( (toupper(a[0])!=toupper(b[0])) || smycmp((a)+1,(b)+1) )
 extern int smycmp(const char *, const char *);
@@ -105,34 +95,13 @@ extern int smycmp(const char *, const char *);
 extern int myncmp(const char *, const char *, int);
 #endif
 
-#ifdef NEED_STRTOK
-extern char *strtok2(char *, char *);
-#endif
-#ifdef NEED_STRTOKEN
 extern char *strtoken(char **, char *, char *);
-#endif
-#ifdef NEED_INET_ADDR
-extern unsigned long inet_addr(char *);
-#endif
-
-#if defined(NEED_INET_NTOA) || defined(NEED_INET_NETOF) && !defined(_WIN32)
-#include <netinet/in.h>
-#endif
-#ifdef NEED_INET_NTOA
-extern char *inet_ntoa(struct in_addr);
-#endif
-
-#ifndef HAVE_INET_NTOP
-const char *inet_ntop(int, const void *, char *, size_t);
-#endif
-
-#ifndef HAVE_INET_PTON
-int inet_pton(int af, const char *src, void *dst);
-#endif
 
 extern MODVAR int  global_count, max_global_count;
 extern char *myctime(time_t);
-extern char *strtoken(char **, char *, char *);
+#ifdef _WIN32
+extern int gettimeofday(struct timeval *tp, void *tzp);
+#endif
 
 #define PRECISE_CHECK
 
@@ -144,19 +113,10 @@ extern char *strtoken(char **, char *, char *);
 #endif
 
 extern MODVAR u_char tolowertab[], touppertab[];
-
-#if defined(NICK_GB2312) || defined(NICK_GBK) || defined(NICK_GBK_JAP)
-#define USE_LOCALE
-#include <ctype.h>
-#endif
-
-#ifndef USE_LOCALE
 #undef tolower
 #define tolower(c) (tolowertab[(u_char)(c)])
-
 #undef toupper
 #define toupper(c) (touppertab[(u_char)(c)])
-
 #undef isalpha
 #undef isdigit
 #undef isxdigit
@@ -169,7 +129,6 @@ extern MODVAR u_char tolowertab[], touppertab[];
 #undef isupper
 #undef isspace
 #undef iscntrl
-#endif
 extern MODVAR unsigned char char_atribs[];
 
 #define PRINT 1
@@ -189,7 +148,6 @@ extern MODVAR unsigned char char_atribs[];
 #endif
 
 #define isallowed(c) (char_atribs[(u_char)(c)]&ALLOW)
-#ifndef USE_LOCALE
 #define	iscntrl(c) (char_atribs[(u_char)(c)]&CNTRL)
 #define isalpha(c) (char_atribs[(u_char)(c)]&ALPHA)
 #define isspace(c) (char_atribs[(u_char)(c)]&SPACE)
@@ -203,34 +161,8 @@ extern MODVAR unsigned char char_atribs[];
 #define isascii(c) ((u_char)(c) >= 0 && (u_char)(c) <= 0x7f)
 #define isgraph(c) ((char_atribs[(u_char)(c)]&PRINT) && ((u_char)(c) != 0x32))
 #define ispunct(c) (!(char_atribs[(u_char)(c)]&(CNTRL|ALPHA|DIGIT)))
-#endif
 #define iswseperator(c) (!isalnum(c) && !((u_char)c >= 128))
 
-#ifndef MALLOCD
-#define MyFree free
-#define MyMalloc malloc
-#define MyRealloc realloc
-#else
-#define MyFree(x) do {debug(DEBUG_MALLOC, "%s:%i: free %02x", __FILE__, __LINE__, x); free(x); } while(0)
-#define MyMalloc(x) StsMalloc(x, __FILE__, __LINE__)
-#define MyRealloc realloc
-static char *StsMalloc(size_t size, char *file, long line)
-{
-	void *x;
-	
-	x = malloc(size);
-	debug(DEBUG_MALLOC, "%s:%i: malloc %02x", file, line, x);
-	return x;
-}
-
-#endif
-
-#define safestrdup(x,y) do { if (x) MyFree(x); if (!(y)) x = NULL; else x = strdup(y); } while(0)
-#define safestrldup(x,y,sz) do { if (x) MyFree(x); if (!y) x = NULL; else x = strldup(y,sz); } while(0)
-#define safefree(x) do { if (x) MyFree(x); x = NULL; } while(0)
-
-extern struct SLink *find_user_link( /* struct SLink *, struct Client * */ );
-
 /*
  * Protocol support text.  DO NO CHANGE THIS unless you know what
  * you are doing.
@@ -274,7 +206,7 @@ extern int lu_noninv, lu_inv, lu_serv, lu_oper,
     lu_unknown, lu_channel, lu_lu, lu_lulocal, lu_lserv,
     lu_clu, lu_mlu, lu_cglobalu, lu_mglobalu;
 
-extern MODVAR TS   now;
+extern MODVAR time_t now;
 
 #ifndef _WIN32
 #if defined(__STDC__)
