@@ -1,6 +1,7 @@
 /************************************************************************
  *   Unreal Internet Relay Chat Daemon, include/dynconf.h
- *   Copyright (C) 1999 Carsten Munk
+ *   Copyright (C) 1999-2003 Carsten Munk
+ *   Copyright (C) 2003-2021 Bram Matthys
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,12 +22,18 @@
 
 
 #define DYNCONF_H
-/* config level */
-#define DYNCONF_CONF_VERSION "1.5"
-#define DYNCONF_NETWORK_VERSION "2.2"
 
-typedef struct zNetwork aNetwork;
-struct zNetwork {
+typedef struct FloodSettings FloodSettings;
+
+struct FloodSettings {
+	FloodSettings *prev, *next;
+	char *name;
+	int limit[MAXFLOODOPTIONS];
+	long period[MAXFLOODOPTIONS];
+};
+
+typedef struct NetworkConfiguration NetworkConfiguration;
+struct NetworkConfiguration {
 	unsigned x_inah:1;
 	char *x_ircnetwork;
 	char *x_ircnet005;
@@ -47,13 +54,22 @@ struct ChMode {
 	char *extparams[EXTCMODETABLESZ];
 };
 
-typedef struct _OperStat {
-	struct _OperStat *prev, *next;
+typedef struct OperStat {
+	struct OperStat *prev, *next;
 	char *flag;
 } OperStat;
 
-typedef struct zConfiguration aConfiguration;
-struct zConfiguration {
+typedef enum BroadcastChannelMessagesOption { BROADCAST_CHANNEL_MESSAGES_AUTO=1, BROADCAST_CHANNEL_MESSAGES_ALWAYS=2, BROADCAST_CHANNEL_MESSAGES_NEVER=3 } BroadcastChannelMessagesOption;
+
+typedef enum AllowedChannelChars { ALLOWED_CHANNELCHARS_ANY=1, ALLOWED_CHANNELCHARS_ASCII=2, ALLOWED_CHANNELCHARS_UTF8=3 } AllowedChannelChars;
+
+typedef enum BanTarget { BAN_TARGET_IP=1, BAN_TARGET_USERIP=2, BAN_TARGET_HOST=3, BAN_TARGET_USERHOST=4, BAN_TARGET_ACCOUNT=5, BAN_TARGET_CERTFP=6 } BanTarget;
+
+typedef enum HideIdleTimePolicy { HIDE_IDLE_TIME_NEVER=1, HIDE_IDLE_TIME_ALWAYS=2, HIDE_IDLE_TIME_USERMODE=3, HIDE_IDLE_TIME_OPER_USERMODE=4 } HideIdleTimePolicy;
+
+/** The set { } block configuration */
+typedef struct Configuration Configuration;
+struct Configuration {
 	unsigned som:1;
 	unsigned hide_ulines:1;
 	unsigned flat_map:1;
@@ -61,7 +77,7 @@ struct zConfiguration {
 	unsigned ident_check:1;
 	unsigned fail_oper_warn:1;
 	unsigned show_connect_info:1;
-	unsigned no_connect_ssl_info:1;
+	unsigned no_connect_tls_info:1;
 	unsigned dont_resolve:1;
 	unsigned use_ban_version:1;
 	unsigned mkpasswd_for_everyone:1;
@@ -80,22 +96,22 @@ struct zConfiguration {
 	long conn_modes;
 	long oper_modes;
 	char *oper_snomask;
-	char *user_snomask;
 	char *auto_join_chans;
 	char *oper_auto_join_chans;
-	char *oper_only_stats;
-	OperStat *oper_only_stats_ext;
+	char *allow_user_stats;
+	OperStat *allow_user_stats_ext;
+	int  ping_warning;
 	int  maxchannelsperuser;
 	int  maxdccallow;
 	int  anti_spam_quit_message_time;
 	char *egd_path;
 	char *static_quit;
 	char *static_part;
-	SSLOptions *ssl_options;
+	TLSOptions *tls_options;
 	Policy plaintext_policy_user;
-	char *plaintext_policy_user_message;
+	MultiLine *plaintext_policy_user_message;
 	Policy plaintext_policy_oper;
-	char *plaintext_policy_oper_message;
+	MultiLine *plaintext_policy_oper_message;
 	Policy plaintext_policy_server;
 	Policy outdated_tls_policy_user;
 	char *outdated_tls_policy_user_message;
@@ -106,22 +122,13 @@ struct zConfiguration {
 	char *restrict_usermodes;
 	char *restrict_channelmodes;
 	char *restrict_extendedbans;
-	int new_linking_protocol;
 	char *channel_command_prefix;
-	long unknown_flood_bantime;
-	long unknown_flood_amount;
+	long handshake_data_flood_amount;
+	long handshake_data_flood_ban_time;
+	int handshake_data_flood_ban_action;
 	struct ChMode modes_on_join;
 	int level_on_join;
-	unsigned char away_count;
-	long away_period;
-	unsigned char nick_count;
-	long nick_period;
-	unsigned char invite_count;
-	long invite_period;
-	unsigned char knock_count;
-	long knock_period;
-	unsigned char max_concurrent_conversations_users;
-	unsigned char max_concurrent_conversations_new_user_every;
+	FloodSettings *floodsettings;
 	int ident_connect_timeout;
 	int ident_read_timeout;
 	long default_bantime;
@@ -139,14 +146,12 @@ struct zConfiguration {
 	int spamfilter_stop_on_first_match;
 	int maxbans;
 	int maxbanlength;
-	int timesynch_enabled;
-	int timesynch_timeout;
-	char *timesynch_server;
 	int watch_away_notification;
 	int uhnames;
-	aNetwork network;
+	NetworkConfiguration network;
 	unsigned short default_ipv6_clone_mask;
 	int ping_cookie;
+	int min_nick_length;
 	int nick_length;
 	int topic_length;
 	int kick_length;
@@ -155,9 +160,10 @@ struct zConfiguration {
 	int hide_list;
 	int max_unknown_connections_per_ip;
 	long handshake_timeout;
+	long sasl_timeout;
 	long handshake_delay;
-	int ban_include_username;
-	char *reject_message_password_mismatch;
+	BanTarget automatic_ban_target;
+	BanTarget manual_ban_target;
 	char *reject_message_too_many_connections;
 	char *reject_message_server_full;
 	char *reject_message_unauthorized;
@@ -166,24 +172,26 @@ struct zConfiguration {
 	int topic_setter;
 	int ban_setter;
 	int ban_setter_sync;
+	int part_instead_of_quit_on_comment_change;
+	BroadcastChannelMessagesOption broadcast_channel_messages;
+	AllowedChannelChars allowed_channelchars;
+	HideIdleTimePolicy hide_idle_time;
 };
 
-#ifndef DYNCONF_C
-extern MODVAR aConfiguration iConf;
-extern MODVAR aConfiguration tempiConf;
+extern MODVAR Configuration iConf;
+extern MODVAR Configuration tempiConf;
 extern MODVAR int ipv6_disabled;
-#endif
 
 #define KLINE_ADDRESS			iConf.kline_address
 #define GLINE_ADDRESS			iConf.gline_address
 #define CONN_MODES			iConf.conn_modes
 #define OPER_MODES			iConf.oper_modes
 #define OPER_SNOMASK			iConf.oper_snomask
-#define CONNECT_SNOMASK			iConf.user_snomask
 #define SHOWOPERMOTD			iConf.som
 #define HIDE_ULINES			iConf.hide_ulines
 #define FLAT_MAP			iConf.flat_map
 #define ALLOW_CHATOPS			iConf.allow_chatops
+#define PINGWARNING			iConf.ping_warning
 #define MAXCHANNELSPERUSER		iConf.maxchannelsperuser
 #define MAXDCCALLOW			iConf.maxdccallow
 #define DONT_RESOLVE			iConf.dont_resolve
@@ -194,8 +202,8 @@ extern MODVAR int ipv6_disabled;
 #define IDENT_CHECK			iConf.ident_check
 #define FAILOPER_WARN			iConf.fail_oper_warn
 #define SHOWCONNECTINFO			iConf.show_connect_info
-#define NOCONNECTSSLINFO		iConf.no_connect_ssl_info
-#define OPER_ONLY_STATS			iConf.oper_only_stats
+#define NOCONNECTTLSLINFO		iConf.no_connect_tls_info
+#define ALLOW_USER_STATS			iConf.allow_user_stats
 #define ANTI_SPAM_QUIT_MSG_TIME		iConf.anti_spam_quit_message_time
 #ifdef HAVE_RAND_EGD
 #define USE_EGD				iConf.use_egd
@@ -221,24 +229,12 @@ extern MODVAR int ipv6_disabled;
 #define RESTRICT_USERMODES		iConf.restrict_usermodes
 #define RESTRICT_CHANNELMODES		iConf.restrict_channelmodes
 #define RESTRICT_EXTENDEDBANS		iConf.restrict_extendedbans
-#define NEW_LINKING_PROTOCOL		iConf.new_linking_protocol
 #define THROTTLING_PERIOD		iConf.throttle_period
 #define THROTTLING_COUNT		iConf.throttle_count
 #define USE_BAN_VERSION			iConf.use_ban_version
-#define UNKNOWN_FLOOD_BANTIME		iConf.unknown_flood_bantime
-#define UNKNOWN_FLOOD_AMOUNT		iConf.unknown_flood_amount
 #define MODES_ON_JOIN			iConf.modes_on_join.mode
 #define LEVEL_ON_JOIN			iConf.level_on_join
 
-#define AWAY_PERIOD			iConf.away_period
-#define AWAY_COUNT			iConf.away_count
-#define NICK_PERIOD			iConf.nick_period
-#define NICK_COUNT			iConf.nick_count
-#define KNOCK_PERIOD			iConf.knock_period
-#define KNOCK_COUNT			iConf.knock_count
-#define INVITE_PERIOD			iConf.invite_period
-#define INVITE_COUNT			iConf.invite_count
-
 #define IDENT_CONNECT_TIMEOUT	iConf.ident_connect_timeout
 #define IDENT_READ_TIMEOUT		iConf.ident_read_timeout
 
@@ -273,15 +269,13 @@ extern MODVAR int ipv6_disabled;
 #define MAXBANS		iConf.maxbans
 #define MAXBANLENGTH	iConf.maxbanlength
 
-#define TIMESYNCH	iConf.timesynch_enabled
-#define TIMESYNCH_TIMEOUT	iConf.timesynch_timeout
-#define TIMESYNCH_SERVER	iConf.timesynch_server
-
 #define WATCH_AWAY_NOTIFICATION	iConf.watch_away_notification
 
 #define UHNAMES_ENABLED	iConf.uhnames
 
-/* Used for "is present?" and duplicate checking */
+/** Used for testing the set { } block configuration.
+ * It tests if a setting is present and is also used for duplicate checking.
+ */
 struct SetCheck {
 	unsigned has_show_opermotd:1;
 	unsigned has_hide_ulines:1;
@@ -293,9 +287,8 @@ struct SetCheck {
 	unsigned has_dont_resolve:1;
 	unsigned has_mkpasswd_for_everyone:1;
 	unsigned has_allow_part_if_shunned:1;
-	unsigned has_ssl_egd:1;
-	unsigned has_ssl_server_cipher_list :1;
-	unsigned has_ssl_protocols :1;
+	unsigned has_tls_server_cipher_list :1;
+	unsigned has_tls_protocols :1;
 	unsigned has_dns_bind_ip:1;
 	unsigned has_link_bind_ip:1;
 	unsigned has_throttle_period:1;
@@ -311,7 +304,8 @@ struct SetCheck {
 	unsigned has_check_target_nick_bans:1;
 	unsigned has_watch_away_notification:1;
 	unsigned has_uhnames:1;
-	unsigned has_oper_only_stats:1;
+	unsigned has_allow_user_stats:1;
+	unsigned has_ping_warning:1;
 	unsigned has_maxchannelsperuser:1;
 	unsigned has_maxdccallow:1;
 	unsigned has_anti_spam_quit_message_time:1;
@@ -322,18 +316,9 @@ struct SetCheck {
 	unsigned has_restrict_usermodes:1;
 	unsigned has_restrict_channelmodes:1;
 	unsigned has_restrict_extendedbans:1;
-	unsigned has_new_linking_protocol:1;
 	unsigned has_channel_command_prefix:1;
-	unsigned has_anti_flood_unknown_flood_bantime:1;
-	unsigned has_anti_flood_unknown_flood_amount:1;
 	unsigned has_modes_on_join:1;
 	unsigned has_level_on_join:1;
-	unsigned has_anti_flood_away_count:1;
-	unsigned has_anti_flood_away_period:1;
-	unsigned has_anti_flood_nick_flood:1;
-	unsigned has_anti_flood_connect_flood:1;
-	unsigned has_anti_flood_invite_flood:1;
-	unsigned has_anti_flood_knock_flood:1;
 	unsigned has_ident_connect_timeout:1;
 	unsigned has_ident_read_timeout:1;
 	unsigned has_default_bantime:1;
@@ -363,15 +348,14 @@ struct SetCheck {
 	unsigned has_options_fail_oper_warn:1;
 	unsigned has_options_dont_resolve:1;
 	unsigned has_options_show_connect_info:1;
-	unsigned has_options_no_connect_ssl_info:1;
+	unsigned has_options_no_connect_tls_info:1;
 	unsigned has_options_mkpasswd_for_everyone:1;
 	unsigned has_options_allow_insane_bans:1;
 	unsigned has_options_allow_part_if_shunned:1;
 	unsigned has_options_disable_cap:1;
 	unsigned has_options_disable_ipv6:1;
 	unsigned has_ping_cookie:1;
+	unsigned has_min_nick_length:1;
 	unsigned has_nick_length:1;
 	unsigned has_hide_ban_reason:1;
 };
-
-
