@@ -1,6 +1,7 @@
 /*
  *   Unreal Internet Relay Chat Daemon, src/auth.c
  *   (C) 2001 Carsten V. Munk (stskeeps@tspre.org)
+ *   (C) 2003-2019 Bram Matthys (syzop@vulnscan.org) and the UnrealIRCd team
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,35 +20,61 @@
 
 #include "unrealircd.h"
 #include "crypt_blowfish.h"
-#include <argon2.h>
 
-anAuthStruct MODVAR AuthTypes[] = {
+typedef struct AuthTypeList AuthTypeList;
+struct AuthTypeList {
+	char			*name;
+	AuthenticationType	type;
+};
+
+/** The list of authentication types that we support. */
+AuthTypeList MODVAR AuthTypeLists[] = {
 	{"plain",           AUTHTYPE_PLAINTEXT},
 	{"plaintext",       AUTHTYPE_PLAINTEXT},
-	{"md5",             AUTHTYPE_MD5},
-	{"sha1",            AUTHTYPE_SHA1},
-	{"ripemd160",       AUTHTYPE_RIPEMD160},
 	{"crypt",           AUTHTYPE_UNIXCRYPT},
 	{"unixcrypt",       AUTHTYPE_UNIXCRYPT},
 	{"bcrypt",          AUTHTYPE_BCRYPT},
-	{"sslclientcert",   AUTHTYPE_SSL_CLIENTCERT},
-	{"cert",            AUTHTYPE_SSL_CLIENTCERT},
-	{"sslclientcertfp", AUTHTYPE_SSL_CLIENTCERTFP},
-	{"certfp",          AUTHTYPE_SSL_CLIENTCERTFP},
+	{"cert",            AUTHTYPE_TLS_CLIENTCERT},
+	{"sslclientcert",   AUTHTYPE_TLS_CLIENTCERT},
+	{"tlsclientcert",   AUTHTYPE_TLS_CLIENTCERT},
+	{"certfp",          AUTHTYPE_TLS_CLIENTCERTFP},
+	{"sslclientcertfp", AUTHTYPE_TLS_CLIENTCERTFP},
+	{"tlsclientcertfp", AUTHTYPE_TLS_CLIENTCERTFP},
 	{"spkifp",          AUTHTYPE_SPKIFP},
 	{"argon2",          AUTHTYPE_ARGON2},
 	{NULL,              0}
 };
 
-/* Forward declarations */
-static int parsepass(char *str, char **salt, char **hash);
+/* Helper function for Auth_AutoDetectHashType() */
+static int parsepass(char *str, char **salt, char **hash)
+{
+	static char saltbuf[512], hashbuf[512];
+	char *p;
+	int max;
+
+	/* Syntax: $<salt>$<hash> */
+	if (*str != '$')
+		return 0;
+	p = strchr(str+1, '$');
+	if (!p || (p == str+1) || !p[1])
+		return 0;
+
+	max = p - str;
+	if (max > sizeof(saltbuf))
+		max = sizeof(saltbuf);
+	strlcpy(saltbuf, str+1, max);
+	strlcpy(hashbuf, p+1, sizeof(hashbuf));
+	*salt = saltbuf;
+	*hash = hashbuf;
+	return 1;
+}
 
 /** Auto detect hash type for input hash 'hash'.
  * Will fallback to AUTHTYPE_PLAINTEXT when not found (or invalid).
  */
 int Auth_AutoDetectHashType(char *hash)
 {
-	static char buf[512], hashbuf[256];
+	static char hashbuf[256];
 	char *saltstr, *hashstr;
 	int bits;
 
@@ -63,8 +90,8 @@ int Auth_AutoDetectHashType(char *hash)
 			for (p = hash; *p; p++)
 				if ((*p != ':') && !strchr(hexchars, *p))
 					return AUTHTYPE_PLAINTEXT; /* not hex and not colon */
-			
-			return AUTHTYPE_SSL_CLIENTCERTFP;
+
+			return AUTHTYPE_TLS_CLIENTCERTFP;
 		}
 
 		if (strlen(hash) == 44)
@@ -78,10 +105,10 @@ int Auth_AutoDetectHashType(char *hash)
 			return AUTHTYPE_SPKIFP;
 		}
 	}
-	
+
 	if ((*hash != '$') || !strchr(hash+1, '$'))
 		return AUTHTYPE_PLAINTEXT;
-	
+
 	if (!strncmp(hash, "$2a$", 4) || !strncmp(hash, "$2b$", 4) || !strncmp(hash, "$2y$", 4))
 		return AUTHTYPE_BCRYPT;
 
@@ -96,16 +123,6 @@ int Auth_AutoDetectHashType(char *hash)
 	if (bits <= 0)
 		return AUTHTYPE_UNIXCRYPT; /* decode failed. likely some other crypt() type. */
 
-	/* We (only) detect MD5 and SHA1 automatically.
-	 * If, for some reason, you use RIPEMD160 then you'll have to be explicit about it ;)
-	 */
-	
-	if (bits == 128)
-		return AUTHTYPE_MD5;
-	
-	if (bits == 160)
-		return AUTHTYPE_SHA1;
-	
 	/* else it's likely some other crypt() type */
 	return AUTHTYPE_UNIXCRYPT;
 }
@@ -117,36 +134,33 @@ int Auth_AutoDetectHashType(char *hash)
  *               than trying to determine the type on the 'hash' parameter.
  *               Or leave NULL, then we use hash autodetection.
  */
-int	Auth_FindType(char *hash, char *type)
+AuthenticationType Auth_FindType(char *hash, char *type)
 {
 	if (type)
 	{
-		anAuthStruct *p = AuthTypes;
-		while (p->data)
+		AuthTypeList *e = AuthTypeLists;
+		while (e->name)
 		{
-			if (!mycmp(p->data, type))
-				return p->type;
-			p++;
+			if (!mycmp(e->name, type))
+				return e->type;
+			e++;
 		}
-		return -1; /* Not found */
+		return AUTHTYPE_INVALID; /* Not found */
 	}
 
 	if (hash)
 		return Auth_AutoDetectHashType(hash);
 
-	return -1; /* both 'hash' and 'type' are NULL */
+	return AUTHTYPE_INVALID; /* both 'hash' and 'type' are NULL */
 }
 
-/*
- * This is for converting something like:
- * {
- * 	password "data" { type; };
- * } 
+/** Check the syntax of an authentication block.
+ * This is a block like: password "data" { type; };
+ * in the configuration file.
 */
-
-int		Auth_CheckError(ConfigEntry *ce)
+int Auth_CheckError(ConfigEntry *ce)
 {
-	short		type = AUTHTYPE_PLAINTEXT;
+	AuthenticationType type = AUTHTYPE_PLAINTEXT;
 	X509 *x509_filecert = NULL;
 	FILE *x509_f = NULL;
 	if (!ce->ce_vardata)
@@ -182,11 +196,11 @@ int		Auth_CheckError(ConfigEntry *ce)
 				return -1;
 			}
 			break;
-		case AUTHTYPE_SSL_CLIENTCERT:
+		case AUTHTYPE_TLS_CLIENTCERT:
 			convert_to_absolute_path(&ce->ce_vardata, CONFDIR);
 			if (!(x509_f = fopen(ce->ce_vardata, "r")))
 			{
-				config_error("%s:%i: authentication module failure: AUTHTYPE_SSL_CLIENTCERT: error opening file %s: %s",
+				config_error("%s:%i: authentication module failure: AUTHTYPE_TLS_CLIENTCERT: error opening file %s: %s",
 					ce->ce_fileptr->cf_filename, ce->ce_varlinenum, ce->ce_vardata, strerror(errno));
 				return -1;
 			}
@@ -194,7 +208,7 @@ int		Auth_CheckError(ConfigEntry *ce)
 			fclose(x509_f);
 			if (!x509_filecert)
 			{
-				config_error("%s:%i: authentication module failure: AUTHTYPE_SSL_CLIENTCERT: PEM_read_X509 errored in file %s (format error?)",
+				config_error("%s:%i: authentication module failure: AUTHTYPE_TLS_CLIENTCERT: PEM_read_X509 errored in file %s (format error?)",
 					ce->ce_fileptr->cf_filename, ce->ce_varlinenum, ce->ce_vardata);
 				return -1;
 			}
@@ -203,15 +217,6 @@ int		Auth_CheckError(ConfigEntry *ce)
 		default: ;
 	}
 
-	if ((type == AUTHTYPE_MD5) || (type == AUTHTYPE_SHA1) || (type == AUTHTYPE_RIPEMD160))
-	{
-		config_warn("%s:%i: Deprecated authentication type. "
-		            "Consider using the more secure auth-type 'argon2' instead. "
-		            "See https://www.unrealircd.org/docs/Authentication_types for the complete list.",
-		            ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
-		/* do not return, not an error. */
-	}
-
 	/* Unix crypt is a bit more complicated: most types are outright 'bad',
 	 * while other types have reasonable security similar to 'bcrypt'.
 	 * To be honest these people should probably use 'argon2' since it's
@@ -236,37 +241,37 @@ int		Auth_CheckError(ConfigEntry *ce)
 			ce->ce_fileptr->cf_filename, ce->ce_varlinenum, PASSWDLEN);
 		return -1;
 	}
-	return 1;	
+	return 1;
 }
 
-anAuthStruct	*Auth_ConvertConf2AuthStruct(ConfigEntry *ce)
+/** Convert an authentication block from the configuration file
+ * into an AuthConfig structure so it can be used at runtime.
+ */
+AuthConfig *AuthBlockToAuthConfig(ConfigEntry *ce)
 {
-	short		type = AUTHTYPE_PLAINTEXT;
-	anAuthStruct 	*as = NULL;
+	AuthenticationType type = AUTHTYPE_PLAINTEXT;
+	AuthConfig *as = NULL;
 
 	type = Auth_FindType(ce->ce_vardata, ce->ce_entries ? ce->ce_entries->ce_varname : NULL);
-	if (type == -1)
+	if (type == AUTHTYPE_INVALID)
 		type = AUTHTYPE_PLAINTEXT;
 
-	as = MyMallocEx(sizeof(anAuthStruct));
-	as->data = strdup(ce->ce_vardata);
+	as = safe_alloc(sizeof(AuthConfig));
+	safe_strdup(as->data, ce->ce_vardata);
 	as->type = type;
 	return as;
 }
 
-void	Auth_DeleteAuthStruct(anAuthStruct *as)
+/** Free an AuthConfig struct */
+void Auth_FreeAuthConfig(AuthConfig *as)
 {
-	if (!as)
-		return;
-	if (as->data) 
-		MyFree(as->data);
-	MyFree(as);
+	if (as)
+	{
+		safe_free(as->data);
+		safe_free(as);
+	}
 }
 
-/* Both values are pretty insane as of 2004, but... just in case. */
-#define MAXSALTLEN		127
-#define MAXHASHLEN		255
-
 /* RAW salt length (before b64_encode) to use in /MKPASSWD
  * and REAL salt length (after b64_encode, including terminating nul),
  * used for reserving memory.
@@ -274,42 +279,12 @@ void	Auth_DeleteAuthStruct(anAuthStruct *as)
 #define RAWSALTLEN		6
 #define REALSALTLEN		12
 
-/** Parses a password.
- * This routine can parse a pass that has a salt (new as of unreal 3.2.1)
- * and will set the 'salt' pointer and 'hash' accordingly.
- * RETURN VALUES:
- * 1 If succeeded, salt and hash can be used.
- * 0 If it's a password without a salt ('old'), salt and hash are not touched.
- */
-static int parsepass(char *str, char **salt, char **hash)
-{
-static char saltbuf[MAXSALTLEN+1], hashbuf[MAXHASHLEN+1];
-char *p;
-int max;
-
-	/* Syntax: $<salt>$<hash> */
-	if (*str != '$')
-		return 0;
-	p = strchr(str+1, '$');
-	if (!p || (p == str+1) || !p[1])
-		return 0;
-
-	max = p - str;
-	if (max > sizeof(saltbuf))
-		max = sizeof(saltbuf);
-	strlcpy(saltbuf, str+1, max);
-	strlcpy(hashbuf, p+1, sizeof(hashbuf));
-	*salt = saltbuf;
-	*hash = hashbuf;
-	return 1;
-}
-
-static int authcheck_argon2(aClient *cptr, anAuthStruct *as, char *para)
+static int authcheck_argon2(Client *client, AuthConfig *as, char *para)
 {
 	argon2_type hashtype;
 
 	if (!para)
-		return -1;
+		return 0;
 
 	/* Find out the hashtype. Why do we need to do this, why is this
 	 * not in the library or irrelevant by using some generic function?
@@ -321,346 +296,181 @@ static int authcheck_argon2(aClient *cptr, anAuthStruct *as, char *para)
 	else if (!strncmp(as->data, "$argon2d", 8))
 		hashtype = Argon2_d;
 	else
-		return -1; /* unknown argon2 type */
+		return 0; /* unknown argon2 type */
 
 	if (argon2_verify(as->data, para, strlen(para), hashtype) == ARGON2_OK)
-		return 2; /* MATCH */
+		return 1; /* MATCH */
 
-	return -1; /* NO MATCH or error */
+	return 0; /* NO MATCH or error */
 }
 
-static int authcheck_bcrypt(aClient *cptr, anAuthStruct *as, char *para)
+static int authcheck_bcrypt(Client *client, AuthConfig *as, char *para)
 {
-char data[512]; /* NOTE: only 64 required by BF_crypt() */
-char *str;
+	char data[512]; /* NOTE: only 64 required by BF_crypt() */
+	char *str;
 
 	if (!para)
-		return -1;
+		return 0;
 
 	memset(data, 0, sizeof(data));
 	str = _crypt_blowfish_rn(para, as->data, data, sizeof(data));
 
 	if (!str)
-		return -1; /* ERROR / INVALID HASH */
+		return 0; /* ERROR / INVALID HASH */
 
 	if (!strcmp(str, as->data))
-		return 2; /* MATCH */
-	
-	return -1; /* NO MATCH */
+		return 1; /* MATCH */
+
+	return 0; /* NO MATCH */
 }
 
-static int authcheck_md5(aClient *cptr, anAuthStruct *as, char *para)
+static int authcheck_tls_clientcert(Client *client, AuthConfig *as, char *para)
 {
-static char buf[512];
-int	i, r;
-char *saltstr, *hashstr;
+	X509 *x509_clientcert = NULL;
+	X509 *x509_filecert = NULL;
+	FILE *x509_f = NULL;
 
-	if (!para)
-		return -1;
-	r = parsepass(as->data, &saltstr, &hashstr);
-	if (r == 0) /* Old method without salt: b64(MD5(<pass>)) */
+	if (!client->local->ssl)
+		return 0;
+	x509_clientcert = SSL_get_peer_certificate(client->local->ssl);
+	if (!x509_clientcert)
+		return 0;
+	if (!(x509_f = fopen(as->data, "r")))
 	{
-		char result[16];
-		
-		DoMD5(result, para, strlen(para));
-		if ((i = b64_encode(result, sizeof(result), buf, sizeof(buf))))
-		{
-			if (!strcmp(buf, as->data))
-				return 2;
-			else
-				return -1;
-		} else
-			return -1;
-	} else {
-		/* New method with salt: b64(MD5(MD5(<pass>)+salt)) */
-		char result1[MAXSALTLEN+16+1];
-		char result2[16];
-		char rsalt[MAXSALTLEN+1];
-		int rsaltlen;
-		
-		/* First, decode the salt to something real... */
-		rsaltlen = b64_decode(saltstr, rsalt, sizeof(rsalt));
-		if (rsaltlen <= 0)
-			return -1;
-		
-		/* Then hash the password (1st round)... */
-		DoMD5(result1, para, strlen(para));
-
-		/* Add salt to result */
-		memcpy(result1+16, rsalt, rsaltlen); /* b64_decode already made sure bounds are ok */
-
-		/* Then hash it all together again (2nd round)... */
-		DoMD5(result2, result1, rsaltlen+16);
-		
-		/* Then base64 encode it all and we are done... */
-		if ((i = b64_encode(result2, sizeof(result2), buf, sizeof(buf))))
-		{
-			if (!strcmp(buf, hashstr))
-				return 2;
-			else
-				return -1;
-		} else
-			return -1;
+		X509_free(x509_clientcert);
+		return 0;
 	}
-	return -1; /* NOTREACHED */
+	x509_filecert = PEM_read_X509(x509_f, NULL, NULL, NULL);
+	fclose(x509_f);
+	if (!x509_filecert)
+	{
+		X509_free(x509_clientcert);
+		return 0;
+	}
+	if (X509_cmp(x509_filecert, x509_clientcert) != 0)
+	{
+		X509_free(x509_clientcert);
+		X509_free(x509_filecert);
+		return 0;
+	}
+	X509_free(x509_clientcert);
+	X509_free(x509_filecert);
+	return 1;
 }
 
-static int authcheck_sha1(aClient *cptr, anAuthStruct *as, char *para)
+static int authcheck_tls_clientcert_fingerprint(Client *client, AuthConfig *as, char *para)
 {
-char buf[512];
-int i, r;
-char *saltstr, *hashstr;
+	int i, k;
+	char hexcolon[EVP_MAX_MD_SIZE * 3 + 1];
+	char *fp;
 
-	if (!para)
-		return -1;
-	r = parsepass(as->data, &saltstr, &hashstr);
-	if (r)
+	if (!client->local->ssl)
+		return 0;
+
+	fp = moddata_client_get(client, "certfp");
+	if (!fp)
+		return 0;
+
+	/* Make a colon version so that we keep in line with
+	 * previous versions, based on Nath's patch -dboyz
+	 */
+	k=0;
+	for (i=0; i<strlen(fp); i++)
 	{
-		/* New method with salt: b64(SHA1(SHA1(<pass>)+salt)) */
-		char result1[MAXSALTLEN+20+1];
-		char result2[20];
-		char rsalt[MAXSALTLEN+1];
-		int rsaltlen;
-		SHA_CTX hash;
-		
-		/* First, decode the salt to something real... */
-		rsaltlen = b64_decode(saltstr, rsalt, sizeof(rsalt));
-		if (rsaltlen <= 0)
-			return -1;
-
-		/* Then hash the password (1st round)... */
-		SHA1_Init(&hash);
-		SHA1_Update(&hash, para, strlen(para));
-		SHA1_Final(result1, &hash);
-
-		/* Add salt to result */
-		memcpy(result1+20, rsalt, rsaltlen); /* b64_decode already made sure bounds are ok */
-
-		/* Then hash it all together again (2nd round)... */
-		SHA1_Init(&hash);
-		SHA1_Update(&hash, result1, rsaltlen+20);
-		SHA1_Final(result2, &hash);
-
-		/* Then base64 encode it all and we are done... */
-		if ((i = b64_encode(result2, sizeof(result2), buf, sizeof(buf))))
-		{
-			if (!strcmp(buf, hashstr))
-				return 2;
-			else
-				return -1;
-		} else
-			return -1;
-	} else {
-		/* OLD auth */
-		if ((i = b64_encode(SHA1(para, strlen(para), NULL), 20, buf, sizeof(buf))))
-		{
-			if (!strcmp(buf, as->data))
-				return 2;
-			else
-				return -1;
-		} else
-			return -1;
+		if (i != 0 && i % 2 == 0)
+			hexcolon[k++] = ':';
+		hexcolon[k++] = fp[i];
 	}
+	hexcolon[k] = '\0';
+
+	if (strcasecmp(as->data, hexcolon) && strcasecmp(as->data, fp))
+		return 0;
+
+	return 1;
 }
 
-static int authcheck_ripemd160(aClient *cptr, anAuthStruct *as, char *para)
+static int authcheck_spkifp(Client *client, AuthConfig *as, char *para)
 {
-char buf[512];
-int i, r;
-char *saltstr, *hashstr;
+	char *fp = spki_fingerprint(client);
 
-	if (!para)
-		return -1;
-	r = parsepass(as->data, &saltstr, &hashstr);
-	if (r)
-	{
-		/* New method with salt: b64(RIPEMD160(RIPEMD160(<pass>)+salt)) */
-		char result1[MAXSALTLEN+20+1];
-		char result2[20];
-		char rsalt[MAXSALTLEN+1];
-		int rsaltlen;
-		RIPEMD160_CTX hash;
-		
-		/* First, decode the salt to something real... */
-		rsaltlen = b64_decode(saltstr, rsalt, sizeof(rsalt));
-		if (rsaltlen <= 0)
-			return -1;
+	if (!fp)
+		return 0; /* auth failed: not SSL (or other failure) */
 
-		/* Then hash the password (1st round)... */
-		RIPEMD160_Init(&hash);
-		RIPEMD160_Update(&hash, para, strlen(para));
-		RIPEMD160_Final(result1, &hash);
-		/* Add salt to result */
-		memcpy(result1+20, rsalt, rsaltlen); /* b64_decode already made sure bounds are ok */
+	if (strcasecmp(as->data, fp))
+		return 0; /* auth failed: mismatch */
 
-		/* Then hash it all together again (2nd round)... */
-		RIPEMD160_Init(&hash);
-		RIPEMD160_Update(&hash, result1, rsaltlen+20);
-		RIPEMD160_Final(result2, &hash);
-		/* Then base64 encode it all and we are done... */
-		if ((i = b64_encode(result2, sizeof(result2), buf, sizeof(buf))))
-		{
-			if (!strcmp(buf, hashstr))
-				return 2;
-			else
-				return -1;
-		} else
-			return -1;
-	} else {
-		/* OLD auth */
-		if ((i = b64_encode(RIPEMD160(para, strlen(para), NULL), 20, buf, sizeof(buf))))
-		{
-			if (!strcmp(buf, as->data))
-				return 2;
-			else
-				return -1;
-		} else
-			return -1;
-	}
+	return 1; /* SUCCESS */
 }
 
 
 /*
- * cptr MUST be a local client
+ * client MUST be a local client
  * as is what it will be compared with
  * para will used in coordination with the auth type	
 */
 
-/*
- * -1 if authentication failed
- *  1 if authentication succeeded
- *  2 if authentication succeeded, using parameter
- * -2 if authentication is delayed, don't error
- * No AuthStruct = everyone allowed
-*/
-int	Auth_Check(aClient *cptr, anAuthStruct *as, char *para)
+/** Check authentication, such as a password against the
+ * provided AuthConfig (which was parsed from the configuration
+ * file earlier).
+ * @param client    The client.
+ * @param as      The authentication config.
+ * @param para    The provided parameter (NULL allowed)
+ * @returns 1 if passed, 0 if incorrect (eg: invalid password)
+ * @note
+ * - The return value was different in versions before UnrealIRCd 5.0.0!
+ * - In older versions a NULL 'as' was treated as an allow, now it's deny.
+ */
+int Auth_Check(Client *client, AuthConfig *as, char *para)
 {
-	extern	char *crypt();
+	extern char *crypt();
 	char *res;
 
-	if (!as)
-		return 1;
-		
+	if (!as || !as->data)
+		return 0; /* Should not happen, but better be safe.. */
+
 	switch (as->type)
 	{
 		case AUTHTYPE_PLAINTEXT:
 			if (!para)
-				return -1;
+				return 0;
 			if (!strcmp(as->data, "changemeplease") && !strcmp(para, as->data))
 			{
 				sendto_realops("Rejecting default password 'changemeplease'. "
 				               "Please change the password in the configuration file.");
-				return -1;
+				return 0;
 			}
 			/* plain text compare */
 			if (!strcmp(para, as->data))
-				return 2;
-			return -1;
+				return 1;
+			return 0;
 
 		case AUTHTYPE_ARGON2:
-			return authcheck_argon2(cptr, as, para);
+			return authcheck_argon2(client, as, para);
 
 		case AUTHTYPE_BCRYPT:
-			return authcheck_bcrypt(cptr, as, para);
+			return authcheck_bcrypt(client, as, para);
 
 		case AUTHTYPE_UNIXCRYPT:
 			if (!para)
-				return -1;
-			/* If our data is like 1 or none, we just let em through .. */
-			if (!(as->data[0] && as->data[1]))
-				return 1;
+				return 0;
 			res = crypt(para, as->data);
 			if (res && !strcmp(res, as->data))
-				return 2;
-			return -1;
+				return 1;
+			return 0;
 
-		case AUTHTYPE_MD5:
-			return authcheck_md5(cptr, as, para);
+		case AUTHTYPE_TLS_CLIENTCERT:
+			return authcheck_tls_clientcert(client, as, para);
 
-		case AUTHTYPE_SHA1:
-			return authcheck_sha1(cptr, as, para);
-
-		case AUTHTYPE_RIPEMD160:
-			return authcheck_ripemd160(cptr, as, para);
-		
-		case AUTHTYPE_SSL_CLIENTCERT:
-		{
-			X509 *x509_clientcert = NULL;
-			X509 *x509_filecert = NULL;
-			FILE *x509_f = NULL;
-
-			if (!cptr->local->ssl)
-				return -1;
-			x509_clientcert = SSL_get_peer_certificate(cptr->local->ssl);
-			if (!x509_clientcert)
-				return -1;
-			if (!(x509_f = fopen(as->data, "r")))
-			{
-				X509_free(x509_clientcert);
-				return -1;
-			}
-			x509_filecert = PEM_read_X509(x509_f, NULL, NULL, NULL);
-			fclose(x509_f);
-			if (!x509_filecert)
-			{
-				X509_free(x509_clientcert);
-				return -1;
-			}
-			if (X509_cmp(x509_filecert, x509_clientcert) != 0)
-			{
-				X509_free(x509_clientcert);
-				X509_free(x509_filecert);
-				break;
-			}
-			X509_free(x509_clientcert);
-			X509_free(x509_filecert);
-			return 2;	
-		}
-
-		case AUTHTYPE_SSL_CLIENTCERTFP:
-		{
-			int i, k;
-			char hexcolon[EVP_MAX_MD_SIZE * 3 + 1];
-			char *fp;
-
-			if (!cptr->local->ssl)
-				return -1;
-			
-			fp = moddata_client_get(cptr, "certfp");
-			if (!fp)
-				return -1;
-				
-			/* Make a colon version so that we keep in line with
-			 * previous versions, based on Nath's patch -dboyz
-			 */
-			k=0;
-			for (i=0; i<strlen(fp); i++) {
-				if (i != 0 && i % 2 == 0)
-					hexcolon[k++] = ':';
-				hexcolon[k++] = fp[i];
- 			}
-			hexcolon[k] = '\0';
-
-			if (strcasecmp(as->data, hexcolon) && strcasecmp(as->data, fp))
-				return -1;
-
-			return 2;
-		}
+		case AUTHTYPE_TLS_CLIENTCERTFP:
+			return authcheck_tls_clientcert_fingerprint(client, as, para);
 
 		case AUTHTYPE_SPKIFP:
-		{
-			char *fp = spki_fingerprint(cptr);
+			return authcheck_spkifp(client, as, para);
 
-			if (!fp)
-				return -1; /* auth failed: not SSL (or other failure) */
-
-			if (strcasecmp(as->data, fp))
-				return -1; /* auth failed: mismatch */
-
-			return 2; /* SUCCESS */
-		}
+		case AUTHTYPE_INVALID:
+			return 0; /* Should never happen */
 	}
-	return -1;
+	return 0;
 }
 
 #define UNREALIRCD_ARGON2_DEFAULT_TIME_COST             3
@@ -718,11 +528,11 @@ static char *mkpass_bcrypt(char *para)
 
 	for (i=0; i<sizeof(random_data); i++)
 		random_data[i] = getrandom8();
-	
+
 	saltstr = _crypt_gensalt_blowfish_rn("$2y", 9, random_data, sizeof(random_data), salt, sizeof(salt));
 	if (!saltstr)
 		return NULL;
-	
+
 	str = _crypt_blowfish_rn(para, saltstr, data, sizeof(data));
 
 	if (!str)
@@ -732,18 +542,23 @@ static char *mkpass_bcrypt(char *para)
 	return buf;
 }
 
-char	*Auth_Make(short type, char *para)
+/** Create a hashed password for the specified string.
+ * @param type  One of AUTHTYPE_*, eg AUTHTYPE_ARGON2.
+ * @param text  The password in plaintext.
+ * @returns The hashed password.
+ */
+char *Auth_Hash(AuthenticationType type, char *text)
 {
 	switch (type)
 	{
 		case AUTHTYPE_PLAINTEXT:
-			return para;
+			return text;
 
 		case AUTHTYPE_ARGON2:
-			return mkpass_argon2(para);
+			return mkpass_argon2(text);
 
 		case AUTHTYPE_BCRYPT:
-			return mkpass_bcrypt(para);
+			return mkpass_bcrypt(text);
 
 		default:
 			return NULL;
