@@ -1,34 +1,20 @@
 /* UnrealIRCd crash reporter code.
- * (C) Copyright 2015 Bram Matthys ("Syzop") and the UnrealIRCd Team.
- *
- * GPLv2
+ * (C) Copyright 2015-2019 Bram Matthys ("Syzop") and the UnrealIRCd Team.
+ * License: GPLv2
  */
+
 #include "unrealircd.h"
-#ifndef _WIN32
-#include <dirent.h>
-#else
+#ifdef _WIN32
 extern void StartUnrealAgain(void);
 #endif
-
 #include "version.h"
 
 extern char *getosname(void);
 
-
-time_t get_file_time(char *fname)
-{
-	struct stat st;
-	
-	if (stat(fname, &st) != 0)
-		return 0;
-
-	return (time_t)st.st_ctime;
-}
-
 char *find_best_coredump(void)
 {
 	static char best_fname[512];
-	TS best_time = 0, t;
+	time_t best_time = 0, t;
 	struct dirent *dir;
 #ifndef _WIN32
 	DIR *fd = opendir(TMPDIR);
@@ -66,7 +52,7 @@ char *find_best_coredump(void)
 	if (hFile == INVALID_HANDLE_VALUE)
 		return NULL;
 	
-	while (FindNextFile(hFile, &hData))
+	do
 	{
 		char *fname = hData.cFileName;
 		if (!strstr(fname, ".done"))
@@ -80,7 +66,7 @@ char *find_best_coredump(void)
 				strlcpy(best_fname, buf, sizeof(best_fname));
 			}
 		}
-	}
+	} while (FindNextFile(hFile, &hData));
 	FindClose(hFile);
 #endif	
 	
@@ -90,16 +76,43 @@ char *find_best_coredump(void)
 	return NULL; /* none found */
 }
 
-void stripcrlf(char *p)
+/** Find the latest AddressSanitizer log file */
+char *find_best_asan_log(void)
 {
-	for (; *p; p++)
+#ifndef _WIN32
+	static char best_fname[512];
+	time_t best_time = 0, t;
+	struct dirent *dir;
+	DIR *fd = opendir(TMPDIR);
+
+	if (!fd)
+		return NULL;
+
+	*best_fname = '\0';
+
+	while ((dir = readdir(fd)))
 	{
-		if ((*p == '\r') || (*p == '\n'))
+		char *fname = dir->d_name;
+		if (strstr(fname, "unrealircd_asan.") && !strstr(fname, ".so") &&
+		    !strstr(fname, ".conf") && !strstr(fname, ".txt") &&
+		    !strstr(fname, ".done"))
 		{
-			*p = '\0';
-			return;
+			char buf[512];
+
+			snprintf(buf, sizeof(buf), "%s/%s", TMPDIR, fname);
+			t = get_file_time(buf);
+			if (t && (t > best_time))
+			{
+				best_time = t;
+				strlcpy(best_fname, buf, sizeof(best_fname));
+			}
 		}
 	}
+	closedir(fd);
+	return BadPtr(best_fname) ? NULL : best_fname;
+#else
+	return NULL;
+#endif
 }
 
 #define EL_AR_MAX MAXPARA
@@ -235,10 +248,6 @@ int crash_report_backtrace(FILE *reportfd, char *coredump)
 	fprintf(reportfd, "START OF BACKTRACE\n");
 	while((fgets(buf, sizeof(buf), fd)))
 	{
-		char *file, *path;
-		char target[512];
-		char **arr;
-
 		stripcrlf(buf);
 		fprintf(reportfd, " %s\n", buf);
 	}
@@ -266,9 +275,51 @@ int crash_report_backtrace(FILE *reportfd, char *coredump)
 #endif
 }
 
+int crash_report_asan_log(FILE *reportfd, char *coredump)
+{
+#ifndef _WIN32
+	time_t coretime, asantime;
+	FILE *fd;
+	char buf[1024];
+	char *asan_log = find_best_asan_log();
+	int n;
+
+	if (!asan_log)
+		return 0;
+
+	coretime = get_file_time(coredump);
+	asantime = get_file_time(asan_log);
+
+	fprintf(reportfd, "ASan log file found '%s' which is %ld seconds older than core file\n",
+		asan_log,
+		(long)((long)(coretime) - (long)asantime));
+
+	fd = fopen(asan_log, "r");
+	if (!fd)
+	{
+		fprintf(reportfd, "Could not open ASan log (%s)\n", strerror(errno));
+		return 0;
+	}
+	fprintf(reportfd, "START OF ASAN LOG\n");
+	while((fgets(buf, sizeof(buf), fd)))
+	{
+		stripcrlf(buf);
+		fprintf(reportfd, " %s\n", buf);
+	}
+	n = pclose(fd);
+	fprintf(reportfd, "END OF ASAN LOG\n");
+
+	if (WEXITSTATUS(n) == 127)
+		return 0;
+
+	return 1;
+#else
+	return 0;
+#endif
+}
+
 void crash_report_header(FILE *reportfd, char *coredump)
 {
-	char buf[512];
 	time_t t;
 	
 	fprintf(reportfd, "== UNREALIRCD CRASH REPORT ==\n"
@@ -339,7 +390,7 @@ int attach_file(FILE *fdi, FILE *fdo)
 
 		total += strlen(printbuf);
 
-		if (total > 9500000)
+		if (total > 15000000)
 			return 0; /* Safety limit */
 	}
 
@@ -347,16 +398,71 @@ int attach_file(FILE *fdi, FILE *fdo)
 	return 1;
 }
 
+/** Figure out the libc library name (.so file), copy it to tmp/
+ * to include it in the bug report. This can improve the backtrace
+ * a lot (read: make it actually readable / useful) in case we
+ * crash in a libc function.
+ */
+char *copy_libc_so(void)
+{
+#ifdef _WIN32
+	return "";
+#else
+	FILE *fd;
+	char buf[1024];
+	static char ret[512];
+	char *basename = NULL, *libcname = NULL, *p, *start;
+
+	snprintf(buf, sizeof(buf), "ldd %s/unrealircd 2>/dev/null", BINDIR);
+	fd = popen(buf, "r");
+	if (!fd)
+		return "";
+
+	while ((fgets(buf, sizeof(buf), fd)))
+	{
+		stripcrlf(buf);
+		p = strstr(buf, "libc.so");
+		if (!p)
+			continue;
+		basename = p;
+		p = strchr(p, ' ');
+		if (!p)
+			continue;
+		*p++ = '\0';
+		p = strstr(p, "=> ");
+		if (!p)
+			continue;
+		start = p += 3; /* skip "=> " */
+		p = strchr(start, ' ');
+		if (!p)
+			continue;
+		*p = '\0';
+		libcname = start;
+		break;
+	}
+	pclose(fd);
+
+	if (!basename || !libcname)
+		return ""; /* not found, weird */
+
+	snprintf(ret, sizeof(ret), "%s/%s", TMPDIR, basename);
+	if (!unreal_copyfile(libcname, ret))
+		return ""; /* copying failed */
+
+	return ret;
+#endif
+}
 int attach_coredump(FILE *fdo, char *coredump)
 {
 	FILE *fdi;
 	char fname[512];
+	char *libcname = copy_libc_so();
 
 #ifndef _WIN32
 	/* On *NIX we create a .tar.bz2 / .tar.gz (may take a couple of seconds) */
 	printf("Please wait...\n");
-	snprintf(fname, sizeof(fname), "tar c %s/unrealircd %s %s 2>/dev/null|(bzip2 || gzip) 2>/dev/null",
-		BINDIR, coredump, MODULESDIR);
+	snprintf(fname, sizeof(fname), "tar c %s/unrealircd %s %s %s 2>/dev/null|(bzip2 || gzip) 2>/dev/null",
+		BINDIR, coredump, MODULESDIR, libcname);
 
 	fdi = popen(fname, "r");
 #else
@@ -411,7 +517,7 @@ char *generate_crash_report(char *coredump, int *thirdpartymods)
 	crash_report_fix_libs(coredump, thirdpartymods);
 	
 	crash_report_backtrace(reportfd, coredump);
-
+	crash_report_asan_log(reportfd, coredump);
 	attach_coredump(reportfd, coredump);
 
 	fclose(reportfd);
@@ -419,46 +525,15 @@ char *generate_crash_report(char *coredump, int *thirdpartymods)
 	return reportfname;
 }
 
-int running_interactive(void)
-{
-#ifndef _WIN32
-	char *s;
-	
-	if (!isatty(0))
-		return 0;
-	
-	s = getenv("TERM");
-	if (!s || !strcasecmp(s, "dumb") || !strcasecmp(s, "none"))
-		return 0;
-
-	return 1;
-#else
-	return IsService ? 0 : 1;
-#endif
-}
-
 #define REPORT_NEVER	-1
 #define REPORT_ASK		0
 #define REPORT_AUTO		1
 
-
-int getfilesize(char *fname)
-{
-	struct stat st;
-	int size;
-	
-	if (stat(fname, &st) != 0)
-		return -1;
-	
-	return (int)st.st_size;
-}
-
 #define CRASH_REPORT_HOST "crash.unrealircd.org"
 
 SSL_CTX *crashreport_init_ssl(void)
 {
 	SSL_CTX *ctx_client;
-	X509_VERIFY_PARAM *param = NULL;
 	char buf[512];
 	
 	SSL_load_error_strings();
@@ -467,10 +542,13 @@ SSL_CTX *crashreport_init_ssl(void)
 	ctx_client = SSL_CTX_new(SSLv23_client_method());
 	if (!ctx_client)
 		return NULL;
-	SSL_CTX_set_options(ctx_client, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3);
+#ifdef HAS_SSL_CTX_SET_MIN_PROTO_VERSION
+	SSL_CTX_set_min_proto_version(ctx_client, TLS1_2_VERSION);
+#endif
+	SSL_CTX_set_options(ctx_client, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1|SSL_OP_NO_TLSv1_1);
 
 	/* Verify peer certificate */
-	snprintf(buf, sizeof(buf), "%s/ssl/curl-ca-bundle.crt", CONFDIR);
+	snprintf(buf, sizeof(buf), "%s/tls/curl-ca-bundle.crt", CONFDIR);
 	SSL_CTX_load_verify_locations(ctx_client, buf, NULL);
 	SSL_CTX_set_verify(ctx_client, SSL_VERIFY_PEER, NULL);
 
@@ -494,7 +572,7 @@ int crashreport_send(char *fname)
 	int xfr = 0;
 	char *errstr = NULL;
 	
-	filesize = getfilesize(fname);
+	filesize = get_file_size(fname);
 	if (filesize < 0)
 		return 0;
 	
@@ -512,14 +590,14 @@ int crashreport_send(char *fname)
 	ctx_client = crashreport_init_ssl();
 	if (!ctx_client)
 	{
-		printf("ERROR: SSL initalization failure (I)\n");
+		printf("ERROR: TLS initalization failure (I)\n");
 		return 0;
 	}
 	
 	socket = BIO_new_ssl_connect(ctx_client);
 	if (!socket)
 	{
-		printf("ERROR: SSL initalization failure (II)\n");
+		printf("ERROR: TLS initalization failure (II)\n");
 		return 0;
 	}
 	
@@ -533,14 +611,14 @@ int crashreport_send(char *fname)
 	
 	if (BIO_do_handshake(socket) != 1)
 	{
-		printf("ERROR: Could not connect to %s (SSL handshake failed)\n", CRASH_REPORT_HOST);
+		printf("ERROR: Could not connect to %s (TLS handshake failed)\n", CRASH_REPORT_HOST);
 		return 0;
 	}
 
 	BIO_get_ssl(socket, &ssl);
 	if (!ssl)
 	{
-		printf("ERROR: Could not get SSL connection from BIO\n");
+		printf("ERROR: Could not get TLS connection from BIO\n");
 		return 0;
 	}
 
@@ -571,6 +649,13 @@ int crashreport_send(char *fname)
 	if ((n < 0) || strncmp(buf, "HTTP/1.1 100", 12))
 	{
 		printf("Error transmitting bug report (stage II, n=%d)\n", n);
+		if (!strncmp(buf, "HTTP/1.1 403", 12))
+		{
+			printf("Your crash report was rejected automatically.\n"
+			       "This normally means your UnrealIRCd version is too old and unsupported.\n"
+			       "Chances are that your crash issue is already fixed in a later release.\n"
+			       "Check https://www.unrealircd.org/ for latest releases!\n");
+		}
 		return 0;
 	}
 	
@@ -598,7 +683,7 @@ int crashreport_send(char *fname)
 
 	BIO_puts(socket, footer);
 
-	do { } while(BIO_should_retry(socket)); /* make sure we are really finished (you never know with SSL) */
+	do { } while(BIO_should_retry(socket)); /* make sure we are really finished (you never know with TLS) */
 
 #ifndef _WIN32
 	printf("\n");
@@ -621,13 +706,50 @@ void mark_coredump_as_read(char *coredump)
 
 static int report_pref = REPORT_ASK;
 
+void report_crash_not_sent(char *fname)
+{
+		printf("Crash report will not be sent to UnrealIRCd Team.\n"
+		       "\n"
+		       "Feel free to read the report at %s and delete it.\n"
+		       "Or, if you change your mind, you can submit it anyway at https://bugs.unrealircd.org/\n"
+		       " (if you do, please set the option 'View Status' at the end of the bug report page to 'private'!!)\n", fname);
+}
+
+/** This checks if there are indications that 3rd party modules are
+ * loaded. This is used to provide a small warning to the user that
+ * the crash may be likely due to that.
+ */
+int check_third_party_mods_present(void)
+{
+#ifndef _WIN32
+	struct dirent *dir;
+	DIR *fd = opendir(TMPDIR);
+
+	if (!fd)
+		return 0;
+
+	/* We search for files like tmp/FC5C3116.third.somename.so */
+	while ((dir = readdir(fd)))
+	{
+		char *fname = dir->d_name;
+		if (strstr(fname, ".third.") && strstr(fname, ".so"))
+		{
+			closedir(fd);
+			return 1;
+		}
+	}
+	closedir(fd);
+#endif
+	return 0;
+}
+
 void report_crash(void)
 {
 	char *coredump, *fname;
 	int thirdpartymods = 0;
 	int crashed_secs_ago;
 
-	if (!running_interactive() && (report_pref != REPORT_AUTO))
+	if (!running_interactively() && (report_pref != REPORT_AUTO))
 		exit(0); /* don't bother if we run through cron or something similar */
 
 	coredump = find_best_coredump();
@@ -643,6 +765,24 @@ void report_crash(void)
 	if (!fname)
 		return;
 
+	if (time(NULL) > 1688169600)
+	{
+		printf("Crash report generated in '%s' but NOT sent.\n\n"
+		       "UnrealIRCd 5 is no longer supported since July 1, 2023.\n"
+		       "All support stopped after that date. You had 18+ months to upgrade.\n"
+		       "See https://www.unrealircd.org/docs/UnrealIRCd_5_EOL.\n",
+		       fname);
+		return;
+	} else
+	if (time(NULL) > 1656633600)
+	{
+		printf("WARNING: UnrealIRCd 5 is only receiving security fixes until July 1, 2023.\n"
+		       "         See https://www.unrealircd.org/docs/UnrealIRCd_5_EOL\n");
+		/* continue below.. */
+	}
+
+	if (thirdpartymods == 0)
+		thirdpartymods = check_third_party_mods_present();
 #ifndef _WIN32
 	printf("The IRCd has been started now (and is running), but it did crash %d seconds ago.\n", crashed_secs_ago);
 	printf("Crash report generated in: %s\n\n", fname);
@@ -654,24 +794,26 @@ void report_crash(void)
                "by someone other than the UnrealIRCd team). If you installed new 3rd party\n"
                "module(s) in the past few weeks we suggest to unload these modules and see if\n"
                "the crash issue dissapears. If so, that module is probably to blame.\n"
-               "If you keep crashing without 3rd party modules then please do report it to\n"
-               "the UnrealIRCd team.\n"
-               "The reason we ask you to do this is because more than 95%% of the crash issues\n"
-               "reported nowadays are caused by 3rd party modules and not by an UnrealIRCd bug.\n"
+               "If you keep crashing without any 3rd party modules loaded then please do report\n"
+               "it to the UnrealIRCd team.\n"
+               "The reason we ask you to do this is because MORE THAN 95%% OF ALL CRASH ISSUES\n"
+               "ARE CAUSED BY 3RD PARTY MODULES and not by an UnrealIRCd bug.\n"
                "\n");
 	}
 		
 	if (report_pref == REPORT_NEVER)
 	{
-		printf("Crash report will not be sent to UnrealIRCd Team.\n\n");
-		printf("Feel free to read the report at %s and if you change your mind you can submit it anyway at https://bugs.unrealircd.org/\n", fname);
+		report_crash_not_sent(fname);
+		return;
 	} else
 	if (report_pref == REPORT_ASK)
 	{
 		char answerbuf[64], *answer;
 		printf("Shall I send a crash report to the UnrealIRCd developers?\n");
 		if (!thirdpartymods)
-		    printf("Crash reports help us greatly with fixing bugs that affect you and others\n");
+			printf("Crash reports help us greatly with fixing bugs that affect you and others\n");
+		else
+			printf("NOTE: If the crash is caused by a 3rd party module then UnrealIRCd devs can't fix that.\n");
 		printf("\n");
 		
 		do
@@ -682,8 +824,7 @@ void report_crash(void)
 			
 			if (answer && (toupper(*answer) == 'N'))
 			{
-				printf("Ok, not sending bug report.\n\n");
-				printf("Feel free to read the report at %s and if you change your mind you can submit it anyway at https://bugs.unrealircd.org/\n", fname);
+				report_crash_not_sent(fname);
 				return;
 			}
 			if (answer && (toupper(*answer) == 'Y'))
@@ -699,7 +840,7 @@ void report_crash(void)
 		return;
 	}
 
-	if (running_interactive())
+	if (running_interactively())
 	{
 		char buf[8192], *line;
 
