@@ -31,37 +31,29 @@ ID_Copyright("(C) 1990 Jarkko Oikarinen");
  *		1, if no match
  */
 
-#ifndef USE_LOCALE
 u_char touppertab[], tolowertab[];
 #define tolowertab2 tolowertab
-#endif
-
-#ifndef USE_LOCALE
-#define lc(x) tolowertab2[x]	/* use mylowertab, because registers are FASTER */
-#else				/* maybe in the old 4mb hard drive days but not anymore -- codemastr */
-#define lc(x) tolower(x)
-#endif
-
+#define lc(x) tolowertab2[x]
 
 /* Match routine for special cases where escaping is needed in a normal fashion.
  * Checks a string ('name') against a globbing(+more) pattern ('mask').
  * Original by Douglas A Lewis (dalewis@acsu.buffalo.edu).
  * Code based on hybrid7's version (match_esc()).
  * Various modifications by Bram Matthys (Syzop).
+ * Returns 1 on match and 0 for no match.
  * Instead of our previous code, this one is less optimized but actually  _readable_ ;).
  * Modifications I (Syzop) had to do vs the hybrid7 code:
  * - Got rid of (u_char *) casts, since we already compile with
  *   chars defaulting to unsigned [or else major things break] ;).
- * - Use 0 for match and non-zero for no match (a la strcmp), not the reverse.
  * - Support for '_'.
  * - Rip out support for '#'.
  */
 int match_esc(const char *mask, const char *name)
 {
-const u_char *m = mask;
-const u_char *n = name;
-const u_char *ma = NULL;
-const u_char *na = name;
+	const u_char *m = mask;
+	const u_char *n = name;
+	const u_char *ma = NULL;
+	const u_char *na = name;
 
 	while(1)
 	{
@@ -76,12 +68,12 @@ const u_char *na = name;
 		if (!*m)
 		{
 			if (!*n)
-				return 0;
-			if (!ma)
 				return 1;
+			if (!ma)
+				return 0;
 			for (m--; (m > (const u_char *)mask) && (*m == '?'); m--);
 			if (*m == '*')
-				return 0;
+				return 1;
 			m = ma;
 			n = ++na;
 		} else
@@ -89,18 +81,18 @@ const u_char *na = name;
 		{
 			while (*m == '*') /* collapse.. */
 				m++;
-			return (*m != 0);
+			return (*m == 0);
 		}
 		
 		if (*m != '?')
 		{
 			if (*m == '\\')
 				if (!*++m)
-					return 1; /* unfinished escape sequence */
+					return 0; /* unfinished escape sequence */
 			if ((lc(*m) != lc(*n)) && !((*m == '_') && (*n == ' ')))
 			{
 				if (!ma)
-					return 1;
+					return 0;
 				m = ma;
 				n = ++na;
 			} else
@@ -114,16 +106,16 @@ const u_char *na = name;
 			n++;
 		}
 	}
-	return 1;
+	return 0;
 }
 
 /** Same credit/copyright as match_esc() applies, except escaping removed.. ;p */
-static inline int match2(const char *mask, const char *name)
+int match_simple(const char *mask, const char *name)
 {
-const u_char *m = mask;
-const u_char *n = name;
-const u_char *ma = NULL;
-const u_char *na = name;
+	const u_char *m = mask;
+	const u_char *n = name;
+	const u_char *ma = NULL;
+	const u_char *na = name;
 
 	while(1)
 	{
@@ -138,12 +130,12 @@ const u_char *na = name;
 		if (!*m)
 		{
 			if (!*n)
-				return 0;
-			if (!ma)
 				return 1;
+			if (!ma)
+				return 0;
 			for (m--; (m > (const u_char *)mask) && (*m == '?'); m--);
 			if (*m == '*')
-				return 0;
+				return 1;
 			m = ma;
 			n = ++na;
 		} else
@@ -151,13 +143,13 @@ const u_char *na = name;
 		{
 			while (*m == '*') /* collapse.. */
 				m++;
-			return (*m != 0);
+			return (*m == 0);
 		}
 		
 		if ((lc(*m) != lc(*n)) && !((*m == '_') && (*n == ' ')) && (*m != '?'))
 		{
 			if (!ma)
-				return 1;
+				return 0;
 			m = ma;
 			n = ++na;
 		} else
@@ -166,7 +158,7 @@ const u_char *na = name;
 			n++;
 		}
 	}
-	return 1;
+	return 0;
 }
 
 /*
@@ -211,10 +203,12 @@ char *collapse(char *pattern)
 }
 
 
-/*
- *  Case insensitive comparison of two NULL terminated strings.
- *
- *	returns	 0, if s1 equal to s2
+/* Case insensitive comparison of two NULL terminated strings,
+ * using the "IRC nick comparison" rules. Or, well, partially
+ * anyway.
+ * Should be used for NICK-related comparisons. And probably
+ * not even then, since this does not deal with multibyte.
+ * @returns 	 0, if s1 equal to s2
  *		<0, if s1 lexicographically less than s2
  *		>0, if s1 lexicographically greater than s2
  */
@@ -237,28 +231,6 @@ int  smycmp(const char *s1, const char *s2)
 	return (res);
 }
 
-
-int  myncmp(const char *str1, const char *str2, int n)
-{
-	u_char *s1;
-	u_char *s2;
-	int  res;
-
-	s1 = (u_char *)str1;
-	s2 = (u_char *)str2;
-
-	while ((res = toupper(*s1) - toupper(*s2)) == 0)
-	{
-		s1++;
-		s2++;
-		n--;
-		if (n == 0 || (*s1 == '\0' && *s2 == '\0'))
-			return 0;
-	}
-	return (res);
-}
-
-#ifndef USE_LOCALE
 u_char tolowertab[] = {
 	0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
 	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
@@ -329,7 +301,6 @@ u_char touppertab[] = {
 	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };
 
-#endif
 u_char char_atribs[] = {
 /* 0-7 */ CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL,
 /* 8-12 */ CNTRL, CNTRL | SPACE, CNTRL | SPACE, CNTRL | SPACE,
@@ -398,67 +369,29 @@ u_char char_atribs[] = {
 /* f0-ff */ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
-/* Old match() */
-int _match(const char *mask, const char *name) {
-	return match2(mask,name);
-}
-
-
-/* Old match() plus some optimizations from bahamut */
-int match(const char *mask, const char *name) {
-	if (mask[0] == '*' && mask[1] == '!') {
-		mask += 2;
-		while (*name != '!' && *name)
-			name++;
-		if (!*name)
-			return 1;
-		name++;
-	}
-		
-	if (mask[0] == '*' && mask[1] == '@') {
-		mask += 2;
-		while (*name != '@' && *name)
-			name++;
-		if (!*name)
-			return 1;
-		name++;
-	}
-	return match2(mask,name);
-}
-
-/** Free up all resources of an aMatch entry (including the struct itself).
- * NOTE: this function may (also) be called for aMatch structs that have only been
+/** Free up all resources of an Match entry (including the struct itself).
+ * NOTE: this function may (also) be called for Match structs that have only been
  *       setup half-way, so use special care when accessing members (NULL checks!)
  */
-void unreal_delete_match(aMatch *m)
+void unreal_delete_match(Match *m)
 {
-	safefree(m->str);
-#ifdef USE_TRE
-	if (m->type == MATCH_TRE_REGEX)
-	{
-		if (m->ext.tre_expr)
-		{
-			regfree(m->ext.tre_expr);
-			MyFree(m->ext.tre_expr);
-		}
-	}
-#endif
+	safe_free(m->str);
 	if (m->type == MATCH_PCRE_REGEX)
 	{
 		if (m->ext.pcre2_expr)
 			pcre2_code_free(m->ext.pcre2_expr);
 	}
-	MyFree(m);
+	safe_free(m);
 }
 
-aMatch *unreal_create_match(MatchType type, char *str, char **error)
+Match *unreal_create_match(MatchType type, char *str, char **error)
 {
-	aMatch *m = MyMallocEx(sizeof(aMatch));
+	Match *m = safe_alloc(sizeof(Match));
 	static char errorbuf[512];
 
 	*errorbuf = '\0';
 
-	m->str = strdup(str);
+	safe_strdup(m->str, str);
 	m->type = type;
 	
 	if (m->type == MATCH_SIMPLE)
@@ -477,6 +410,7 @@ aMatch *unreal_create_match(MatchType type, char *str, char **error)
 		m->ext.pcre2_expr = pcre2_compile(str, PCRE2_ZERO_TERMINATED, options, &errorcode, &erroroffset, NULL);
 		if (m->ext.pcre2_expr == NULL)
 		{
+			*buf2 = '\0';
 			pcre2_get_error_message(errorcode, buf2, sizeof(buf2));
 			if (error)
 			{
@@ -492,27 +426,6 @@ aMatch *unreal_create_match(MatchType type, char *str, char **error)
 		pcre2_jit_compile(m->ext.pcre2_expr, PCRE2_JIT_COMPLETE);
 		return m;
 	}
-#ifdef USE_TRE
-	else if (m->type == MATCH_TRE_REGEX)
-	{
-		int errorcode;
-		
-		m->ext.tre_expr = MyMallocEx(sizeof(regex_t));
-		errorcode = regcomp(m->ext.tre_expr, str, REG_ICASE|REG_EXTENDED|REG_NOSUB);
-		if (errorcode > 0)
-		{
-			int errorbufsize = 512;
-			char *errtmp = MyMallocEx(errorbufsize);
-			regerror(errorcode, m->ext.tre_expr, errtmp, errorbufsize);
-			strlcpy(errorbuf, errtmp, sizeof(errorbuf));
-			MyFree(errtmp);
-			if (error)
-				*error = errorbuf;
-			unreal_delete_match(m);
-			return NULL;
-		}
-	}
-#endif
 	else {
 		/* Unknown type, how did that happen ? */
 		unreal_delete_match(m);
@@ -521,15 +434,15 @@ aMatch *unreal_create_match(MatchType type, char *str, char **error)
 	return m;
 }
 
-/** Try to match an aMatch entry ('m') against a string ('str').
+/** Try to match an Match entry ('m') against a string ('str').
  * @returns 1 if matched, 0 if not.
- * @notes These (more logical) return values are opposite to the match() function.
+ * @note These (more logical) return values are opposite to the match_simple() function.
  */
-int unreal_match(aMatch *m, char *str)
+int unreal_match(Match *m, char *str)
 {
 	if (m->type == MATCH_SIMPLE)
 	{
-		if (_match(m->str, str) == 0)
+		if (match_simple(m->str, str))
 			return 1;
 		return 0;
 	}
@@ -547,15 +460,6 @@ int unreal_match(aMatch *m, char *str)
 		return 0; /* NO MATCH */
 	}
 
-#ifdef USE_TRE
-	if (m->type == MATCH_TRE_REGEX)
-	{
-		if (regexec(m->ext.tre_expr, str, 0, NULL, 0) == 0)
-			return 1;
-		return 0;
-	}
-#endif
-
 	return 0;
 }
 
@@ -563,8 +467,6 @@ int unreal_match_method_strtoval(char *str)
 {
 	if (!strcmp(str, "regex") || !strcmp(str, "pcre"))
 		return MATCH_PCRE_REGEX;
-	if (!strcmp(str, "posix") || !strcmp(str, "tre"))
-		return MATCH_TRE_REGEX;
 	if (!strcmp(str, "simple") || !strcmp(str, "glob"))
 		return MATCH_SIMPLE;
 	return 0;
@@ -574,8 +476,6 @@ char *unreal_match_method_valtostr(int val)
 {
 	if (val == MATCH_PCRE_REGEX)
 		return "regex";
-	if (val == MATCH_TRE_REGEX)
-		return "posix";
 	if (val == MATCH_SIMPLE)
 		return "simple";
 	
@@ -845,8 +745,8 @@ char *stripbadwords(char *str, ConfigItem_badword *start_bw, int *blocked)
  */
 char *badword_config_check_regex(char *str, int fastsupport, int check_broadness)
 {
-	int errorcode, errorbufsize, regex=0;
-	char *errtmp, *tmp;
+	int regex=0;
+	char *tmp;
 	static char errorbuf[512];
 
 	if (fastsupport)
@@ -891,7 +791,6 @@ int badword_config_process(ConfigItem_badword *ca, char *str)
 {
 	char *tmp;
 	short regex = 0;
-	int regflags = 0;
 	int ast_l = 0, ast_r = 0;
 
 	/* The fast badwords routine can do: "blah" "*blah" "blah*" and "*blah*",
@@ -916,10 +815,9 @@ int badword_config_process(ConfigItem_badword *ca, char *str)
 		int errorcode = 0;
 		PCRE2_SIZE erroroffset = 0;
 		int options = 0;
-		char buf2[512];
 
 		ca->type = BADW_TYPE_REGEX;
-		safestrdup(ca->word, str);
+		safe_strdup(ca->word, str);
 
 		options = PCRE2_CASELESS|PCRE2_NEVER_UTF|PCRE2_NEVER_UCP;
 
@@ -938,7 +836,7 @@ int badword_config_process(ConfigItem_badword *ca, char *str)
 	{
 		char *tmpw;
 		ca->type = BADW_TYPE_FAST;
-		ca->word = tmpw = MyMallocEx(strlen(str) - ast_l - ast_r + 1);
+		ca->word = tmpw = safe_alloc(strlen(str) - ast_l - ast_r + 1);
 		/* Copy except for asterisks */
 		for (tmp = str; *tmp; tmp++)
 			if (*tmp != '*')
@@ -959,10 +857,9 @@ int badword_config_process(ConfigItem_badword *ca, char *str)
  */
 void badword_config_free(ConfigItem_badword *e)
 {
-	safefree(e->word);
-	if (e->replace)
-		safefree(e->replace);
+	safe_free(e->word);
+	safe_free(e->replace);
 	if (e->pcre2_expr)
 		pcre2_code_free(e->pcre2_expr);
-	MyFree(e);
+	safe_free(e);
 }
