@@ -42,18 +42,18 @@
 
 #include "unrealircd.h"
 
-ModuleHeader MOD_HEADER(antimixedutf8)
+ModuleHeader MOD_HEADER
 = {
 	"antimixedutf8",
 	"1.0",
 	"Mixed UTF8 character filter (look-alike character spam) - by Syzop",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
 };
 
 struct {
 	int score;
-	int ban_action;
+	BanAction ban_action;
 	char *ban_reason;
 	long ban_time;
 } cfg;
@@ -66,6 +66,10 @@ int antimixedutf8_config_run(ConfigFile *, ConfigEntry *, int);
 #define SCRIPT_UNDEFINED	0
 #define SCRIPT_LATIN		1
 #define SCRIPT_CYRILLIC		2
+#define SCRIPT_CJK		3
+#define SCRIPT_HANGUL		4
+#define SCRIPT_CANADIAN		5
+#define SCRIPT_TELUGU		6
 
 /**** the detection algorithm follows first, the module/config code is at the end ****/
 
@@ -95,6 +99,24 @@ int detect_script(const char *t)
 	else if ((t[0] == 0xd3) && (t[1] >= 0x80) && (t[1] <= 0xbf))
 		return SCRIPT_CYRILLIC;
 
+	if((t[0] == 0xe4) && (t[1] >= 0xb8) && (t[1] <= 0xbf))
+		return SCRIPT_CJK;
+	else if ((t[0] >= 0xe5) && (t[0] <= 0xe9) && (t[1] >= 0x80) && (t[1] <= 0xbf))
+		return SCRIPT_CJK;
+
+	if((t[0] == 0xea) && (t[1] >= 0xb0) && (t[1] <= 0xbf))
+		return SCRIPT_HANGUL;
+	else if ((t[0] >= 0xeb) && (t[0] <= 0xec) && (t[1] >= 0x80) && (t[1] <= 0xbf))
+		return SCRIPT_HANGUL;
+	else if ((t[0] == 0xed) && (t[1] >= 0x80) && (t[1] <= 0x9f))
+		return SCRIPT_HANGUL;
+
+	if((t[0] == 0xe1) && (t[1] >= 0x90) && (t[1] <= 0x99))
+		return SCRIPT_CANADIAN;
+
+	if((t[0] == 0xe0) && (t[1] >= 0xb0) && (t[1] <= 0xb1))
+		return SCRIPT_TELUGU;
+
 	if ((t[0] >= 'a') && (t[0] <= 'z'))
 		return SCRIPT_LATIN;
 	if ((t[0] >= 'A') && (t[0] <= 'Z'))
@@ -174,49 +196,46 @@ CMD_OVERRIDE_FUNC(override_msg)
 {
 	int score, ret;
 	
-	if (!MyClient(sptr) || (parc < 3) || BadPtr(parv[2]))
+	if (!MyUser(client) || (parc < 3) || BadPtr(parv[2]))
 	{
 		/* Short circuit for: remote clients or insufficient parameters */
-		return CallCmdoverride(ovr, cptr, sptr, parc, parv);
+		CallCommandOverride(ovr, client, recv_mtags, parc, parv);
+		return;
 	}
 
 	score = lookalikespam_score(StripControlCodes(parv[2]));
-	if (score >= cfg.score)
+	if ((score >= cfg.score) && !find_tkl_exception(TKL_ANTIMIXEDUTF8, client))
 	{
 		if (cfg.ban_action == BAN_ACT_KILL)
 		{
 			sendto_realops("[antimixedutf8] Killed connection from %s (score %d)",
-				GetIP(sptr), score);
+				GetIP(client), score);
 		} /* no else here!! */
 
-		if ((cfg.ban_action == BAN_ACT_BLOCK)
-#ifdef BAN_ACT_SOFT_BLOCK
-		    || ((cfg.ban_action == BAN_ACT_SOFT_BLOCK) && !IsLoggedIn(sptr))
-#endif
-		    )
+		if ((cfg.ban_action == BAN_ACT_BLOCK) ||
+		    ((cfg.ban_action == BAN_ACT_SOFT_BLOCK) && !IsLoggedIn(client)))
 		{
-			sendnotice(sptr, "%s", cfg.ban_reason);
-			return 0;
+			sendnotice(client, "%s", cfg.ban_reason);
+			return;
 		} else {
-			ret = place_host_ban(sptr, cfg.ban_action, cfg.ban_reason, cfg.ban_time);
-			if (ret != 0)
-				return ret;
+			if (place_host_ban(client, cfg.ban_action, cfg.ban_reason, cfg.ban_time))
+				return;
 			/* a return value of 0 means the user is exempted, so fallthrough.. */
 		}
 	}
 
-	return CallCmdoverride(ovr, cptr, sptr, parc, parv);
+	CallCommandOverride(ovr, client, recv_mtags, parc, parv);
 }
 
 /*** rest is module and config stuff ****/
 
-MOD_TEST(antimixedutf8)
+MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, antimixedutf8_config_test);
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(antimixedutf8)
+MOD_INIT()
 {
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	
@@ -225,18 +244,18 @@ MOD_INIT(antimixedutf8)
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(antimixedutf8)
+MOD_LOAD()
 {
-	if (!CmdoverrideAdd(modinfo->handle, "PRIVMSG", override_msg))
+	if (!CommandOverrideAdd(modinfo->handle, "PRIVMSG", override_msg))
 		return MOD_FAILED;
 
-	if (!CmdoverrideAdd(modinfo->handle, "NOTICE", override_msg))
+	if (!CommandOverrideAdd(modinfo->handle, "NOTICE", override_msg))
 		return MOD_FAILED;
 
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(antimixedutf8)
+MOD_UNLOAD()
 {
 	free_config();
 	return MOD_SUCCESS;
@@ -247,14 +266,14 @@ static void init_config(void)
 	memset(&cfg, 0, sizeof(cfg));
 	/* Default values */
 	cfg.score = 10;
-	cfg.ban_reason = strdup("Possible mixed character spam");
+	safe_strdup(cfg.ban_reason, "Possible mixed character spam");
 	cfg.ban_action = BAN_ACT_BLOCK;
 	cfg.ban_time = 60 * 60 * 4; /* irrelevant for block, but some default for others */
 }
 
 static void free_config(void)
 {
-	safefree(cfg.ban_reason);
+	safe_free(cfg.ban_reason);
 	memset(&cfg, 0, sizeof(cfg)); /* needed! */
 }
 
@@ -336,9 +355,7 @@ int antimixedutf8_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		} else
 		if (!strcmp(cep->ce_varname, "ban-reason"))
 		{
-			if (cfg.ban_reason)
-				MyFree(cfg.ban_reason);
-			cfg.ban_reason = strdup(cep->ce_vardata);
+			safe_strdup(cfg.ban_reason, cep->ce_vardata);
 		} else
 		if (!strcmp(cep->ce_varname, "ban-time"))
 		{
