@@ -33,23 +33,19 @@
  */
 #undef TIMING
 
-ModuleHeader MOD_HEADER(antirandom)
+ModuleHeader MOD_HEADER
   = {
 	"antirandom",
-	"v1.4",
-	"Randomness detector",
-	"3.2-b8-1",
-	NULL 
+	"1.4",
+	"Detect and ban users with random names",
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 #ifndef MAX
  #define MAX(x,y) ((x) > (y) ? (x) : (y))
 #endif
 
-#ifndef BAN_ACT_WARN
- #define BAN_ACT_WARN 11
-#endif
-
 /* "<char1><char2>" followed by "<rest>" */
 static char *triples_txt[] = {
 	"aj", "fqtvxz",
@@ -496,8 +492,8 @@ static char *triples_txt[] = {
 
 /* Used for parsed triples: */
 #define TRIPLES_REST_SIZE	32
-typedef struct _triples Triples;
-struct _triples {
+typedef struct Triples Triples;
+struct Triples {
 	Triples *next;
 	char two[3];
 	char rest[TRIPLES_REST_SIZE];
@@ -514,7 +510,7 @@ struct {
 
 struct {
 	int threshold;
-	int ban_action;
+	BanAction ban_action;
 	char *ban_reason;
 	long ban_time;
 	int convert_to_lowercase;
@@ -532,10 +528,10 @@ static void free_config(void);
 int antirandom_config_test(ConfigFile *, ConfigEntry *, int, int *);
 int antirandom_config_run(ConfigFile *, ConfigEntry *, int);
 int antirandom_config_posttest(int *);
-int antirandom_preconnect(aClient *sptr);
-static int is_exempt(aClient *sptr);
+int antirandom_preconnect(Client *client);
+static int is_exempt(Client *client);
 
-MOD_TEST(antirandom)
+MOD_TEST()
 {
 	memset(&req, 0, sizeof(req));
 	memset(&cfg, 0, sizeof(cfg));
@@ -544,7 +540,7 @@ MOD_TEST(antirandom)
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(antirandom)
+MOD_INIT()
 {
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	if (!init_stuff())
@@ -566,38 +562,23 @@ MOD_INIT(antirandom)
 
 void check_all_users(void);
 
-MOD_LOAD(antirandom)
+MOD_LOAD()
 {
 	if (cfg.fullstatus_on_load)
 		check_all_users();
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(antirandom)
+MOD_UNLOAD()
 {
 	free_stuff();
 	free_config();
 	return MOD_SUCCESS;
 }
 
-/* Sends a message to all (local) opers AND logs to the ircdlog (as LOG_ERROR) */
-static void multi_log(char *fmt, ...)
-{
-	va_list vl;
-	static char buf[2048];
-
-	va_start(vl, fmt);
-	vsnprintf(buf, sizeof(buf), fmt, vl);
-	va_end(vl);
-	
-	sendto_realops("%s", buf);
-	ircd_log(LOG_ERROR, "%s", buf);
-}
-
-
 static void free_config(void)
 {
-	safefree(cfg.ban_reason);
+	safe_free(cfg.ban_reason);
 	unreal_delete_masks(cfg.except_hosts);
 	memset(&cfg, 0, sizeof(cfg)); /* needed! */
 }
@@ -644,7 +625,7 @@ int antirandom_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 		} else
 		if (!strcmp(cep->ce_varname, "ban-action"))
 		{
-			if (!banact_stringtoval(cep->ce_vardata) && strcmp(cep->ce_vardata, "warn"))
+			if (!banact_stringtoval(cep->ce_vardata))
 			{
 				config_error("%s:%i: set::antirandom::ban-action: unknown action '%s'",
 					cep->ce_fileptr->cf_filename, cep->ce_varlinenum, cep->ce_vardata);
@@ -708,14 +689,10 @@ int antirandom_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		if (!strcmp(cep->ce_varname, "ban-action"))
 		{
 			cfg.ban_action = banact_stringtoval(cep->ce_vardata);
-			if (!cfg.ban_action && strcmp(cep->ce_vardata, "warn"))
-				cfg.ban_action = BAN_ACT_WARN;
 		} else
 		if (!strcmp(cep->ce_varname, "ban-reason"))
 		{
-			if (cfg.ban_reason)
-				MyFree(cfg.ban_reason);
-			cfg.ban_reason = strdup(cep->ce_vardata);
+			safe_strdup(cfg.ban_reason, cep->ce_vardata);
 		} else
 		if (!strcmp(cep->ce_varname, "ban-time"))
 		{
@@ -767,7 +744,7 @@ static int init_triples(void)
 	for (s=triples_txt; *s; s++)
 	{
 		cnt++;
-		e = MyMallocEx(sizeof(Triples));
+		e = safe_alloc(sizeof(Triples));
 		if (strlen(*s) > 2)
 		{
 			config_error("init_triples: error parsing triples_txt, cnt=%d, item='%s' (length>2)",
@@ -845,21 +822,21 @@ static int internal_getscore(char *str)
 	{
 		score += 5 + (digits - 5);
 #ifdef DEBUGMODE
-		multi_log("score@'%s': MATCH for digits check", str);
+		sendto_ops_and_log("score@'%s': MATCH for digits check", str);
 #endif
 	}
 	if (vowels >= 4)
 	{
 		score += 4 + (vowels - 4);
 #ifdef DEBUGMODE
-		multi_log("score@'%s': MATCH for vowels check", str);
+		sendto_ops_and_log("score@'%s': MATCH for vowels check", str);
 #endif
 	}
 	if (consonants >= 4)
 	{
 		score += 4 + (consonants - 4);
 #ifdef DEBUGMODE
-		multi_log("score@'%s': MATCH for consonants check", str);
+		sendto_ops_and_log("score@'%s': MATCH for consonants check", str);
 #endif
 	}
 	
@@ -870,7 +847,7 @@ static int internal_getscore(char *str)
 			{
 				score++; /* OK */
 #ifdef DEBUGMODE
-				multi_log("score@'%s': MATCH for '%s[%s]' %c/%c/%c", str, t->two, t->rest,
+				sendto_ops_and_log("score@'%s': MATCH for '%s[%s]' %c/%c/%c", str, t->two, t->rest,
 					s[0], s[1], s[2]);
 #endif
 			}
@@ -881,30 +858,14 @@ static int internal_getscore(char *str)
 	return score;
 }
 
-void strtolower_safe(char *dst, char *src, int size)
-{
-	int i;
-
-	if (!size)
-		return; /* size of 0 is unworkable */
-	size--; /* for \0 */
-	
-	for (; *src && size; src++)
-	{
-		*dst++ = tolower(*src);
-		size--;
-	}
-	*dst = '\0';
-}
-
 /** Returns "spam score".
  * @note a user is expected, do not call for anything else (eg: servers)
  */
-static int get_spam_score(aClient *sptr)
+static int get_spam_score(Client *client)
 {
-	char *nick = sptr->name;
-	char *user = sptr->user->username;
-	char *gecos = sptr->info;
+	char *nick = client->name;
+	char *user = client->user->username;
+	char *gecos = client->info;
 	char nbuf[NICKLEN+1], ubuf[USERLEN+1], rbuf[REALLEN+1];
 	int nscore, uscore, gscore, score;
 #ifdef TIMING
@@ -934,7 +895,7 @@ static int get_spam_score(aClient *sptr)
 		((tv_beta.tv_sec - tv_alpha.tv_sec) * 1000000) + (tv_beta.tv_usec - tv_alpha.tv_usec));
 #endif
 #ifdef DEBUGMODE
-	multi_log("got score: %d/%d/%d = %d",
+	sendto_ops_and_log("got score: %d/%d/%d = %d",
 		nscore, uscore, gscore, score);
 #endif
 
@@ -943,23 +904,23 @@ static int get_spam_score(aClient *sptr)
 
 void check_all_users(void)
 {
-	aClient *acptr;
-	int i, matches=0, score;
+	Client *client;
+	int matches=0, score;
 	
-	list_for_each_entry(acptr, &lclient_list, lclient_node)
+	list_for_each_entry(client, &lclient_list, lclient_node)
 	{
-		if (IsPerson(acptr))
+		if (IsUser(client))
 		{
-			if (is_exempt(acptr))
+			if (is_exempt(client))
 				continue;
 
-			score = get_spam_score(acptr);
+			score = get_spam_score(client);
 			if (score > cfg.threshold)
 			{
 				if (!matches)
 					sendto_realops("[antirandom] Full status report follows:");
 				sendto_realops("%d points: %s!%s@%s:%s",
-					score, acptr->name, acptr->user->username, acptr->user->realhost, acptr->info);
+					score, client->name, client->user->username, client->user->realhost, client->info);
 				matches++;
 			}
 		}
@@ -968,28 +929,29 @@ void check_all_users(void)
 		sendto_realops("[antirandom] %d match%s", matches, matches == 1 ? "" : "es");
 }
 
-int antirandom_preconnect(aClient *sptr)
+int antirandom_preconnect(Client *client)
 {
 	int score;
 
-	if (!is_exempt(sptr))
+	if (is_exempt(client))
+		return HOOK_CONTINUE;
+
+	score = get_spam_score(client);
+	if (score > cfg.threshold)
 	{
-		score = get_spam_score(sptr);
-		if (score > cfg.threshold)
+		if (cfg.ban_action == BAN_ACT_WARN)
 		{
-			if (cfg.ban_action == BAN_ACT_WARN)
-			{
-				multi_log("[antirandom] would have denied access to user with score %d: %s!%s@%s:%s",
-					score, sptr->name, sptr->user->username, sptr->user->realhost, sptr->info);
-				return 0;
-			}
-			if (cfg.show_failedconnects)
-				multi_log("[antirandom] denied access to user with score %d: %s!%s@%s:%s",
-					score, sptr->name, sptr->user->username, sptr->user->realhost, sptr->info);
-			return place_host_ban(sptr, cfg.ban_action, cfg.ban_reason, cfg.ban_time);
+			sendto_ops_and_log("[antirandom] would have denied access to user with score %d: %s!%s@%s:%s",
+				score, client->name, client->user->username, client->user->realhost, client->info);
+			return HOOK_CONTINUE;
 		}
+		if (cfg.show_failedconnects)
+			sendto_ops_and_log("[antirandom] denied access to user with score %d: %s!%s@%s:%s",
+				score, client->name, client->user->username, client->user->realhost, client->info);
+		place_host_ban(client, cfg.ban_action, cfg.ban_reason, cfg.ban_time);
+		return HOOK_DENY;
 	}
-	return 0;
+	return HOOK_CONTINUE;
 }
 
 static void free_stuff(void)
@@ -999,26 +961,29 @@ static void free_stuff(void)
 	for (t=triples; t; t=t_next)
 	{
 		t_next = t->next;
-		MyFree(t);
+		safe_free(t);
 	}
 	triples = NULL;
 }
 
 /** Is this user exempt from antirandom interventions? */
-static int is_exempt(aClient *sptr)
+static int is_exempt(Client *client)
 {
 	/* WEBIRC gateway and exempt? */
 	if (cfg.except_webirc)
 	{
-		char *val = moddata_client_get(sptr, "webirc");
+		char *val = moddata_client_get(client, "webirc");
 		if (val && (atoi(val)>0))
 			return 1;
 	}
 
+	if (find_tkl_exception(TKL_ANTIRANDOM, client))
+		return 1;
+
 	/* Soft ban and logged in? */
-	if (IsSoftBanAction(cfg.ban_action) && IsLoggedIn(sptr))
+	if (IsSoftBanAction(cfg.ban_action) && IsLoggedIn(client))
 		return 1;
 
 	/* On except host? */
-	return unreal_mask_match(sptr, cfg.except_hosts);
+	return unreal_mask_match(client, cfg.except_hosts);
 }
