@@ -18,15 +18,15 @@
  */
 
 #include "unrealircd.h"
-#include "res.h"
+#include "dns.h"
 
-ModuleHeader MOD_HEADER(blacklist)
+ModuleHeader MOD_HEADER
 = {
 	"blacklist",
-	"4.2",
+	"5.0",
 	"Check connecting users against DNS Blacklists",
-	"3.2-b8-1",
-	NULL
+	"UnrealIRCd Team",
+	"unrealircd-5",
 };
 
 /* In this module and the config syntax I tried to 'abstract' things
@@ -41,15 +41,15 @@ typedef enum {
 	DNSBL_RECORD=1, DNSBL_BITMASK=2
 } DNSBLType;
 
-typedef struct _dnsbl DNSBL;
-struct _dnsbl {
+typedef struct DNSBL DNSBL;
+struct DNSBL {
 	char *name;
 	DNSBLType type;
 	int *reply;
 };
 
-typedef union _blacklistbackend BlacklistBackend;
-union _blacklistbackend
+typedef union BlacklistBackend BlacklistBackend;
+union BlacklistBackend
 {
 	DNSBL *dns;
 };
@@ -58,8 +58,8 @@ typedef enum {
 	BLACKLIST_BACKEND_DNS = 1
 } BlacklistBackendType;
 
-typedef struct _blacklist Blacklist;
-struct _blacklist {
+typedef struct Blacklist Blacklist;
+struct Blacklist {
 	Blacklist *prev, *next;
 	char *name;
 	BlacklistBackendType backend_type;
@@ -70,12 +70,12 @@ struct _blacklist {
 };
 
 /* Blacklist user struct. In the c-ares DNS reply callback we need to pass
- * some metadata. We can't use cptr directly there as the client may
+ * some metadata. We can't use client directly there as the client may
  * be gone already by the time we receive the DNS reply.
  */
-typedef struct _bluser BLUser;
-struct _bluser {
-	aClient *cptr;
+typedef struct BLUser BLUser;
+struct BLUser {
+	Client *client;
 	int is_ipv6;
 	int refcnt;
 	/* The following save_* fields are used by softbans: */
@@ -95,11 +95,12 @@ int blacklist_config_run(ConfigFile *, ConfigEntry *, int);
 void blacklist_free_conf(void);
 void delete_blacklist_block(Blacklist *e);
 void blacklist_md_free(ModData *md);
-int blacklist_handshake(aClient *cptr);
-int blacklist_preconnect(aClient *sptr);
+int blacklist_handshake(Client *client);
+int blacklist_quit(Client *client, MessageTag *mtags, char *comment);
+int blacklist_preconnect(Client *client);
 void blacklist_resolver_callback(void *arg, int status, int timeouts, struct hostent *he);
-int blacklist_start_check(aClient *cptr);
-int blacklist_dns_request(aClient *cptr, Blacklist *bl);
+int blacklist_start_check(Client *client);
+int blacklist_dns_request(Client *client, Blacklist *bl);
 int blacklist_rehash(void);
 int blacklist_rehash_complete(void);
 void blacklist_set_handshake_delay(void);
@@ -110,7 +111,7 @@ void blacklist_free_bluser_if_able(BLUser *bl);
 
 long SNO_BLACKLIST = 0L;
 
-MOD_TEST(blacklist)
+MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, blacklist_config_test);
 
@@ -119,11 +120,16 @@ MOD_TEST(blacklist)
 }
 
 /** Called upon module init */
-MOD_INIT(blacklist)
+MOD_INIT()
 {
 	ModDataInfo mreq;
 
 	MARK_AS_OFFICIAL_MODULE(modinfo);
+	/* This module needs to be permanent.
+	 * Not because of UnrealIRCd restrictions,
+	 * but because we use c-ares callbacks and the address
+	 * of those functions will change if we REHASH.
+	 */
 	ModuleSetOptions(modinfo->handle, MOD_OPT_PERM, 1);
 	
 	memset(&mreq, 0, sizeof(mreq));
@@ -142,21 +148,22 @@ MOD_INIT(blacklist)
 	HookAdd(modinfo->handle, HOOKTYPE_PRE_LOCAL_CONNECT, 0, blacklist_preconnect);
 	HookAdd(modinfo->handle, HOOKTYPE_REHASH, 0, blacklist_rehash);
 	HookAdd(modinfo->handle, HOOKTYPE_REHASH_COMPLETE, 0, blacklist_rehash_complete);
+	HookAdd(modinfo->handle, HOOKTYPE_LOCAL_QUIT, 0, blacklist_quit);
 
-	SnomaskAdd(modinfo->handle, 'b', 1, umode_allow_opers, &SNO_BLACKLIST);
+	SnomaskAdd(modinfo->handle, 'b', umode_allow_opers, &SNO_BLACKLIST);
 
 	return MOD_SUCCESS;
 }
 
 /** Called upon module load */
-MOD_LOAD(blacklist)
+MOD_LOAD()
 {
 	blacklist_set_handshake_delay();
 	return MOD_SUCCESS;
 }
 
 /** Called upon unload */
-MOD_UNLOAD(blacklist)
+MOD_UNLOAD()
 {
 	blacklist_free_conf();
 	return MOD_SUCCESS;
@@ -218,17 +225,17 @@ void delete_blacklist_block(Blacklist *e)
 	{
 		if (e->backend->dns)
 		{
-			MyFree(e->backend->dns->name);
-			MyFree(e->backend->dns->reply);
-			MyFree(e->backend->dns);
+			safe_free(e->backend->dns->name);
+			safe_free(e->backend->dns->reply);
+			safe_free(e->backend->dns);
 		}
 	}
 	
-	MyFree(e->backend);
+	safe_free(e->backend);
 
-	MyFree(e->name);
-	MyFree(e->reason);
-	MyFree(e);
+	safe_free(e->name);
+	safe_free(e->reason);
+	safe_free(e);
 }
 
 int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
@@ -237,7 +244,6 @@ int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 	int errors = 0;
 	char has_reason = 0, has_ban_time = 0, has_action = 0;
 	char has_dns_type = 0, has_dns_reply = 0, has_dns_name = 0;
-	DNSBLType blacklist_type = 0;
 
 	if (type != CONFIG_MAIN)
 		return 0;
@@ -337,9 +343,9 @@ int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 					}
 					has_dns_type = 1;
 					if (!strcmp(cepp->ce_vardata, "record"))
-						blacklist_type = DNSBL_RECORD;
+						;
 					else if (!strcmp(cepp->ce_vardata, "bitmask"))
-						blacklist_type = DNSBL_BITMASK;
+						;
 					else
 					{
 						config_error("%s:%i: unknown blacklist::dns::type '%s', must be either 'record' or 'bitmask'",
@@ -450,17 +456,17 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "blacklist"))
 		return 0; /* not interested */
 
-	d = MyMallocEx(sizeof(Blacklist));
-	d->name = strdup(ce->ce_vardata);
+	d = safe_alloc(sizeof(Blacklist));
+	safe_strdup(d->name, ce->ce_vardata);
 	/* set some defaults. TODO: use set::blacklist or something ? */
 	d->action = BAN_ACT_KILL;
-	d->reason = strdup("Your IP is on a DNS Blacklist");
+	safe_strdup(d->reason, "Your IP is on a DNS Blacklist");
 	d->ban_time = 3600;
 	
 	/* assume dns for now ;) */
 	d->backend_type = BLACKLIST_BACKEND_DNS;
-	d->backend = MyMallocEx(sizeof(BlacklistBackend));
-	d->backend->dns = MyMallocEx(sizeof(DNSBL));
+	d->backend = safe_alloc(sizeof(BlacklistBackend));
+	d->backend->dns = safe_alloc(sizeof(DNSBL));
 
 	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
 	{
@@ -473,7 +479,7 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 					if (cepp->ce_vardata)
 					{
 						/* single reply */
-						d->backend->dns->reply = MyMallocEx(sizeof(int)*2);
+						d->backend->dns->reply = safe_alloc(sizeof(int)*2);
 						d->backend->dns->reply[0] = atoi(cepp->ce_vardata);
 						d->backend->dns->reply[1] = 0;
 					} else
@@ -490,7 +496,7 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 						if (cnt == 0)
 							abort(); /* impossible */
 						
-						d->backend->dns->reply = MyMallocEx(sizeof(int)*(cnt+1));
+						d->backend->dns->reply = safe_alloc(sizeof(int)*(cnt+1));
 						
 						cnt = 0;
 						for (ceppp = cepp->ce_entries; ceppp; ceppp = ceppp->ce_next)
@@ -509,7 +515,7 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 				} else
 				if (!strcmp(cepp->ce_varname, "name"))
 				{
-					safestrdup(d->backend->dns->name, cepp->ce_vardata);
+					safe_strdup(d->backend->dns->name, cepp->ce_vardata);
 				}
 			}
 		}
@@ -519,7 +525,7 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		}
 		else if (!strcmp(cep->ce_varname, "reason"))
 		{
-			safestrdup(d->reason, cep->ce_vardata);
+			safe_strdup(d->reason, cep->ce_vardata);
 		}
 		else if (!strcmp(cep->ce_varname, "ban-time"))
 		{
@@ -536,51 +542,57 @@ void blacklist_md_free(ModData *md)
 {
 	BLUser *bl = md->ptr;
 
-	/* Mark bl->cptr as dead. Free the struct, if able. */
+	/* Mark bl->client as dead. Free the struct, if able. */
 	blacklist_free_bluser_if_able(bl);
 
 	md->ptr = NULL;
 }
 
-int blacklist_handshake(aClient *cptr)
+int blacklist_handshake(Client *client)
 {
-	blacklist_start_check(cptr);
+	blacklist_start_check(client);
 	return 0;
 }
 
-int blacklist_start_check(aClient *cptr)
+int blacklist_start_check(Client *client)
 {
 	Blacklist *bl;
 
-	/* If the user is on 'except blacklist' then don't bother checking... */
-	if (Find_except(cptr, CONF_EXCEPT_BLACKLIST))
-		return 0;
-	
-	if (!BLUSER(cptr))
+	if (find_tkl_exception(TKL_BLACKLIST, client))
 	{
-		SetBLUser(cptr, MyMallocEx(sizeof(BLUser)));
-		BLUSER(cptr)->cptr = cptr;
+		/* If the user is exempt from DNSBL checking then:
+		 * 1) Don't bother checking DNSBL's
+		 * 2) Disable handshake delay for this user, since it serves no purpose.
+		 */
+		SetNoHandshakeDelay(client);
+		return 0;
+	}
+	
+	if (!BLUSER(client))
+	{
+		SetBLUser(client, safe_alloc(sizeof(BLUser)));
+		BLUSER(client)->client = client;
 	}
 
 	for (bl = conf_blacklist; bl; bl = bl->next)
 	{
 		/* Stop processing if client is (being) killed already */
-		if (!BLUSER(cptr))
+		if (!BLUSER(client))
 			break;
 
 		/* Initiate blacklist requests */
 		if (bl->backend_type == BLACKLIST_BACKEND_DNS)
-			blacklist_dns_request(cptr, bl);
+			blacklist_dns_request(client, bl);
 	}
 	
 	return 0;
 }
 
-int blacklist_dns_request(aClient *cptr, Blacklist *d)
+int blacklist_dns_request(Client *client, Blacklist *d)
 {
 	char buf[256], wbuf[128];
 	unsigned int e[8];
-	char *ip = GetIP(cptr);
+	char *ip = GetIP(client);
 	
 	if (!ip)
 		return 0;
@@ -597,9 +609,9 @@ int blacklist_dns_request(aClient *cptr, Blacklist *d)
 	} else
 	if (strchr(ip, ':'))
 	{
-		int i, j;
 		/* IPv6 */
-		BLUSER(cptr)->is_ipv6 = 1;
+		int i;
+		BLUSER(client)->is_ipv6 = 1;
 		if (sscanf(ip, "%x:%x:%x:%x:%x:%x:%x:%x",
 		    &e[0], &e[1], &e[2], &e[3], &e[4], &e[5], &e[6], &e[7]) != 8)
 		{
@@ -620,22 +632,22 @@ int blacklist_dns_request(aClient *cptr, Blacklist *d)
 	else
 		return 0; /* unknown IP format */
 
-	BLUSER(cptr)->refcnt++; /* one (more) blacklist result remaining */
+	BLUSER(client)->refcnt++; /* one (more) blacklist result remaining */
 	
-	unreal_gethostbyname(buf, AF_INET, blacklist_resolver_callback, BLUSER(cptr));
+	unreal_gethostbyname(buf, AF_INET, blacklist_resolver_callback, BLUSER(client));
 	
 	return 0;
 }
 
 void blacklist_cancel(BLUser *bl)
 {
-	bl->cptr = NULL;
+	bl->client = NULL;
 }
 
-int blacklist_quit(aClient *cptr, char *comment)
+int blacklist_quit(Client *client, MessageTag *mtags, char *comment)
 {
-	if (BLUSER(cptr))
-		blacklist_cancel(BLUSER(cptr));
+	if (BLUSER(client))
+		blacklist_cancel(BLUSER(client));
 
 	return 0;
 }
@@ -650,33 +662,39 @@ int blacklist_quit(aClient *cptr, char *comment)
  */
 void blacklist_free_bluser_if_able(BLUser *bl)
 {
-	if (bl->cptr)
-		bl->cptr = NULL;
+	if (bl->client)
+		bl->client = NULL;
 
 	if (bl->refcnt > 0)
 		return; /* unable, still have DNS requests/replies in-flight */
 
-	safefree(bl->save_opernotice);
-	safefree(bl->save_reason);
-	MyFree(bl);
+	safe_free(bl->save_opernotice);
+	safe_free(bl->save_reason);
+	safe_free(bl);
 }
 
-char *getdnsblname(char *p, aClient *cptr)
+char *getdnsblname(char *p, Client *client)
 {
-int dots = 0;
+	int dots = 0;
 	int dots_count;
-	if(!cptr) return NULL;
-	if(BLUSER(cptr)->is_ipv6)
+
+	if (!client)
+		return NULL;
+
+	if (BLUSER(client)->is_ipv6)
 		dots_count = 32;
 	else
 		dots_count = 4;
+
 	for (; *p; p++)
+	{
 		if (*p == '.')
 		{
 			dots++;
 			if (dots == dots_count)
 				return p+1;
 		}
+	}
 	return NULL;
 }
 
@@ -707,31 +725,33 @@ int blacklist_parse_reply(struct hostent *he, int entry)
  * from blacklist_preconnect() for softbans that need to be delayed
  * as to give the user the opportunity to do SASL Authentication.
  */
-int blacklist_action(aClient *acptr, char *opernotice, int ban_action, char *ban_reason, long ban_time)
+int blacklist_action(Client *client, char *opernotice, BanAction ban_action, char *ban_reason, long ban_time)
 {
 	sendto_snomask(SNO_BLACKLIST, "%s", opernotice);
 	ircd_log(LOG_KILL, "%s", opernotice);
-	return place_host_ban(acptr, ban_action, ban_reason, ban_time);
+	if (ban_action == BAN_ACT_WARN)
+		return 0;
+	return place_host_ban(client, ban_action, ban_reason, ban_time);
 }
 
-void blacklist_hit(aClient *acptr, Blacklist *bl, int reply)
+void blacklist_hit(Client *client, Blacklist *bl, int reply)
 {
 	char opernotice[512], banbuf[512];
 	const char *name[4], *value[4];
-	BLUser *blu = BLUSER(acptr);
+	BLUser *blu = BLUSER(client);
 
-	if (find_tkline_match(acptr, 1) < 0)
+	if (find_tkline_match(client, 1))
 		return; /* already klined/glined. Don't send the warning from below. */
 
-	if (IsPerson(acptr))
+	if (IsUser(client))
 		snprintf(opernotice, sizeof(opernotice), "[Blacklist] IP %s (%s) matches blacklist %s (%s/reply=%d)",
-			GetIP(acptr), acptr->name, bl->name, bl->backend->dns->name, reply);
+			GetIP(client), client->name, bl->name, bl->backend->dns->name, reply);
 	else
 		snprintf(opernotice, sizeof(opernotice), "[Blacklist] IP %s matches blacklist %s (%s/reply=%d)",
-			GetIP(acptr), bl->name, bl->backend->dns->name, reply);
+			GetIP(client), bl->name, bl->backend->dns->name, reply);
 
 	name[0] = "ip";
-	value[0] = GetIP(acptr);
+	value[0] = GetIP(client);
 	name[1] = "server";
 	value[1] = me.name;
 	name[2] = NULL;
@@ -744,15 +764,15 @@ void blacklist_hit(aClient *acptr, Blacklist *bl, int reply)
 		/* For soft bans, delay the action until later (so user can do SASL auth) */
 		blu->save_action = bl->action;
 		blu->save_tkltime = bl->ban_time;
-		safestrdup(blu->save_opernotice, opernotice);
-		safestrdup(blu->save_reason, banbuf);
+		safe_strdup(blu->save_opernotice, opernotice);
+		safe_strdup(blu->save_reason, banbuf);
 	} else {
 		/* Otherwise, execute the action immediately */
-		blacklist_action(acptr, opernotice, bl->action, banbuf, bl->ban_time);
+		blacklist_action(client, opernotice, bl->action, banbuf, bl->ban_time);
 	}
 }
 
-void blacklist_process_result(aClient *acptr, int status, struct hostent *he)
+void blacklist_process_result(Client *client, int status, struct hostent *he)
 {
 	Blacklist *bl;
 	char *domain;
@@ -763,7 +783,7 @@ void blacklist_process_result(aClient *acptr, int status, struct hostent *he)
 	if ((status != 0) || (he->h_length != 4) || !he->h_name)
 		return; /* invalid reply */
 	
-	domain = getdnsblname(he->h_name, acptr);
+	domain = getdnsblname(he->h_name, client);
 	if (!domain)
 		return; /* odd */
 	bl = blacklist_find_block_by_dns(domain);
@@ -781,7 +801,7 @@ void blacklist_process_result(aClient *acptr, int status, struct hostent *he)
 				( (bl->backend->dns->type == DNSBL_BITMASK) && (reply & bl->backend->dns->reply[i]) ) ||
 				( (bl->backend->dns->type == DNSBL_RECORD) && (bl->backend->dns->reply[i] == reply) ) )
 			{
-				blacklist_hit(acptr, bl, reply);
+				blacklist_hit(client, bl, reply);
 				return;
 			}
 		}
@@ -791,35 +811,37 @@ void blacklist_process_result(aClient *acptr, int status, struct hostent *he)
 void blacklist_resolver_callback(void *arg, int status, int timeouts, struct hostent *he)
 {
 	BLUser *blu = (BLUser *)arg;
-	aClient *acptr = blu->cptr;
+	Client *client = blu->client;
 
 	blu->refcnt--; /* one less outstanding DNS request remaining */
 
 	/* If we are the last to resolve something and the client is gone
 	 * already then free the struct.
 	 */
-	if ((blu->refcnt == 0) && !acptr)
+	if ((blu->refcnt == 0) && !client)
 		blacklist_free_bluser_if_able(blu);
 
 	blu = NULL;
 
-	if (!acptr)
+	if (!client)
 		return; /* Client left already */
 	/* ^^ note: do not merge this with the other 'if' a few lines up (refcnt!) */
 
-	blacklist_process_result(acptr, status, he);
+	blacklist_process_result(client, status, he);
 }
 
-int blacklist_preconnect(aClient *acptr)
+int blacklist_preconnect(Client *client)
 {
-	BLUser *blu = BLUSER(acptr);
+	BLUser *blu = BLUSER(client);
 
 	if (!blu || !blu->save_action)
-		return 0;
+		return HOOK_CONTINUE;
 
 	/* There was a pending softban... has the user authenticated via SASL by now? */
-	if (IsLoggedIn(acptr))
-		return 0; /* yup, so the softban does not apply. */
+	if (IsLoggedIn(client))
+		return HOOK_CONTINUE; /* yup, so the softban does not apply. */
 
-	return blacklist_action(acptr, blu->save_opernotice, blu->save_action, blu->save_reason, blu->save_tkltime);
+	if (blacklist_action(client, blu->save_opernotice, blu->save_action, blu->save_reason, blu->save_tkltime))
+		return HOOK_DENY;
+	return HOOK_CONTINUE; /* exempt */
 }
