@@ -21,27 +21,27 @@
 
 CMD_FUNC(noctcp);
 
-ModuleHeader MOD_HEADER(noctcp)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/noctcp",
 	"4.2",
 	"Channel Mode +C",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 Cmode_t EXTCMODE_NOCTCP;
 
-#define IsNoCTCP(chptr)    (chptr->mode.extmode & EXTCMODE_NOCTCP)
+#define IsNoCTCP(channel)    (channel->mode.extmode & EXTCMODE_NOCTCP)
 
-char *noctcp_prechanmsg(aClient *sptr, aChannel *chptr, char *text, int notice);
+int noctcp_can_send_to_channel(Client *client, Channel *channel, Membership *lp, char **msg, char **errmsg, SendType sendtype);
 
-MOD_TEST(noctcp)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(noctcp)
+MOD_INIT()
 {
 CmodeInfo req;
 
@@ -51,18 +51,18 @@ CmodeInfo req;
 	req.is_ok = extcmode_default_requirehalfop;
 	CmodeAdd(modinfo->handle, req, &EXTCMODE_NOCTCP);
 	
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_CHANMSG, 0, noctcp_prechanmsg);
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_CHANNEL, 0, noctcp_can_send_to_channel);
 	
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(noctcp)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(noctcp)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
@@ -78,18 +78,12 @@ static int IsACTCP(char *s)
 	return 0;
 }
 
-char *noctcp_prechanmsg(aClient *sptr, aChannel *chptr, char *text, int notice)
+int noctcp_can_send_to_channel(Client *client, Channel *channel, Membership *lp, char **msg, char **errmsg, SendType sendtype)
 {
-	if (MyClient(sptr) && IsNoCTCP(chptr) && IsACTCP(text))
+	if (IsNoCTCP(channel) && IsACTCP(*msg))
 	{
-		if (!notice)
-		{
-			sendto_one(sptr, err_str(ERR_CANNOTSENDTOCHAN),
-				   me.name, sptr->name, chptr->chname,
-				   "CTCPs are not permitted in this channel", chptr->chname);
-		}
-
-		return NULL;
+		*errmsg = "CTCPs are not permitted in this channel";
+		return HOOK_DENY;
 	}
-	return text;
+	return HOOK_CONTINUE;
 }
