@@ -19,29 +19,29 @@
 
 #include "unrealircd.h"
 
-ModuleHeader MOD_HEADER(noknock)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/noknock",
 	"4.2",
 	"Channel Mode +K",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 Cmode_t EXTCMODE_NOKNOCK;
 
-#define IsNoKnock(chptr)    (chptr->mode.extmode & EXTCMODE_NOKNOCK)
+#define IsNoKnock(channel)    (channel->mode.extmode & EXTCMODE_NOKNOCK)
 
-int noknock_check (aClient *sptr, aChannel *chptr);
-int noknock_mode_allow(aClient *cptr, aChannel *chptr, char mode, char *para, int checkt, int what);
-int noknock_mode_del (aChannel *chptr, int modeChar);
+int noknock_check (Client *client, Channel *channel);
+int noknock_mode_allow(Client *client, Channel *channel, char mode, char *para, int checkt, int what);
+int noknock_mode_del (Channel *channel, int modeChar);
 
-MOD_TEST(noknock)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(noknock)
+MOD_INIT()
 {
 CmodeInfo req;
 
@@ -59,49 +59,47 @@ CmodeInfo req;
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(noctcp)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(noctcp)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
 
-int noknock_check (aClient *sptr, aChannel *chptr)
+int noknock_check (Client *client, Channel *channel)
 {
-	if (MyClient(sptr) && IsNoKnock(chptr))
+	if (MyUser(client) && IsNoKnock(channel))
 	{
-		sendto_one(sptr, err_str(ERR_CANNOTKNOCK),
-				    me.name,
-				    sptr->name, chptr->chname, "No knocks are allowed! (+K)");
+		sendnumeric(client, ERR_CANNOTKNOCK, channel->chname, "No knocks are allowed! (+K)");
 		return HOOK_DENY;
 	}
 
 	return HOOK_CONTINUE;
 }
 
-int noknock_mode_del (aChannel *chptr, int modeChar)
+int noknock_mode_del (Channel *channel, int modeChar)
 {
 	// Remove noknock when we're removing invite only
 	if (modeChar == 'i')
-		chptr->mode.extmode &= ~EXTCMODE_NOKNOCK;
+		channel->mode.extmode &= ~EXTCMODE_NOKNOCK;
 
 	return 0;
 }
 
-int noknock_mode_allow(aClient *cptr, aChannel *chptr, char mode, char *para, int checkt, int what)
+int noknock_mode_allow(Client *client, Channel *channel, char mode, char *para, int checkt, int what)
 {
-
-	if (!(chptr->mode.mode & MODE_INVITEONLY))
+	if (!(channel->mode.mode & MODE_INVITEONLY))
 	{
-		sendto_one(cptr, err_str(ERR_CANNOTCHANGECHANMODE),
-						me.name, cptr->name, 'K', "+i must be set");
+		if (checkt == EXCHK_ACCESS_ERR)
+		{
+			sendnumeric(client, ERR_CANNOTCHANGECHANMODE, 'K', "+i must be set");
+		}
 		return EX_DENY;
 	}
 
-	return extcmode_default_requirehalfop(cptr,chptr,mode,para,checkt,what);
+	return extcmode_default_requirehalfop(client, channel, mode, para, checkt, what);
 }
-
