@@ -20,27 +20,27 @@
 #include "unrealircd.h"
 
 
-ModuleHeader MOD_HEADER(nonickchange)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/nonickchange",
 	"4.2",
 	"Channel Mode +N",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 Cmode_t EXTCMODE_NONICKCHANGE;
 
-#define IsNoNickChange(chptr)    (chptr->mode.extmode & EXTCMODE_NONICKCHANGE)
+#define IsNoNickChange(channel)    (channel->mode.extmode & EXTCMODE_NONICKCHANGE)
 
-int nonickchange_check (aClient *sptr, aChannel *chptr);
+int nonickchange_check (Client *client, Channel *channel);
 
-MOD_TEST(nonickchange)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(nonickchange)
+MOD_INIT()
 {
 CmodeInfo req;
 
@@ -57,22 +57,24 @@ CmodeInfo req;
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(nonickchange)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(nonickchange)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-int nonickchange_check (aClient *sptr, aChannel *chptr)
+int nonickchange_check (Client *client, Channel *channel)
 {
-	if (!IsOper(sptr) && !IsULine(sptr)
-		&& IsNoNickChange(chptr)
-		&& !is_chanownprotop(sptr, chptr))
+	if (!IsOper(client) && !IsULine(client)
+		&& IsNoNickChange(channel)
+		&& !is_chan_op(client, channel))
+	{
 		return HOOK_DENY;
+	}
 
 	return HOOK_ALLOW;
 }
