@@ -20,28 +20,28 @@
 #include "unrealircd.h"
 
 
-ModuleHeader MOD_HEADER(regonly)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/regonly",
 	"4.2",
 	"Channel Mode +R",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 Cmode_t EXTCMODE_REGONLY;
 
-#define IsRegOnly(chptr)    (chptr->mode.extmode & EXTCMODE_REGONLY)
+#define IsRegOnly(channel)    (channel->mode.extmode & EXTCMODE_REGONLY)
 
-int regonly_check (aClient *cptr, aChannel *chptr, char *key, char *parv[]);
+int regonly_check (Client *client, Channel *channel, char *key, char *parv[]);
 
 
-MOD_TEST(regonly)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(regonly)
+MOD_INIT()
 {
 CmodeInfo req;
 
@@ -58,19 +58,19 @@ CmodeInfo req;
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(regonly)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(regonly)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-int regonly_check (aClient *cptr, aChannel *chptr, char *key, char *parv[])
+int regonly_check (Client *client, Channel *channel, char *key, char *parv[])
 {
-	if (IsRegOnly(chptr) && !IsLoggedIn(cptr))
+	if (IsRegOnly(channel) && !IsLoggedIn(client))
 		return ERR_NEEDREGGEDNICK;
 	return 0;
 }
