@@ -21,29 +21,29 @@
 
 CMD_FUNC(stripcolor);
 
-ModuleHeader MOD_HEADER(stripcolor)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/stripcolor",
 	"4.2",
 	"Channel Mode +S",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 Cmode_t EXTCMODE_STRIPCOLOR;
 
-#define IsStripColor(chptr)    (chptr->mode.extmode & EXTCMODE_STRIPCOLOR)
+#define IsStripColor(channel)    (channel->mode.extmode & EXTCMODE_STRIPCOLOR)
 
-char *stripcolor_prechanmsg(aClient *sptr, aChannel *chptr, char *text, int notice);
-char *stripcolor_prelocalpart(aClient *sptr, aChannel *chptr, char *comment);
-char *stripcolor_prelocalquit(aClient *sptr, char *comment);
+int stripcolor_can_send_to_channel(Client *client, Channel *channel, Membership *lp, char **msg, char **errmsg, SendType sendtype);
+char *stripcolor_prelocalpart(Client *client, Channel *channel, char *comment);
+char *stripcolor_prelocalquit(Client *client, char *comment);
 
-MOD_TEST(stripcolor)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(stripcolor)
+MOD_INIT()
 {
 CmodeInfo req;
 
@@ -54,75 +54,76 @@ CmodeInfo req;
 	req.is_ok = extcmode_default_requirechop;
 	CmodeAdd(modinfo->handle, req, &EXTCMODE_STRIPCOLOR);
 	
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_CHANMSG, 0, stripcolor_prechanmsg);
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_CHANNEL, 0, stripcolor_can_send_to_channel);
 	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_PART, 0, stripcolor_prelocalpart);
+	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT_CHAN, 0, stripcolor_prelocalpart);
 	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT, 0, stripcolor_prelocalquit);
 	
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(stripcolor)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(stripcolor)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-char *stripcolor_prechanmsg(aClient *sptr, aChannel *chptr, char *text, int notice)
+int stripcolor_can_send_to_channel(Client *client, Channel *channel, Membership *lp, char **msg, char **errmsg, SendType sendtype)
 {
 	Hook *h;
 	int i;
 
-	if (MyClient(sptr) && IsStripColor(chptr))
+	if (MyUser(client) && IsStripColor(channel))
 	{
 		for (h = Hooks[HOOKTYPE_CAN_BYPASS_CHANNEL_MESSAGE_RESTRICTION]; h; h = h->next)
 		{
-			i = (*(h->func.intfunc))(sptr, chptr, BYPASS_CHANMSG_COLOR);
+			i = (*(h->func.intfunc))(client, channel, BYPASS_CHANMSG_COLOR);
+			if (i == HOOK_ALLOW)
+				return HOOK_CONTINUE; /* bypass this restriction */
 			if (i != HOOK_CONTINUE)
 				break;
 		}
-		if (i == HOOK_ALLOW)
-			return text; /* bypass */
 
-		text = StripColors(text);
+		*msg = StripColors(*msg);
 	}
 
-	return text;
+	return HOOK_CONTINUE;
 }
 
-char *stripcolor_prelocalpart(aClient *sptr, aChannel *chptr, char *comment)
+char *stripcolor_prelocalpart(Client *client, Channel *channel, char *comment)
 {
 	if (!comment)
 		return NULL;
 
-	if (MyClient(sptr) && IsStripColor(chptr))
+	if (MyUser(client) && IsStripColor(channel))
 		comment = StripColors(comment);
 
 	return comment;
 }
 
 /** Is any channel where the user is in +S? */
-static int IsAnyChannelStripColor(aClient *sptr)
+static int IsAnyChannelStripColor(Client *client)
 {
 	Membership *lp;
 
-	for (lp = sptr->user->channel; lp; lp = lp->next)
-		if (IsStripColor(lp->chptr))
+	for (lp = client->user->channel; lp; lp = lp->next)
+		if (IsStripColor(lp->channel))
 			return 1;
 	return 0;
 }
 
 
-char *stripcolor_prelocalquit(aClient *sptr, char *comment)
+char *stripcolor_prelocalquit(Client *client, char *comment)
 {
 	if (!comment)
 		return NULL;
 
-	if (MyClient(sptr) && !BadPtr(comment) && IsAnyChannelStripColor(sptr))
+	if (MyUser(client) && !BadPtr(comment) && IsAnyChannelStripColor(client))
 		comment = StripColors(comment);
         
 	return comment;
