@@ -26,13 +26,13 @@
  #define ARRAY_SIZEOF(x) (sizeof((x))/sizeof((x)[0]))
 #endif
 
-ModuleHeader MOD_HEADER(charsys)
+ModuleHeader MOD_HEADER
 = {
 	"charsys",	/* Name of module */
-	"4.2", /* Version */
+	"5.0", /* Version */
 	"Character System (set::allowed-nickchars)", /* Short description of module */
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
 };
 
 /* NOTE: it is guaranteed that char is unsigned by compiling options
@@ -42,8 +42,8 @@ ModuleHeader MOD_HEADER(charsys)
  */
 
 /** Our multibyte structure */
-typedef struct _mblist MBList;
-struct _mblist
+typedef struct MBList MBList;
+struct MBList
 {
 	MBList *next;
 	char s1, e1, s2, e2;
@@ -53,7 +53,7 @@ MBList *mblist = NULL, *mblist_tail = NULL;
 /* Use this to prevent mixing of certain combinations
  * (such as GBK & high-ascii, etc)
  */
-static int langav;
+static int langav = 0;
 char langsinuse[4096];
 
 /* bitmasks: */
@@ -67,13 +67,15 @@ char langsinuse[4096];
 #define LANGAV_W1251			0x000080 /* windows-1251 (eg: russian) */
 #define LANGAV_LATIN2W1250		0x000100 /* Compatible with both latin2 AND windows-1250 (eg: hungarian) */
 #define LANGAV_ISO8859_6		0x000200 /* arabic */
-#define LANGAV_GBK				0x001000 /* (Chinese) GBK encoding */
-#define LANGAV_LATIN_UTF8		0x002000 /* UTF8: latin script */
-#define LANGAV_CYRILLIC_UTF8	0x004000 /* UTF8: cyrillic script */
-#define LANGAV_GREEK_UTF8		0x008000 /* UTF8: greek script */
-#define LANGAV_HEBREW_UTF8		0x010000 /* UTF8: hebrew script */
-typedef struct _langlist LangList;
-struct _langlist
+#define LANGAV_GBK			0x001000 /* (Chinese) GBK encoding */
+#define LANGAV_UTF8			0x002000 /* any UTF8 encoding */
+#define LANGAV_LATIN_UTF8		0x004000 /* UTF8: latin script */
+#define LANGAV_CYRILLIC_UTF8		0x008000 /* UTF8: cyrillic script */
+#define LANGAV_GREEK_UTF8		0x010000 /* UTF8: greek script */
+#define LANGAV_HEBREW_UTF8		0x020000 /* UTF8: hebrew script */
+#define LANGAV_ARABIC_UTF8		0x040000 /* UTF8: arabic script */
+typedef struct LangList LangList;
+struct LangList
 {
 	char *directive;
 	char *code;
@@ -83,57 +85,61 @@ struct _langlist
 /* MUST be alphabetized (first column) */
 static LangList langlist[] = {
 /*	{ "arabic",       "ara", LANGAV_ASCII|LANGAV_ISO8859_6 }, -- TODO: check if this has issues first! */
-	{ "belarussian-utf8", "blr-utf8", LANGAV_ASCII|LANGAV_CYRILLIC_UTF8 },
+	{ "arabic-utf8", "ara-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_ARABIC_UTF8 },
+	{ "belarussian-utf8", "blr-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_CYRILLIC_UTF8 },
 	{ "belarussian-w1251", "blr", LANGAV_ASCII|LANGAV_W1251 },
 	{ "catalan",      "cat", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "catalan-utf8", "cat-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "catalan-utf8", "cat-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "chinese",      "chi-j,chi-s,chi-t", LANGAV_GBK },
 	{ "chinese-ja",   "chi-j", LANGAV_GBK },
 	{ "chinese-simp", "chi-s", LANGAV_GBK },
 	{ "chinese-trad", "chi-t", LANGAV_GBK },
-	{ "cyrillic-utf8", "blr-utf8,rus-utf8,ukr-utf8", LANGAV_ASCII|LANGAV_CYRILLIC_UTF8 },
+	{ "cyrillic-utf8", "blr-utf8,rus-utf8,ukr-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_CYRILLIC_UTF8 },
 	{ "czech",        "cze-m", LANGAV_ASCII|LANGAV_W1250 },
-	{ "czech-utf8",   "cze-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "czech-utf8",   "cze-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "danish",       "dan", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "danish-utf8",  "dan-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "danish-utf8",  "dan-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "dutch",        "dut", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "dutch-utf8",   "dut-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "dutch-utf8",   "dut-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
+	{ "estonian-utf8","est-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "french",       "fre", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "french-utf8",  "fre-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "french-utf8",  "fre-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "gbk",          "chi-s,chi-t,chi-j", LANGAV_GBK },
 	{ "german",       "ger", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "german-utf8",  "ger-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "german-utf8",  "ger-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "greek",        "gre", LANGAV_ASCII|LANGAV_ISO8859_7 },
-	{ "greek-utf8",   "gre-utf8", LANGAV_ASCII|LANGAV_GREEK_UTF8 },
+	{ "greek-utf8",   "gre-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_GREEK_UTF8 },
 	{ "hebrew",       "heb", LANGAV_ASCII|LANGAV_ISO8859_8I },
-	{ "hebrew-utf8",  "heb", LANGAV_ASCII|LANGAV_HEBREW_UTF8 },
+	{ "hebrew-utf8",  "heb-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_HEBREW_UTF8 },
 	{ "hungarian",    "hun", LANGAV_ASCII|LANGAV_LATIN2W1250 },
-	{ "hungarian-utf8","hun-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "hungarian-utf8","hun-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "icelandic",    "ice", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "icelandic-utf8","ice-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "icelandic-utf8","ice-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "italian",      "ita", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "italian-utf8", "ita-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
-	{ "latin-utf8",   "cat-utf8,cze-utf8,dan-utf8,dut-utf8,fre-utf8,ger-utf8,hun-utf8,ice-utf8,ita-utf8,pol-utf8,rum-utf8,slo-utf8,spa-utf8,swe-utf8,tur-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "italian-utf8", "ita-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
+	{ "latin-utf8",   "cat-utf8,cze-utf8,dan-utf8,dut-utf8,fre-utf8,ger-utf8,hun-utf8,ice-utf8,ita-utf8,pol-utf8,rum-utf8,slo-utf8,spa-utf8,swe-utf8,tur-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "latin1",       "cat,dut,fre,ger,ita,spa,swe", LANGAV_ASCII|LANGAV_LATIN1 },
 	{ "latin2",       "hun,pol,rum", LANGAV_ASCII|LANGAV_LATIN2 },
+	{ "latvian-utf8", "lav-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
+	{ "lithuanian-utf8","lit-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "polish",       "pol", LANGAV_ASCII|LANGAV_LATIN2 },
-	{ "polish-utf8",  "pol-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "polish-utf8",  "pol-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "polish-w1250", "pol-m", LANGAV_ASCII|LANGAV_W1250 },
 	{ "romanian",     "rum", LANGAV_ASCII|LANGAV_LATIN2W1250 },
-	{ "romanian-utf8","rum-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
-	{ "russian-utf8", "rus-utf8", LANGAV_ASCII|LANGAV_CYRILLIC_UTF8 },
+	{ "romanian-utf8","rum-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
+	{ "russian-utf8", "rus-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_CYRILLIC_UTF8 },
 	{ "russian-w1251","rus", LANGAV_ASCII|LANGAV_W1251 },
 	{ "slovak",       "slo-m", LANGAV_ASCII|LANGAV_W1250 },
-	{ "slovak-utf8",  "slo-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "slovak-utf8",  "slo-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "spanish",      "spa", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "spanish-utf8", "spa-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "spanish-utf8", "spa-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "swedish",      "swe", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "swedish-utf8", "swe-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "swedish-utf8", "swe-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "swiss-german", "swg", LANGAV_ASCII|LANGAV_LATIN1 },
-	{ "swiss-german-utf8", "swg-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
+	{ "swiss-german-utf8", "swg-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
 	{ "turkish",      "tur", LANGAV_ASCII|LANGAV_ISO8859_9 },
-	{ "turkish-utf8", "tur-utf8", LANGAV_ASCII|LANGAV_LATIN_UTF8 },
-	{ "ukrainian-utf8", "ukr-utf8", LANGAV_ASCII|LANGAV_CYRILLIC_UTF8 },
+	{ "turkish-utf8", "tur-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_LATIN_UTF8 },
+	{ "ukrainian-utf8", "ukr-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_CYRILLIC_UTF8 },
 	{ "ukrainian-w1251", "ukr", LANGAV_ASCII|LANGAV_W1251 },
 	{ "windows-1250", "cze-m,pol-m,rum,slo-m,hun",  LANGAV_ASCII|LANGAV_W1250 },
 	{ "windows-1251", "rus,ukr,blr", LANGAV_ASCII|LANGAV_W1251 },
@@ -141,8 +147,8 @@ static LangList langlist[] = {
 };
 
 /* For temporary use during config_run */
-typedef struct _ilanglist ILangList;
-struct _ilanglist
+typedef struct ILangList ILangList;
+struct ILangList
 {
 	ILangList *prev, *next;
 	char *name;
@@ -180,7 +186,7 @@ int charsys_config_run(ConfigFile *cf, ConfigEntry *ce, int type);
 int charsys_config_posttest(int *errs);
 char *_charsys_get_current_languages(void);
 
-MOD_TEST(charsys)
+MOD_TEST()
 {
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	EfunctionAdd(modinfo->handle, EFUNC_DO_NICK_NAME, _do_nick_name);
@@ -193,7 +199,7 @@ MOD_TEST(charsys)
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(charsys)
+MOD_INIT()
 {
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, charsys_config_run);
@@ -201,14 +207,14 @@ MOD_INIT(charsys)
 }
 
 /* Is first run when server is 100% ready */
-MOD_LOAD(charsys)
+MOD_LOAD()
 {
 	charsys_finish();
 	return MOD_SUCCESS;
 }
 
 /* Called when module is unloaded */
-MOD_UNLOAD(charsys)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
@@ -256,7 +262,7 @@ int charsys_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 
 	if (type != CONFIG_SET)
 		return 0;
-	
+
 	/* We are only interrested in set::allowed-nickchars... */
 	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "allowed-nickchars"))
 		return 0;
@@ -309,6 +315,8 @@ int charsys_config_posttest(int *errs)
 	    x++;
 	if (x > 1)
 	{
+#if 0
+// I don't think this should be hard error, right? Some combinations may be problematic, but not all.
 		if (langav & LANGAV_LATIN_UTF8)
 		{
 			config_error("ERROR: set::allowed-nickchars: you cannot combine 'latin-utf8' with any other character set");
@@ -329,8 +337,13 @@ int charsys_config_posttest(int *errs)
 			config_error("ERROR: set::allowed-nickchars: you cannot combine 'hebrew-utf8' with any other character set");
 			errors++;
 		}
-		config_status("WARNING: set::allowed-nickchars: "
-		            "Mixing of charsets (eg: latin1+latin2) can cause display problems");
+		if (langav & LANGAV_ARABIC_UTF8)
+		{
+			config_error("ERROR: set::allowed-nickchars: you cannot combine 'arabic-utf8' with any other character set");
+			errors++;
+		}
+#endif
+		config_status("WARNING: set::allowed-nickchars: Mixing of charsets (eg: latin1+latin2) may cause display problems");
 	}
 
 	*errs = errors;
@@ -349,7 +362,7 @@ void charsys_reset(void)
 	for (m=mblist; m; m=m_next)
 	{
 		m_next = m->next;
-		MyFree(m);
+		safe_free(m);
 	}
 	mblist=mblist_tail=NULL;
 	/* Then add the default which will always be allowed */
@@ -365,6 +378,7 @@ void charsys_reset(void)
 void charsys_reset_pretest(void)
 {
 	langav = 0;
+	non_utf8_nick_chars_in_use = 0;
 }
 
 static inline void ilang_swap(ILangList *one, ILangList *two)
@@ -376,8 +390,7 @@ static inline void ilang_swap(ILangList *one, ILangList *two)
 
 static void ilang_sort(void)
 {
-ILangList *outer, *inner;
-char *tmp;
+	ILangList *outer, *inner;
 
 	/* Selection sort -- perhaps optimize to qsort/whatever if
      * possible? ;)
@@ -394,7 +407,7 @@ char *tmp;
 
 void charsys_finish(void)
 {
-ILangList *e, *e_next;
+	ILangList *e, *e_next;
 
 	/* Sort alphabetically */
 	ilang_sort();
@@ -407,13 +420,13 @@ ILangList *e, *e_next;
 		if (e->next)
 			strlcat(langsinuse, ",", sizeof(langsinuse));
 	}
-	
+
 	/* Free everything */
 	for (e=ilanglist; e; e=e_next)
 	{
 		e_next=e->next;
-		MyFree(e->name);
-		MyFree(e);
+		safe_free(e->name);
+		safe_free(e);
 	}
 	ilanglist = NULL;
 #ifdef DEBUGMODE
@@ -424,15 +437,15 @@ ILangList *e, *e_next;
 }
 
 /** Add a character range to the multibyte list.
+ * Eg: charsys_addmultibyterange(0xaa, 0xbb, 0x00, 0xff) for 0xaa00-0xbbff.
  * @param s1 Start of highest byte
  * @param e1 End of highest byte
  * @param s2 Start of lowest byte
  * @param e2 End of lowest byte
- * @example charsys_addmultibyterange(0xaa, 0xbb, 0x00, 0xff) for 0xaa00-0xbbff
  */
 void charsys_addmultibyterange(char s1, char e1, char s2, char e2)
 {
-MBList *m = MyMallocEx(sizeof(MBList));
+MBList *m = safe_alloc(sizeof(MBList));
 
 	m->s1 = s1;
 	m->e1 = e1;
@@ -480,8 +493,8 @@ int _do_nick_name(char *nick)
 
 static int do_nick_name_standard(char *nick)
 {
-int len;
-char *ch;
+	int len;
+	char *ch;
 
 	if ((*nick == '-') || isdigit(*nick))
 		return 0;
@@ -495,7 +508,7 @@ char *ch;
 
 static int isvalidmbyte(unsigned char c1, unsigned char c2)
 {
-MBList *m;
+	MBList *m;
 
 	for (m=mblist; m; m=m->next)
 	{
@@ -512,10 +525,9 @@ MBList *m;
  */
 static int do_nick_name_multibyte(char *nick)
 {
-int len;
-char *ch;
-MBList *m;
-int firstmbchar = 0;
+	int len;
+	char *ch;
+	int firstmbchar = 0;
 
 	if ((*nick == '-') || isdigit(*nick))
 		return 0;
@@ -536,7 +548,10 @@ int firstmbchar = 0;
 			return 0;
 	}
 	if (firstmbchar)
+	{
 		ch--;
+		len--;
+	}
 	*ch = '\0';
 	return len;
 }
@@ -549,8 +564,13 @@ int firstmbchar = 0;
  */
 int _do_remote_nick_name(char *nick)
 {
-char *c;
+	char *c;
 
+	/* Don't allow nicks to start with a digit, ever. */
+	if ((*nick == '-') || isdigit(*nick))
+		return 0;
+
+	/* Now the other, more relaxed checks.. */
 	for (c=nick; *c; c++)
 		if ((*c <= 32) || strchr(illegalnickchars, *c))
 			return 0;
@@ -560,9 +580,9 @@ char *c;
 
 static LangList *charsys_find_language(char *name)
 {
-int start = 0;
-int stop = ARRAY_SIZEOF(langlist)-1;
-int mid;
+	int start = 0;
+	int stop = ARRAY_SIZEOF(langlist)-1;
+	int mid;
 
 	while (start <= stop)
 	{
@@ -577,14 +597,25 @@ int mid;
 	return NULL;
 }
 
+static LangList *charsys_find_language_code(char *code)
+{
+	int i;
+	for (i = 0; langlist[i].code; i++)
+		if (!strcasecmp(langlist[i].code, code))
+			return &langlist[i];
+	return NULL;
+}
+
 /** Check if language is available. */
 int charsys_test_language(char *name)
 {
-LangList *l = charsys_find_language(name);
+	LangList *l = charsys_find_language(name);
 
 	if (l)
 	{
 		langav |= l->setflags;
+		if (!(l->setflags & LANGAV_UTF8))
+			non_utf8_nick_chars_in_use = 1;
 		return 1;
 	}
 	if (!strcmp(name, "euro-west"))
@@ -625,8 +656,8 @@ char tmp[512], *lang, *p;
 		if (!found)
 		{
 			/* Add... */
-			li = MyMallocEx(sizeof(ILangList));
-			li->name = strdup(lang);
+			li = safe_alloc(sizeof(ILangList));
+			safe_strdup(li->name, lang);
 			AddListItem(li, ilanglist);
 		}
 	}
@@ -664,7 +695,7 @@ void charsys_add_language(char *name)
 		w1251 = 1;
 	else if (!strcmp(name, "chinese") || !strcmp(name, "gbk"))
 		chinese = 1;
-	
+
 	/* INDIVIDUAL CHARSETS */
 
 	/* [LATIN1] and [LATIN-UTF8] */
@@ -821,7 +852,7 @@ void charsys_add_language(char *name)
 	if (latin1 || !strcmp(name, "swedish"))
 	{
 		/* supplied by Tank */
-		/* ao, Ao, a", A", o", O" */ 
+		/* ao, Ao, a", A", o", O" */
 		charsys_addallowed("");
 	}
 	if (latin_utf8 || !strcmp(name, "swedish-utf8"))
@@ -907,7 +938,7 @@ void charsys_add_language(char *name)
 		charsys_addmultibyterange(0xc5, 0xc5, 0x9e, 0x9f);
 		charsys_addmultibyterange(0xc5, 0xc5, 0xa2, 0xa3);
 	}
-	
+
 	if (latin2 || !strcmp(name, "polish"))
 	{
 		/* supplied by k4be */
@@ -1000,7 +1031,7 @@ void charsys_add_language(char *name)
 		charsys_addmultibyterange(0xd1, 0xd1, 0x80, 0x8f);
 		charsys_addmultibyterange(0xd1, 0xd1, 0x91, 0x91);
 	}
-	
+
 	if (w1251 || !strcmp(name, "belarussian-w1251"))
 	{
 		/* supplied by Bock (Samets Anton) & ss:
@@ -1024,7 +1055,7 @@ void charsys_add_language(char *name)
 		charsys_addmultibyterange(0xd1, 0xd1, 0x96, 0x96);
 		charsys_addmultibyterange(0xd1, 0xd1, 0x9e, 0x9e);
 	}
-	
+
 	if (w1251 || !strcmp(name, "ukrainian-w1251"))
 	{
 		/* supplied by Anton Samets & ss:
@@ -1048,7 +1079,7 @@ void charsys_add_language(char *name)
 		charsys_addmultibyterange(0xd2, 0xd2, 0x90, 0x91);
 	}
 
-	/* [GREEK] */	
+	/* [GREEK] */
 	if (!strcmp(name, "greek"))
 	{
 		/* supplied by GSF */
@@ -1116,13 +1147,71 @@ void charsys_add_language(char *name)
 		charsys_addmultibyterange(0xaa, 0xfe, 0x40, 0x7e); /* GBK/4 - lower half */
 		charsys_addmultibyterange(0xaa, 0xfe, 0x80, 0xa0); /* GBK/4 - upper half */
 	}
+
+	/* [LATVIAN] */
+	if (latin_utf8 || !strcmp(name, "latvian-utf8"))
+	{
+		/* A a, C c, E e, G g, I i, K k,  , U u,   */
+		charsys_addmultibyterange(0xc4, 0xc4, 0x80, 0x81);
+		charsys_addmultibyterange(0xc4, 0xc4, 0x92, 0x93);
+		charsys_addmultibyterange(0xc4, 0xc4, 0x8c, 0x8d);
+		charsys_addmultibyterange(0xc4, 0xc4, 0x92, 0x93);
+		charsys_addmultibyterange(0xc4, 0xc4, 0xa2, 0xa3);
+		charsys_addmultibyterange(0xc4, 0xc4, 0xaa, 0xab);
+		charsys_addmultibyterange(0xc4, 0xc4, 0xb6, 0xb7);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xa0, 0xa1);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xaa, 0xab);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xbd, 0xbe);
+	}
+
+	/* [ESTONIAN] */
+	if (latin_utf8 || !strcmp(name, "estonian-utf8"))
+	{
+		/* , , , ,  , , ,  */
+		charsys_addmultibyterange(0xc3, 0xc3, 0xb5, 0xb6);
+		charsys_addmultibyterange(0xc3, 0xc3, 0xa4, 0xa4);
+		charsys_addmultibyterange(0xc3, 0xc3, 0xbc, 0xbc);
+		charsys_addmultibyterange(0xc3, 0xc3, 0x95, 0x96);
+		charsys_addmultibyterange(0xc3, 0xc3, 0x84, 0x84);
+		charsys_addmultibyterange(0xc3, 0xc3, 0x9c, 0x9c);
+	}
+
+	/* [LITHUANIAN] */
+	if (latin_utf8 || !strcmp(name, "lithuanian-utf8"))
+	{
+		/* a, c, e, e, i, , u, u, , A, C, E, E, I, , U, U,  */
+		charsys_addmultibyterange(0xc4, 0xc4, 0x84, 0x85);
+		charsys_addmultibyterange(0xc4, 0xc4, 0x8c, 0x8d);
+		charsys_addmultibyterange(0xc4, 0xc4, 0x96, 0x99);
+		charsys_addmultibyterange(0xc4, 0xc4, 0xae, 0xaf);
+		charsys_addmultibyterange(0xc4, 0xc4, 0xae, 0xaf);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xa0, 0xa1);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xb2, 0xb3);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xaa, 0xab);
+		charsys_addmultibyterange(0xc5, 0xc5, 0xbd, 0xbe);
+	}
+
+	/* [ARABIC] */
+	if (latin_utf8 || !strcmp(name, "arabic-utf8"))
+	{
+		/* Supplied by Sensiva */
+		/*charsys_addallowed("اأإآءبتثجحخدذرزسشصضطظعغفقكلمنهؤةويىئ");*/
+		/*- From U+0621 to U+063A (Regex: [\u0621-\u063A])*/
+		/* 0xd8a1 - 0xd8ba */
+		charsys_addmultibyterange(0xd8, 0xd8, 0xa1, 0xba);
+		/*- From U+0641 to U+064A (Regex: [\u0641-\u064A])*/
+		/* 0xd981 - 0xd98a */
+		charsys_addmultibyterange(0xd9, 0xd9, 0x81, 0x8a);
+	}
 }
 
 /** This displays all the nick characters that are permitted */
 char *charsys_displaychars(void)
 {
+#if 0
 	MBList *m;
 	unsigned char hibyte, lobyte;
+#endif
 	static char buf[512];
 	int n = 0;
 	int i, j;
@@ -1169,7 +1258,7 @@ char *charsys_displaychars(void)
 	}
 
 	buf[n] = '\0'; /* there's always room for a NUL */
-	
+
 	return buf;
 }
 
@@ -1183,7 +1272,9 @@ char *charsys_group(int v)
 		return "Greek script";
 	if (v & LANGAV_HEBREW_UTF8)
 		return "Hebrew script";
-	
+	if (v & LANGAV_ARABIC_UTF8)
+		return "Arabic script";
+
 	return "Other";
 }
 
@@ -1194,8 +1285,8 @@ void charsys_dump_table(char *filter)
 	for (i = 0; langlist[i].directive; i++)
 	{
 		char *charset = langlist[i].directive;
-		
-		if (_match(filter, charset))
+
+		if (!match_simple(filter, charset))
 			continue; /* skip */
 
 		charsys_reset();
