@@ -70,22 +70,24 @@
 #undef BENCHMARK
 
 
-ModuleHeader MOD_HEADER(textban)
+ModuleHeader MOD_HEADER
   = {
-	"textban",
-	"v2.2",
+	"extbans/textban",
+	"2.2",
 	"ExtBan ~T (textban) by Syzop",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 /* Forward declarations */
 char *extban_modeT_conv_param(char *para_in);
-int extban_modeT_is_banned(aClient *sptr, aChannel *chptr, char *ban, int type);
-int extban_modeT_is_ok(aClient *sptr, aChannel *chptr, char *para, int checkt, int what, int what2);
-char *textban_chanmsg(aClient *, aChannel *, char *, int);
+int textban_check_ban(Client *client, Channel *channel, char *ban, char **msg, char **errmsg);
+int textban_can_send_to_channel(Client *client, Channel *channel, Membership *lp, char **msg, char **errmsg, SendType sendtype);
+int extban_modeT_is_banned(Client *client, Channel *channel, char *ban, int type, char **msg, char **errmsg);
+int extban_modeT_is_ok(Client *client, Channel *channel, char *para, int checkt, int what, int what2);
+void parse_word(const char *s, char **word, int *type);
 
-MOD_INIT(textban)
+MOD_INIT()
 {
 	ExtbanInfo req;
 
@@ -93,45 +95,51 @@ MOD_INIT(textban)
 
 	memset(&req, 0, sizeof(ExtbanInfo));
 	req.flag = 'T';
+	req.options = EXTBOPT_NOSTACKCHILD; /* disallow things like ~n:~T, as we only affect text. */
 	req.conv_param = extban_modeT_conv_param;
 	req.is_banned = extban_modeT_is_banned;
 	req.is_ok = extban_modeT_is_ok;
-	
+
 	if (!ExtbanAdd(modinfo->handle, req))
 	{
 		config_error("textban module: adding extban ~T failed! module NOT loaded");
 		return MOD_FAILED;
 	}
 
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_CHANMSG, 0, textban_chanmsg);
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_CHANNEL, 0, textban_can_send_to_channel);
 
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(textban)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(textban)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
 #if defined(CENSORFEATURE) || defined(STRIPFEATURE)
-static char *my_strcasestr(char *haystack, char *needle) {
-int i;
-int nlength = strlen (needle);
-int hlength = strlen (haystack);
+static char *my_strcasestr(char *haystack, char *needle)
+{
+	int i;
+	int nlength = strlen (needle);
+	int hlength = strlen (haystack);
 
-	if (nlength > hlength) return NULL;
-	if (hlength <= 0) return NULL;
-	if (nlength <= 0) return haystack;
-	for (i = 0; i <= (hlength - nlength); i++) {
+	if (nlength > hlength)
+		return NULL;
+	if (hlength <= 0)
+		return NULL;
+	if (nlength <= 0)
+		return haystack;
+	for (i = 0; i <= (hlength - nlength); i++)
+	{
 		if (strncasecmp (haystack + i, needle, nlength) == 0)
 			return haystack + i;
 	}
-  return NULL; /* not found */
+	return NULL; /* not found */
 }
 
 #define TEXTBAN_WORD_LEFT	0x1
@@ -145,26 +153,28 @@ int hlength = strlen (haystack);
  */
 int textban_replace(int type, char *badword, char *line, char *buf)
 {
-char *replacew;
-char *pold = line, *pnew = buf; /* Pointers to old string and new string */
-char *poldx = line;
-int replacen;
-int searchn = -1;
-char *startw, *endw;
-char *c_eol = buf + 510 - 1; /* Cached end of (new) line */
-int cleaned = 0;
+	char *replacew;
+	char *pold = line, *pnew = buf; /* Pointers to old string and new string */
+	char *poldx = line;
+	int replacen;
+	int searchn = -1;
+	char *startw, *endw;
+	char *c_eol = buf + 510 - 1; /* Cached end of (new) line */
+	int cleaned = 0;
 
 	replacew = CENSORWORD;
 	replacen = sizeof(CENSORWORD)-1;
 
-	while (1) {
+	while (1)
+	{
 		pold = my_strcasestr(pold, badword);
 		if (!pold)
 			break;
 		if (searchn == -1)
 			searchn = strlen(badword);
 		/* Hunt for start of word */
- 		if (pold > line) {
+ 		if (pold > line)
+ 		{
 			for (startw = pold; (!iswseperator(*startw) && (startw != line)); startw--);
 			if (iswseperator(*startw))
 				startw++; /* Don't point at the space/seperator but at the word! */
@@ -172,7 +182,8 @@ int cleaned = 0;
 			startw = pold;
 		}
 
-		if (!(type & TEXTBAN_WORD_LEFT) && (pold != startw)) {
+		if (!(type & TEXTBAN_WORD_LEFT) && (pold != startw))
+		{
 			/* not matched */
 			pold++;
 			continue;
@@ -184,18 +195,21 @@ int cleaned = 0;
 		 */
 		for (endw = pold+searchn; ((*endw != '\0') && (!iswseperator(*endw))); endw++);
 
-		if (!(type & TEXTBAN_WORD_RIGHT) && (pold+searchn != endw)) {
+		if (!(type & TEXTBAN_WORD_RIGHT) && (pold+searchn != endw))
+		{
 			/* not matched */
 			pold++;
 			continue;
 		}
 
 		cleaned = 1; /* still too soon? Syzop/20050227 */
-		
+
 		/* Do we have any not-copied-yet data? */
-		if (poldx != startw) {
+		if (poldx != startw)
+		{
 			int tmp_n = startw - poldx;
-			if (pnew + tmp_n >= c_eol) {
+			if (pnew + tmp_n >= c_eol)
+			{
 				/* Partial copy and return... */
 				memcpy(pnew, poldx, c_eol - pnew);
 				*c_eol = '\0';
@@ -207,8 +221,10 @@ int cleaned = 0;
 		}
 		/* Now update the word in buf (pnew is now something like startw-in-new-buffer */
 
-		if (replacen) {
-			if ((pnew + replacen) >= c_eol) {
+		if (replacen)
+		{
+			if ((pnew + replacen) >= c_eol)
+			{
 				/* Partial copy and return... */
 				memcpy(pnew, replacew, c_eol - pnew);
 				*c_eol = '\0';
@@ -220,7 +236,8 @@ int cleaned = 0;
 		poldx = pold = endw;
 	}
 	/* Copy the last part */
-	if (*poldx) {
+	if (*poldx)
+	{
 		strncpy(pnew, poldx, c_eol - pnew);
 		*(c_eol) = '\0';
 	} else {
@@ -230,49 +247,64 @@ int cleaned = 0;
 }
 #endif
 
-unsigned int counttextbans(aChannel *chptr)
+unsigned int counttextbans(Channel *channel)
 {
-Ban *ban;
-unsigned int cnt = 0;
+	Ban *ban;
+	unsigned int cnt = 0;
 
-	for (ban = chptr->banlist; ban; ban=ban->next)
+	for (ban = channel->banlist; ban; ban=ban->next)
 		if ((ban->banstr[0] == '~') && (ban->banstr[1] == 'T') && (ban->banstr[2] == ':'))
 			cnt++;
-	for (ban = chptr->exlist; ban; ban=ban->next)
+	for (ban = channel->exlist; ban; ban=ban->next)
 		if ((ban->banstr[0] == '~') && (ban->banstr[1] == 'T') && (ban->banstr[2] == ':'))
 			cnt++;
 	return cnt;
 }
 
 
-int extban_modeT_is_ok(aClient *sptr, aChannel *chptr, char *para, int checkt, int what, int what2)
+int extban_modeT_is_ok(Client *client, Channel *channel, char *para, int checkt, int what, int what2)
 {
-int n;
+	int n;
 
-	if ((what == MODE_ADD) && (what2 == EXBTYPE_EXCEPT) && MyClient(sptr))
+	if ((what == MODE_ADD) && (what2 == EXBTYPE_EXCEPT) && MyUser(client))
 		return 0; /* except is not supported */
 
 	/* We check the # of bans in the channel, may not exceed MAX_EXTBANT_PER_CHAN */
 	if ((what == MODE_ADD) && (checkt == EXBCHK_PARAM) &&
-	     MyClient(sptr) && !IsOper(sptr) &&
-	    ((n = counttextbans(chptr)) >= MAX_EXTBANT_PER_CHAN))
+	     MyUser(client) && !IsOper(client) &&
+	    ((n = counttextbans(channel)) >= MAX_EXTBANT_PER_CHAN))
 	{
 		/* We check the # of bans in the channel, may not exceed MAX_EXTBANT_PER_CHAN */
-		sendto_one(sptr, err_str(ERR_BANLISTFULL), me.name, sptr->name, chptr->chname, para);
-		sendnotice(sptr, "Too many textbans for this channel");
+		sendnumeric(client, ERR_BANLISTFULL, channel->chname, para);
+		sendnotice(client, "Too many textbans for this channel");
 		return 0;
 	}
 	return 1;
 }
 
+char *conv_pattern_asterisks(const char *pattern)
+{
+	static char buf[512];
+	char missing_prefix = 0, missing_suffix = 0;
+	if (*pattern != '*')
+		missing_prefix = 1;
+	if (*pattern && (pattern[strlen(pattern)-1] != '*'))
+		missing_suffix = 1;
+	snprintf(buf, sizeof(buf), "%s%s%s",
+		missing_prefix ? "*" : "",
+		pattern,
+		missing_suffix ? "*" : "");
+	return buf;
+}
+
 /** Ban callbacks */
 char *extban_modeT_conv_param(char *para_in)
 {
-static char retbuf[MAX_LENGTH+1];
-char para[MAX_LENGTH+1], *action, *text, *p;
+	static char retbuf[MAX_LENGTH+1];
+	char para[MAX_LENGTH+1], *action, *text, *p;
 #ifdef UHOSTFEATURE
-char *uhost;
-int ap = 0;
+	char *uhost;
+	int ap = 0;
 #endif
 
 	strlcpy(para, para_in+3, sizeof(para)); /* work on a copy (and truncate it) */
@@ -293,7 +325,7 @@ int ap = 0;
 		return NULL;
 	*text++ = '\0';
 	uhost = para;
-	
+
 	for (p = uhost; *p; p++)
 	{
 		if (*p == '@')
@@ -313,10 +345,13 @@ int ap = 0;
 		return NULL; /* empty text */
 	action = para;
 #endif
-	
+
 	/* ~T:<action>:<text> */
 	if (!strcasecmp(action, "block"))
+	{
 		action = "block"; /* ok */
+		text = conv_pattern_asterisks(text);
+	}
 #ifdef CENSORFEATURE
 	else if (!strcasecmp(action, "censor"))
 	{
@@ -334,34 +369,166 @@ int ap = 0;
 
 	/* check the string.. */
 	for (p=text; *p; p++)
+	{
 		if ((*p == '\003') || (*p == '\002') || 
 		    (*p == '\037') || (*p == '\026') ||
 		    (*p == ' '))
+		{
 			return NULL; /* codes not permitted, would be confusing since they are stripped */
+		}
+	}
 
-	/* Rebuild the string.. */
+	/* Rebuild the string.. can be cut off if too long. */
 #ifdef UHOSTFEATURE
-	snprintf(retbuf, sizeof(retbuf), "~T:%s:%s:%s", uhost, action, text); /* can be cut off if too long */
+	snprintf(retbuf, sizeof(retbuf), "~T:%s:%s:%s", uhost, action, text);
 #else
-	snprintf(retbuf, sizeof(retbuf), "~T:%s:%s", action, text); /* can be cut off if too long */
+	snprintf(retbuf, sizeof(retbuf), "~T:%s:%s", action, text);
 #endif
 	return retbuf;
 }
 
-int extban_modeT_is_banned(aClient *sptr, aChannel *chptr, char *ban, int type)
+/** This is the regular "is banned?" routine. We can't use this as we need to be called for voiced users as well */
+int extban_modeT_is_banned(Client *client, Channel *channel, char *ban, int checktype, char **msg, char **errmsg)
 {
-	/* Never banned here */
 	return 0;
 }
 
+/** Check for text bans (censor and block) */
+int textban_can_send_to_channel(Client *client, Channel *channel, Membership *lp, char **msg, char **errmsg, SendType sendtype)
+{
+	Ban *ban;
+
+	/* +h/+o/+a/+q users bypass textbans */
+	if (is_skochanop(client, channel))
+		return HOOK_CONTINUE;
+
+	/* IRCOps with these privileges bypass textbans too */
+	if (op_can_override("channel:override:message:ban", client, channel, NULL))
+		return HOOK_CONTINUE;
+
+	/* Now we have to manually walk the banlist and check if things match */
+	for (ban = channel->banlist; ban; ban=ban->next)
+	{
+		if (!strncmp(ban->banstr, "~T:", 3))
+		{
+			/* ~T ban */
+			if (textban_check_ban(client, channel, ban->banstr, msg, errmsg))
+				return HOOK_DENY;
+		} else
+		if (!strncmp(ban->banstr, "~t:", 3))
+		{
+			/* Stacked ~t:xx:~T ban (timed text ban) */
+			char *p = strchr(ban->banstr+3, ':');
+			if (p && !strncmp(p+1, "~T:", 3))
+			{
+				if (textban_check_ban(client, channel, p+1, msg, errmsg))
+					return HOOK_DENY;
+			}
+		}
+	}
+
+	return HOOK_CONTINUE;
+}
+
+
+int textban_check_ban(Client *client, Channel *channel, char *ban, char **msg, char **errmsg)
+{
+	static char retbuf[512];
+	char filtered[512]; /* temp input buffer */
+	long fl;
+	int cleaned=0;
+	char *p;
+#ifdef UHOSTFEATURE
+	char buf[512], uhost[USERLEN + HOSTLEN + 16];
+#endif
+	char tmp[1024], *word;
+	int type;
+#ifdef BENCHMARK
+	struct timeval tv_alpha, tv_beta;
+
+	gettimeofday(&tv_alpha, NULL);
+#endif
+
+	/* We can only filter on non-NULL text of course */
+	if ((msg == NULL) || (*msg == NULL))
+		return 0;
+
+	filtered[0] = '\0'; /* NOT needed, but... :P */
+
+#ifdef UHOSTFEATURE
+	ircsprintf(uhost, "%s@%s", client->user->username, GetHost(client));
+#endif
+	strlcpy(filtered, StripControlCodes(*msg), sizeof(filtered));
+
+	p = ban + 3;
+#ifdef UHOSTFEATURE
+	/* First.. deal with userhost... */
+	strcpy(buf, p);
+	p = strchr(buf, ':');
+	if (!p)
+		return 0; /* invalid format */
+	*p++ = '\0';
+
+	if (match_simple(buf, uhost))
+#else
+	if (1)
+#endif
+	{
+		if (!strncasecmp(p, "block:", 6))
+		{
+			if (match_simple(p+6, filtered))
+			{
+				if (errmsg)
+					*errmsg = "Message blocked due to a text ban";
+				return 1; /* BLOCK */
+			}
+		}
+#ifdef CENSORFEATURE
+		else if (!strncasecmp(p, "censor:", 7))
+		{
+			parse_word(p+7, &word, &type);
+			if (textban_replace(type, word, filtered, tmp))
+			{
+				strlcpy(filtered, tmp, sizeof(filtered));
+				cleaned = 1;
+			}
+		}
+#endif
+	}
+
+#ifdef BENCHMARK
+	gettimeofday(&tv_beta, NULL);
+	ircd_log(LOG_ERROR, "TextBan Timing: %ld microseconds (%s / %s / %d)",
+		((tv_beta.tv_sec - tv_alpha.tv_sec) * 1000000) + (tv_beta.tv_usec - tv_alpha.tv_usec),
+		client->name, channel->chname, strlen(*msg));
+#endif
+
+	if (cleaned)
+	{
+		/* check for null string */
+		char *p;
+		for (p = filtered; *p; p++)
+		{
+			if (*p != ' ')
+			{
+				strlcpy(retbuf, filtered, sizeof(retbuf));
+				*msg = retbuf;
+				return 0; /* allow through, but filtered */
+			}
+		}
+		return 1; /* nothing but spaces found.. */
+	}
+	return 0; /* nothing blocked */
+}
+
 #ifdef CENSORFEATURE
 void parse_word(const char *s, char **word, int *type)
 {
-static char buf[512];
-const char *tmp;
-int len;
-int tpe = 0;
-char *o = buf;
+	static char buf[512];
+	const char *tmp;
+	int len;
+	int tpe = 0;
+	char *o = buf;
 
 	for (tmp = s; *tmp; tmp++)
 	{
@@ -376,107 +543,8 @@ char *o = buf;
 		}
 	}
 	*o = '\0';
-	
+
 	*word = buf;
 	*type = tpe;
 }
 #endif
-
-/* Channel message callback */
-char *textban_chanmsg(aClient *sptr, aChannel *chptr, char *text, int notice)
-{
-static char filtered[512]; /* temp buffer */
-Ban *ban;
-long fl;
-int done=0, cleaned=0;
-char *p;
-#ifdef UHOSTFEATURE
-char buf[512], uhost[USERLEN + HOSTLEN + 16];
-#endif
-char tmp[1024], *word;
-int type;
-#ifdef BENCHMARK
-struct timeval tv_alpha, tv_beta;
-
-	gettimeofday(&tv_alpha, NULL);
-#endif
-
-	if (!MyClient(sptr))
-		return text; /* Remote and servers are not affected */
-
-	fl = get_access(sptr, chptr);
-	if (fl & (CHFL_HALFOP|CHFL_CHANOP|CHFL_CHANPROT|CHFL_CHANOWNER))
-		return text; /* halfop or higher */
-
-	filtered[0] = '\0'; /* NOT needed, but... :P */
-
-#ifdef UHOSTFEATURE
-	ircsprintf(uhost, "%s@%s", sptr->user->username, GetHost(sptr));
-#endif
-
-	for (ban = chptr->banlist; ban; ban=ban->next)
-	{
-		if ((ban->banstr[0] == '~') && (ban->banstr[1] == 'T') && (ban->banstr[2] == ':'))
-		{
-			if (!done)
-			{
-				/* Prepare the text [done here, to avoid useless CPU time] */
-				strlcpy(filtered, StripControlCodes(text), sizeof(filtered));
-				done = 1;
-			}
-			p = ban->banstr + 3;
-#ifdef UHOSTFEATURE
-			/* First.. deal with userhost... */
-			strcpy(buf, p);
-			p = strchr(buf, ':');
-			if (!p) continue;
-			*p++ = '\0';
-			if (!_match(buf, uhost))
-#else
-			if (1)
-#endif
-			{
-				if (!strncasecmp(p, "block:", 6))
-				{
-					if (!_match(p+6, filtered))
-					{
-						sendto_one(sptr, err_str(ERR_CANNOTSENDTOCHAN),
-							me.name, sptr->name, chptr->chname,
-							"Message blocked due to a text ban", chptr->chname);
-						return NULL;
-					}
-				}
-#ifdef CENSORFEATURE
-				else if (!strncasecmp(p, "censor:", 7))
-				{
-					parse_word(p+7, &word, &type);
-					if (textban_replace(type, word, filtered, tmp))
-					{
-						strlcpy(filtered, tmp, sizeof(filtered));
-						cleaned = 1;
-					}
-				}
-#endif
-			}
-		}
-	}
-
-#ifdef BENCHMARK
-	gettimeofday(&tv_beta, NULL);
-	ircd_log(LOG_ERROR, "TextBan Timing: %ld microseconds (%s / %s / %d)",
-		((tv_beta.tv_sec - tv_alpha.tv_sec) * 1000000) + (tv_beta.tv_usec - tv_alpha.tv_usec),
-		sptr->name, chptr->chname, strlen(text));
-#endif
-
-	if (cleaned)
-	{
-		/* check for null string */
-		char *p;
-		for (p = filtered; *p; p++)
-			if (*p != ' ')
-				return filtered;
-		return NULL; /* nothing but spaces found.. */
-	}
-	else
-		return text;
-}
