@@ -43,13 +43,13 @@ static ModuleInfo	*MyModInfo;
 #define MyMod		MyModInfo->handle
 #define SAVE_MODINFO	MyModInfo = modinfo;
 
-ModuleHeader MOD_HEADER(hideserver)
+ModuleHeader MOD_HEADER
   = {
 	"hideserver",
-	"v5.0",
+	"5.0",
 	"Hide servers from /MAP & /LINKS",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 static void InitConf()
@@ -61,19 +61,19 @@ static void FreeConf()
 {
 	ConfigItem_ulines	*h, *next;
 
-	safefree(Settings.map_deny_message);
-	safefree(Settings.links_deny_message);
+	safe_free(Settings.map_deny_message);
+	safe_free(Settings.links_deny_message);
 
 	for (h = HiddenServers; h; h = next)
 	{
 		next = h->next;
 		DelListItem(h, HiddenServers);
-		MyFree(h->servername);
-		MyFree(h);
+		safe_free(h->servername);
+		safe_free(h);
 	}
 }
 
-MOD_TEST(hideserver)
+MOD_TEST()
 {
 	SAVE_MODINFO
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, cb_test);
@@ -81,7 +81,7 @@ MOD_TEST(hideserver)
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(hideserver)
+MOD_INIT()
 {
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	SAVE_MODINFO
@@ -93,18 +93,18 @@ MOD_INIT(hideserver)
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(hideserver)
+MOD_LOAD()
 {
-	if (!CmdoverrideAdd(MyMod, "MAP", override_map))
+	if (!CommandOverrideAdd(MyMod, "MAP", override_map))
 		return MOD_FAILED;
 
-	if (!CmdoverrideAdd(MyMod, "LINKS", override_links))
+	if (!CommandOverrideAdd(MyMod, "LINKS", override_links))
 		return MOD_FAILED;
 
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(hideserver)
+MOD_UNLOAD()
 {
 	FreeConf();
 
@@ -113,8 +113,8 @@ MOD_UNLOAD(hideserver)
 
 static int cb_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 {
-	ConfigEntry	*cep, *cepp;
-	int		errors = 0;
+	ConfigEntry *cep;
+	int errors = 0;
 
 	if (type == CONFIG_MAIN)
 	{
@@ -175,11 +175,11 @@ static int cb_conf(ConfigFile *cf, ConfigEntry *ce, int type)
 					Settings.disable_links = config_checkval(cep->ce_vardata, CFG_YESNO);
 				else if (!strcmp(cep->ce_varname, "map-deny-message"))
 				{
-					safestrdup(Settings.map_deny_message, cep->ce_vardata);
+					safe_strdup(Settings.map_deny_message, cep->ce_vardata);
 				}
 				else if (!strcmp(cep->ce_varname, "links-deny-message"))
 				{
-					safestrdup(Settings.links_deny_message, cep->ce_vardata);
+					safe_strdup(Settings.links_deny_message, cep->ce_vardata);
 				}
 				else if (!strcmp(cep->ce_varname, "hide"))
 				{
@@ -188,8 +188,8 @@ static int cb_conf(ConfigFile *cf, ConfigEntry *ce, int type)
 						if (!strcasecmp(cepp->ce_varname, me.name))
 							continue;
 
-						ca = MyMallocEx(sizeof(ConfigItem_ulines));
-						safestrdup(ca->servername, cepp->ce_varname);
+						ca = safe_alloc(sizeof(ConfigItem_ulines));
+						safe_strdup(ca->servername, cepp->ce_varname);
 						AddListItem(ca, HiddenServers);
 					}
 				}
@@ -217,22 +217,21 @@ ConfigItem_ulines *FindHiddenServer(char *servername)
  * New /MAP format -Potvin
  * dump_map function.
  */
-static void dump_map(aClient *cptr, aClient *server, char *mask, int prompt_length, int length)
+static void dump_map(Client *client, Client *server, char *mask, int prompt_length, int length)
 {
 	static char prompt[64];
 	char *p = &prompt[prompt_length];
 	int  cnt = 0;
-	aClient *acptr;
+	Client *acptr;
 
 	*p = '\0';
 
 	if (prompt_length > 60)
-		sendto_one(cptr, rpl_str(RPL_MAPMORE), me.name, cptr->name,
-		    prompt, length, server->name);
+		sendnumeric(client, RPL_MAPMORE, prompt, length, server->name);
 	else
 	{
-		sendto_one(cptr, rpl_str(RPL_MAP), me.name, cptr->name, prompt,
-		    length, server->name, server->serv->users, IsOper(cptr) ? server->id : "");
+		sendnumeric(client, RPL_MAP, prompt,
+		            length, server->name, server->serv->users, IsOper(client) ? server->id : "");
 		cnt = 0;
 	}
 
@@ -250,43 +249,42 @@ static void dump_map(aClient *cptr, aClient *server, char *mask, int prompt_leng
 	list_for_each_entry(acptr, &global_server_list, client_node)
 	{
 		if (acptr->srvptr != server ||
- 		    (IsULine(acptr) && HIDE_ULINES && !ValidatePermissionsForPath("server:info:map:ulines",cptr,NULL,NULL,NULL)))
+ 		    (IsULine(acptr) && HIDE_ULINES && !ValidatePermissionsForPath("server:info:map:ulines",client,NULL,NULL,NULL)))
 			continue;
 		if (FindHiddenServer(acptr->name))
 			break;
-		acptr->flags |= FLAGS_MAP;
+		SetMap(acptr);
 		cnt++;
 	}
 
 	list_for_each_entry(acptr, &global_server_list, client_node)
 	{
-		if (IsULine(acptr) && HIDE_ULINES && !ValidatePermissionsForPath("server:info:map:ulines",cptr,NULL,NULL,NULL))
+		if (IsULine(acptr) && HIDE_ULINES && !ValidatePermissionsForPath("server:info:map:ulines",client,NULL,NULL,NULL))
 			continue;
 		if (FindHiddenServer(acptr->name))
 			break;
 		if (acptr->srvptr != server)
 			continue;
-		if (!(acptr->flags & FLAGS_MAP))
+		if (!IsMap(acptr))
 			continue;
 		if (--cnt == 0)
 			*p = '`';
-		dump_map(cptr, acptr, mask, prompt_length + 2, length - 2);
+		dump_map(client, acptr, mask, prompt_length + 2, length - 2);
 	}
 
 	if (prompt_length > 0)
 		p[-1] = '-';
 }
 
-void dump_flat_map(aClient *cptr, aClient *server, int length)
+void dump_flat_map(Client *client, Client *server, int length)
 {
-char buf[4];
-aClient *acptr;
-int cnt = 0, hide_ulines;
+	char buf[4];
+	Client *acptr;
+	int cnt = 0, hide_ulines;
 
-	hide_ulines = (HIDE_ULINES && !ValidatePermissionsForPath("server:info:map:ulines",cptr,NULL,NULL,NULL)) ? 1 : 0;
+	hide_ulines = (HIDE_ULINES && !ValidatePermissionsForPath("server:info:map:ulines",client,NULL,NULL,NULL)) ? 1 : 0;
 
-	sendto_one(cptr, rpl_str(RPL_MAP), me.name, cptr->name, "",
-	    length, server->name, server->serv->users, "");
+	sendnumeric(client, RPL_MAP, "", length, server->name, server->serv->users, "");
 
 	list_for_each_entry(acptr, &global_server_list, client_node)
 	{
@@ -306,36 +304,37 @@ int cnt = 0, hide_ulines;
 			break;
 		if (--cnt == 0)
 			*buf = '`';
-		sendto_one(cptr, rpl_str(RPL_MAP), me.name, cptr->name, buf,
-		    length-2, acptr->name, acptr->serv->users, "");
+		sendnumeric(client, RPL_MAP, buf, length-2, acptr->name, acptr->serv->users, "");
 	}
 }
 
 /*
 ** New /MAP format. -Potvin
-** m_map (NEW)
+** cmd_map (NEW)
 **
 **      parv[1] = server mask
 **/
 CMD_OVERRIDE_FUNC(override_map)
 {
-	aClient *acptr;
-	int  longest = strlen(me.name);
+	Client *acptr;
+	int longest = strlen(me.name);
 
 	if (parc < 2)
 		parv[1] = "*";
 	
-	if (IsOper(sptr))
-		return CallCmdoverride(ovr, cptr, sptr, parc, parv);
+	if (IsOper(client))
+	{
+		CallCommandOverride(ovr, client, recv_mtags, parc, parv);
+		return;
+	}
 
 	if (Settings.disable_map)
 	{
 		if (Settings.map_deny_message)
-			sendnotice(sptr, "%s", Settings.map_deny_message);
+			sendnotice(client, "%s", Settings.map_deny_message);
 		else
-			sendto_one(sptr, rpl_str(RPL_MAPEND), me.name, sptr->name);
-
-		return 0;
+			sendnumeric(client, RPL_MAPEND);
+		return;
 	}
 
 	list_for_each_entry(acptr, &global_server_list, client_node)
@@ -350,51 +349,48 @@ CMD_OVERRIDE_FUNC(override_map)
 		longest = 60;
 	longest += 2;
 
-	if (FLAT_MAP && !ValidatePermissionsForPath("server:info:map:real-map",sptr,NULL,NULL,NULL))
-		dump_flat_map(sptr, &me, longest);
+	if (FLAT_MAP && !ValidatePermissionsForPath("server:info:map:real-map",client,NULL,NULL,NULL))
+		dump_flat_map(client, &me, longest);
 	else
-		dump_map(sptr, &me, "*", 0, longest);
+		dump_map(client, &me, "*", 0, longest);
 
-	sendto_one(sptr, rpl_str(RPL_MAPEND), me.name, sptr->name);
-
-	return 0;
+	sendnumeric(client, RPL_MAPEND);
 }
 
 CMD_OVERRIDE_FUNC(override_links)
 {
-	aClient *acptr;
-	int flat = (FLAT_MAP && !IsOper(sptr)) ? 1 : 0;
+	Client *acptr;
+	int flat = (FLAT_MAP && !IsOper(client)) ? 1 : 0;
 
-	if (IsOper(sptr))
-		return CallCmdoverride(ovr, cptr, sptr, parc, parv);
+	if (IsOper(client))
+	{
+		CallCommandOverride(ovr, client, recv_mtags, parc, parv);
+		return;
+	}
 
 	if (Settings.disable_links)
 	{
 		if (Settings.links_deny_message)
-			sendnotice(sptr, "%s", Settings.links_deny_message);
+			sendnotice(client, "%s", Settings.links_deny_message);
 		else
-			sendto_one(sptr, rpl_str(RPL_ENDOFLINKS), me.name, sptr->name, "*");
-
-		return 0;
+			sendnumeric(client, RPL_ENDOFLINKS, "*");
+		return;
 	}
 
 	list_for_each_entry(acptr, &global_server_list, client_node)
 	{
 		/* Some checks */
-		if (HIDE_ULINES && IsULine(acptr) && !ValidatePermissionsForPath("server:info:map:ulines",cptr,NULL,NULL,NULL))
+		if (HIDE_ULINES && IsULine(acptr) && !ValidatePermissionsForPath("server:info:map:ulines",client,NULL,NULL,NULL))
 			continue;
 		if (FindHiddenServer(acptr->name))
 			continue;
 		if (flat)
-			sendto_one(sptr, rpl_str(RPL_LINKS),
-			    me.name, sptr->name, acptr->name, me.name,
+			sendnumeric(client, RPL_LINKS, acptr->name, me.name,
 			    1, (acptr->info[0] ? acptr->info : "(Unknown Location)"));
 		else
-			sendto_one(sptr, rpl_str(RPL_LINKS),
-			    me.name, sptr->name, acptr->name, acptr->serv->up,
+			sendnumeric(client, RPL_LINKS, acptr->name, acptr->serv->up,
 			    acptr->hopcount, (acptr->info[0] ? acptr->info : "(Unknown Location)"));
 	}
 
-	sendto_one(sptr, rpl_str(RPL_ENDOFLINKS), me.name, sptr->name, "*");
-	return 0;
+	sendnumeric(client, RPL_ENDOFLINKS, "*");
 }
