@@ -6,13 +6,13 @@
 #include "unrealircd.h"
 
 
-ModuleHeader MOD_HEADER(censor)
+ModuleHeader MOD_HEADER
   = {
 	"usermodes/censor",
 	"4.2",
 	"User Mode +G",
-	"3.2-b8-1",
-	NULL,
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 
@@ -20,7 +20,7 @@ long UMODE_CENSOR = 0L;
 
 #define IsCensored(x) (x->umodes & UMODE_CENSOR)
 
-char *censor_pre_usermsg(aClient *sptr, aClient *target, char *text, int notice);
+int censor_can_send_to_user(Client *client, Client *target, char **text, char **errmsg, SendType sendtype);
 
 int censor_config_test(ConfigFile *, ConfigEntry *, int, int *);
 int censor_config_run(ConfigFile *, ConfigEntry *, int);
@@ -31,34 +31,33 @@ ConfigItem_badword *conf_badword_message = NULL;
 
 static ConfigItem_badword *copy_badword_struct(ConfigItem_badword *ca, int regex, int regflags);
 
+int censor_stats_badwords_user(Client *client, char *para);
 
-MOD_TEST(censor)
+MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, censor_config_test);
 	return MOD_SUCCESS;
 }
 	
-MOD_INIT(censor)
+MOD_INIT()
 {
 	ModInfo = modinfo;
 
 	MARK_AS_OFFICIAL_MODULE(modinfo);
-
 	UmodeAdd(modinfo->handle, 'G', UMODE_GLOBAL, 0, NULL, &UMODE_CENSOR);
-
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_USERMSG, 0, censor_pre_usermsg);
-	
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_USER, 0, censor_can_send_to_user);
+	HookAdd(modinfo->handle, HOOKTYPE_STATS, 0, censor_stats_badwords_user);
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, censor_config_run);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(censor)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
 
-MOD_UNLOAD(censor)
+MOD_UNLOAD()
 {
 ConfigItem_badword *badword, *next;
 
@@ -195,7 +194,7 @@ int censor_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	if (strcmp(ce->ce_vardata, "message") && strcmp(ce->ce_vardata, "all"))
 	        return 0; /* not for us */
 
-	ca = MyMallocEx(sizeof(ConfigItem_badword));
+	ca = safe_alloc(sizeof(ConfigItem_badword));
 	ca->action = BADWORD_REPLACE;
 
 	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
@@ -209,7 +208,7 @@ int censor_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		}
 		else if (!strcmp(cep->ce_varname, "replace"))
 		{
-			safestrdup(ca->replace, cep->ce_vardata);
+			safe_strdup(ca->replace, cep->ce_vardata);
 		}
 		else if (!strcmp(cep->ce_varname, "word"))
 		{
@@ -237,36 +236,36 @@ char *stripbadwords_message(char *str, int *blocked)
 	return stripbadwords(str, conf_badword_message, blocked);
 }
 
-char *censor_pre_usermsg(aClient *sptr, aClient *target, char *text, int notice)
+int censor_can_send_to_user(Client *client, Client *target, char **text, char **errmsg, SendType sendtype)
 {
-int blocked;
+	int blocked = 0;
 
-	if (MyClient(sptr) && IsCensored(target))
+	if (MyUser(client) && IsCensored(target))
 	{
-		text = stripbadwords_message(text, &blocked);
+		*text = stripbadwords_message(*text, &blocked);
 		if (blocked)
 		{
-			if (!notice)
-				sendto_one(sptr, rpl_str(ERR_NOSWEAR),  me.name, sptr->name, target->name);
-			return NULL;
+			*errmsg = "User does not accept private messages containing swearing";
+			return HOOK_DENY;
 		}
 	}
 
-	return text;
+	return HOOK_CONTINUE;
 }
 
-// TODO: when stats is modular, make it call this for badwords
-int stats_badwords(aClient *sptr, char *para)
+int censor_stats_badwords_user(Client *client, char *para)
 {
 	ConfigItem_badword *words;
 
+	if (!para || !(!strcmp(para, "b") || !strcasecmp(para, "badword")))
+		return 0;
+
 	for (words = conf_badword_message; words; words = words->next)
 	{
-		sendto_one(sptr, ":%s %i %s :m %c %s%s%s %s",
-		           me.name, RPL_TEXT, sptr->name, words->type & BADW_TYPE_REGEX ? 'R' : 'F',
+		sendtxtnumeric(client, "m %c %s%s%s %s", words->type & BADW_TYPE_REGEX ? 'R' : 'F',
 		           (words->type & BADW_TYPE_FAST_L) ? "*" : "", words->word,
 		           (words->type & BADW_TYPE_FAST_R) ? "*" : "",
 		           words->action == BADWORD_REPLACE ? (words->replace ? words->replace : "<censored>") : "");
 	}
-	return 0;
+	return 1;
 }
