@@ -19,33 +19,34 @@
 
 #include "unrealircd.h"
 
-#define IsNokick(cptr)    (cptr->umodes & UMODE_NOKICK)
+#define IsNokick(client)    (client->umodes & UMODE_NOKICK)
 
 /* Module header */
-ModuleHeader MOD_HEADER(nokick)
+ModuleHeader MOD_HEADER
   = {
 	"usermodes/nokick",
 	"4.2",
 	"User Mode +q",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 /* Global variables */
 long UMODE_NOKICK = 0L;
 
 /* Forward declarations */
-int nokick_can_kick(aClient *sptr, aClient *target, aChannel *chptr,
-                    char *comment, long sptr_flags, long target_flags, char **reject_reason);
+int umode_allow_unkickable_oper(Client *client, int what);
+int nokick_can_kick(Client *client, Client *target, Channel *channel,
+                    char *comment, long client_flags, long target_flags, char **reject_reason);
 
-MOD_TEST(nokick)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(nokick)
+MOD_INIT()
 {
-	UmodeAdd(modinfo->handle, 'q', UMODE_GLOBAL, 1, umode_allow_opers, &UMODE_NOKICK); // TODO: limit more!!
+	UmodeAdd(modinfo->handle, 'q', UMODE_GLOBAL, 1, umode_allow_unkickable_oper, &UMODE_NOKICK);
 	
 	HookAdd(modinfo->handle, HOOKTYPE_CAN_KICK, 0, nokick_can_kick);
 	
@@ -53,32 +54,44 @@ MOD_INIT(nokick)
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(nokick)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(nokick)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-int nokick_can_kick(aClient *sptr, aClient *target, aChannel *chptr, char *comment,
-                    long sptr_flags, long target_flags, char **reject_reason)
+int umode_allow_unkickable_oper(Client *client, int what)
+{
+	if (MyUser(client))
+	{
+		if (IsOper(client) && ValidatePermissionsForPath("self:unkickablemode",client,NULL,NULL,NULL))
+			return 1;
+		return 0;
+	}
+	/* Always allow remotes: */
+	return 1;
+}
+
+int nokick_can_kick(Client *client, Client *target, Channel *channel, char *comment,
+                    long client_flags, long target_flags, char **reject_reason)
 {
 	static char errmsg[NICKLEN+256];
 
-	if (IsNokick(target) && !IsULine(sptr) && MyClient(sptr) && !ValidatePermissionsForPath("channel:override:kick:nokick",sptr,target,chptr,NULL))
+	if (IsNokick(target) && !IsULine(client) && MyUser(client) && !ValidatePermissionsForPath("channel:override:kick:nokick",client,target,channel,NULL))
 	{
-		ircsnprintf(errmsg, sizeof(errmsg), err_str(ERR_CANNOTDOCOMMAND),
-				   me.name, sptr->name, "KICK",
+		ircsnprintf(errmsg, sizeof(errmsg), ":%s %d %s %s :%s",
+		            me.name, ERR_CANNOTDOCOMMAND, client->name, "KICK",
 				   "user is unkickable (user mode +q)");
 
 		*reject_reason = errmsg;
 
 		sendnotice(target,
 			"*** umode q: %s tried to kick you from channel %s (%s)",
-			sptr->name, chptr->chname, comment);
+			client->name, channel->chname, comment);
 		
 		return EX_ALWAYS_DENY;
 	}
