@@ -19,55 +19,54 @@
 
 #include "unrealircd.h"
 
-#define IsRegOnlyMsg(cptr)    (cptr->umodes & UMODE_REGONLYMSG)
+#define IsRegOnlyMsg(client)    (client->umodes & UMODE_REGONLYMSG)
 
 /* Module header */
-ModuleHeader MOD_HEADER(regonlymsg)
+ModuleHeader MOD_HEADER
   = {
 	"usermodes/regonlymsg",
 	"4.2",
 	"User Mode +R",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 /* Global variables */
 long UMODE_REGONLYMSG = 0L;
 
 /* Forward declarations */
-char *regonlymsg_pre_usermsg(aClient *sptr, aClient *target, char *text, int notice);
+int regonlymsg_can_send_to_user(Client *client, Client *target, char **text, char **errmsg, SendType sendtype);
                     
-MOD_INIT(regonlymsg)
+MOD_INIT()
 {
 	UmodeAdd(modinfo->handle, 'R', UMODE_GLOBAL, 0, umode_allow_all, &UMODE_REGONLYMSG);
 	
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_USERMSG, 0, regonlymsg_pre_usermsg);
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_USER, 0, regonlymsg_can_send_to_user);
 	
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(regonlymsg)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(regonlymsg)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-char *regonlymsg_pre_usermsg(aClient *sptr, aClient *target, char *text, int notice)
+int regonlymsg_can_send_to_user(Client *client, Client *target, char **text, char **errmsg, SendType sendtype)
 {
-	if (IsRegOnlyMsg(target) && !IsServer(sptr) && !IsULine(sptr) && !IsLoggedIn(sptr))
+	if (IsRegOnlyMsg(target) && !IsServer(client) && !IsULine(client) && !IsLoggedIn(client))
 	{
-		if (ValidatePermissionsForPath("client:override:message:regonlymsg",sptr,target,NULL,text))
-			return text; /* TODO: this is actually an override */
+		if (ValidatePermissionsForPath("client:override:message:regonlymsg",client,target,NULL,text))
+			return HOOK_CONTINUE; /* bypass this restriction */
 
-		sendto_one(sptr, err_str(ERR_NONONREG), me.name, sptr->name, target->name);
-
-		return NULL; /* Block the message */
+		*errmsg = "You must identify to a registered nick to private message this user";
+		return HOOK_DENY;
 	}
 
-	return text;
+	return HOOK_CONTINUE;
 }
