@@ -19,38 +19,38 @@
 
 #include "unrealircd.h"
 
-#define IsServiceBot(cptr)    (cptr->umodes & UMODE_SERVICEBOT)
+#define IsServiceBot(client)    (client->umodes & UMODE_SERVICEBOT)
 
 #define WHOIS_SERVICE_STRING ":%s 313 %s %s :is a Network Service"
 
 /* Module header */
-ModuleHeader MOD_HEADER(servicebot)
+ModuleHeader MOD_HEADER
   = {
 	"usermodes/servicebot",
 	"4.2",
-	"User Mode +q",
-	"3.2-b8-1",
-	NULL 
+	"User Mode +S",
+	"UnrealIRCd Team",
+	"unrealircd-5",
     };
 
 /* Global variables */
 long UMODE_SERVICEBOT = 0L;
 
 /* Forward declarations */
-int servicebot_can_kick(aClient *sptr, aClient *target, aChannel *chptr,
-                    char *comment, long sptr_flags, long target_flags, char **reject_reason);
-int servicebot_mode_deop(aClient *sptr, aClient *target, aChannel *chptr,
+int servicebot_can_kick(Client *client, Client *target, Channel *channel,
+                    char *comment, long client_flags, long target_flags, char **reject_reason);
+int servicebot_mode_deop(Client *client, Client *target, Channel *channel,
                     u_int what, int modechar, long my_access, char **reject_reason);
-int servicebot_pre_kill(aClient *sptr, aClient *target, char *reason);
-int servicebot_whois(aClient *sptr, aClient *acptr);
-int servicebot_see_channel_in_whois(aClient *sptr, aClient *target, aChannel *chptr);
+int servicebot_pre_kill(Client *client, Client *target, char *reason);
+int servicebot_whois(Client *requester, Client *acptr);
+int servicebot_see_channel_in_whois(Client *client, Client *target, Channel *channel);
                     
-MOD_TEST(servicebot)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(servicebot)
+MOD_INIT()
 {
 	UmodeAdd(modinfo->handle, 'S', UMODE_GLOBAL, 1, umode_allow_none, &UMODE_SERVICEBOT);
 	
@@ -64,28 +64,28 @@ MOD_INIT(servicebot)
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(servicebot)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(servicebot)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-int servicebot_can_kick(aClient *sptr, aClient *target, aChannel *chptr, char *comment,
-                    long sptr_flags, long target_flags, char **reject_reason)
+int servicebot_can_kick(Client *client, Client *target, Channel *channel, char *comment,
+                    long client_flags, long target_flags, char **reject_reason)
 {
 	static char errmsg[NICKLEN+256];
 
-	if (MyClient(sptr) && !IsULine(sptr) && IsServiceBot(target))
+	if (MyUser(client) && !IsULine(client) && IsServiceBot(target))
 	{
 		char errmsg2[NICKLEN+32];
 		snprintf(errmsg2, sizeof(errmsg2), "%s is a Service Bot", target->name);
 		
-		snprintf(errmsg, sizeof(errmsg), err_str(ERR_CANNOTDOCOMMAND),
-				   me.name, sptr->name, "KICK", errmsg2);
+		snprintf(errmsg, sizeof(errmsg), ":%s %d %s %s :%s",
+		         me.name, ERR_CANNOTDOCOMMAND, client->name, "KICK", errmsg2);
 
 		*reject_reason = errmsg;
 
@@ -95,18 +95,15 @@ int servicebot_can_kick(aClient *sptr, aClient *target, aChannel *chptr, char *c
 	return EX_ALLOW;
 }
 
-int servicebot_mode_deop(aClient *sptr, aClient *target, aChannel *chptr,
+int servicebot_mode_deop(Client *client, Client *target, Channel *channel,
                     u_int what, int modechar, long my_access, char **reject_reason)
 {
 	static char errmsg[NICKLEN+256];
 	
-	if (IsServiceBot(target) && MyClient(sptr) && !ValidatePermissionsForPath("services:servicebot:deop",sptr,target,chptr,NULL) && (what == MODE_DEL))
+	if (IsServiceBot(target) && MyUser(client) && !ValidatePermissionsForPath("services:servicebot:deop",client,target,channel,NULL) && (what == MODE_DEL))
 	{
-		char errmsg2[NICKLEN+32];
-		snprintf(errmsg2, sizeof(errmsg2), "%s is a Service Bot", target->name);
-		
-		snprintf(errmsg, sizeof(errmsg), err_str(ERR_CANNOTCHANGECHANMODE),
-			me.name, sptr->name, (char)modechar, errmsg2);
+		snprintf(errmsg, sizeof(errmsg), ":%s %d %s %c :%s is a Service Bot",
+			me.name, ERR_CANNOTCHANGECHANMODE, client->name, (char)modechar, target->name);
 		
 		*reject_reason = errmsg;
 		
@@ -116,29 +113,28 @@ int servicebot_mode_deop(aClient *sptr, aClient *target, aChannel *chptr,
 	return EX_ALLOW;
 }
 
-int servicebot_pre_kill(aClient *sptr, aClient *target, char *reason)
+int servicebot_pre_kill(Client *client, Client *target, char *reason)
 {
-	if (IsServiceBot(target) && !(ValidatePermissionsForPath("services:servicebot:kill",sptr,target,NULL,NULL) || IsULine(sptr)))
+	if (IsServiceBot(target) && !(ValidatePermissionsForPath("services:servicebot:kill",client,target,NULL,NULL) || IsULine(client)))
 	{
-		sendto_one(sptr, err_str(ERR_KILLDENY), me.name,
-			sptr->name, target->name);
+		sendnumeric(client, ERR_KILLDENY, target->name);
 		return EX_ALWAYS_DENY;
 	}
 	return EX_ALLOW;
 }
 
-int servicebot_whois(aClient *sptr, aClient *acptr)
+int servicebot_whois(Client *requester, Client *acptr)
 {
-	int hideoper = (IsHideOper(acptr) && (sptr != acptr) && !IsOper(sptr)) ? 1 : 0;
+	int hideoper = (IsHideOper(acptr) && (requester != acptr) && !IsOper(requester)) ? 1 : 0;
 
 	if (IsServiceBot(acptr) && !hideoper)
-		sendto_one(sptr, WHOIS_SERVICE_STRING, me.name, sptr->name, acptr->name);
+		sendto_one(requester, NULL, WHOIS_SERVICE_STRING, me.name, requester->name, acptr->name);
 
 	return 0;
 }
 
 /* This hides the servicebot, even if you are in the same channel, unless oper overriding */
-int servicebot_see_channel_in_whois(aClient *sptr, aClient *target, aChannel *chptr)
+int servicebot_see_channel_in_whois(Client *client, Client *target, Channel *channel)
 {
 	if (IsServiceBot(target))
 		return EX_DENY;
