@@ -19,28 +19,28 @@
 #include "unrealircd.h"
 
 /* Types */
-typedef struct _configitem_webirc ConfigItem_webirc;
+typedef struct ConfigItem_webirc ConfigItem_webirc;
 
 typedef enum {
 	WEBIRC_PASS=1, WEBIRC_WEBIRC=2
 } WEBIRCType;
 
-struct _configitem_webirc {
+struct ConfigItem_webirc {
 	ConfigItem_webirc *prev, *next;
 	ConfigFlag flag;
 	ConfigItem_mask *mask;
 	WEBIRCType type;
-	anAuthStruct *auth;
+	AuthConfig *auth;
 };
 
 /* Module header */
-ModuleHeader MOD_HEADER(webirc)
+ModuleHeader MOD_HEADER
 = {
 	"webirc",
-	"4.2",
+	"5.0",
 	"WebIRC/CGI:IRC Support",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-5",
 };
 
 /* Global variables */
@@ -48,9 +48,9 @@ ModDataInfo *webirc_md = NULL;
 ConfigItem_webirc *conf_webirc = NULL;
 
 /* Forward declarations */
-CMD_FUNC(m_webirc);
-int webirc_check_init(aClient *cptr, char *sockn, size_t size);
-int webirc_local_pass(aClient *sptr, char *password);
+CMD_FUNC(cmd_webirc);
+int webirc_check_init(Client *client, char *sockn, size_t size);
+int webirc_local_pass(Client *client, char *password);
 int webirc_config_test(ConfigFile *, ConfigEntry *, int, int *);
 int webirc_config_run(ConfigFile *, ConfigEntry *, int);
 void webirc_free_conf(void);
@@ -58,7 +58,7 @@ void delete_webircblock(ConfigItem_webirc *e);
 char *webirc_md_serialize(ModData *m);
 void webirc_md_unserialize(char *str, ModData *m);
 void webirc_md_free(ModData *md);
-int webirc_secure_connect(aClient *acptr);
+int webirc_secure_connect(Client *client);
 
 #define IsWEBIRC(x)			(moddata_client(x, webirc_md).l)
 #define IsWEBIRCSecure(x)	(moddata_client(x, webirc_md).l == 2)
@@ -68,14 +68,14 @@ int webirc_secure_connect(aClient *acptr);
 
 #define MSG_WEBIRC "WEBIRC"
 
-MOD_TEST(webirc)
+MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, webirc_config_test);
 	return MOD_SUCCESS;
 }
 
 /** Called upon module init */
-MOD_INIT(webirc)
+MOD_INIT()
 {
 	ModDataInfo mreq;
 
@@ -100,19 +100,19 @@ MOD_INIT(webirc)
 	HookAdd(modinfo->handle, HOOKTYPE_LOCAL_PASS, 0, webirc_local_pass);
 	HookAdd(modinfo->handle, HOOKTYPE_SECURE_CONNECT, 0, webirc_secure_connect);
 
-	CommandAdd(modinfo->handle, MSG_WEBIRC, m_webirc, MAXPARA, M_UNREGISTERED);
+	CommandAdd(modinfo->handle, MSG_WEBIRC, cmd_webirc, MAXPARA, CMD_UNREGISTERED);
 		
 	return MOD_SUCCESS;
 }
 
 /** Called upon module load */
-MOD_LOAD(webirc)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
 /** Called upon unload */
-MOD_UNLOAD(webirc)
+MOD_UNLOAD()
 {
 	webirc_free_conf();
 	return MOD_SUCCESS;
@@ -134,14 +134,14 @@ void delete_webircblock(ConfigItem_webirc *e)
 {
 	unreal_delete_masks(e->mask);
 	if (e->auth)
-		Auth_DeleteAuthStruct(e->auth);
+		Auth_FreeAuthConfig(e->auth);
 	DelListItem(e, conf_webirc);
-	MyFree(e);
+	safe_free(e);
 }
 
 int webirc_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 {
-	ConfigEntry	*cep, *cepp;
+	ConfigEntry *cep;
 	int errors = 0;
 	char has_mask = 0; /* mandatory */
 	char has_password = 0; /* mandatory */
@@ -250,7 +250,6 @@ int webirc_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 int webirc_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 {
 	ConfigEntry *cep;
-	ConfigEntry *cepp;
 	ConfigItem_webirc *webirc = NULL;
 	
 	if (type != CONFIG_MAIN)
@@ -259,7 +258,7 @@ int webirc_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "webirc"))
 		return 0; /* not interested */
 
-	webirc = MyMallocEx(sizeof(ConfigItem_webirc));
+	webirc = safe_alloc(sizeof(ConfigItem_webirc));
 	webirc->type = WEBIRC_WEBIRC; /* default */
 
 	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
@@ -267,7 +266,7 @@ int webirc_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		if (!strcmp(cep->ce_varname, "mask"))
 			unreal_add_masks(&webirc->mask, cep);
 		else if (!strcmp(cep->ce_varname, "password"))
-			webirc->auth = Auth_ConvertConf2AuthStruct(cep);
+			webirc->auth = AuthBlockToAuthConfig(cep);
 		else if (!strcmp(cep->ce_varname, "type"))
 		{
 			if (!strcmp(cep->ce_vardata, "webirc"))
@@ -304,19 +303,19 @@ void webirc_md_free(ModData *md)
 	md->l = 0;
 }
 
-ConfigItem_webirc *Find_webirc(aClient *sptr, char *password, WEBIRCType type, char **errorstr)
+ConfigItem_webirc *find_webirc(Client *client, char *password, WEBIRCType type, char **errorstr)
 {
 	ConfigItem_webirc *e;
 	char *error = NULL;
 
 	for (e = conf_webirc; e; e = e->next)
 	{
-		if ((e->type == type) && unreal_mask_match(sptr, e->mask))
+		if ((e->type == type) && unreal_mask_match(client, e->mask))
 		{
 			if (type == WEBIRC_WEBIRC)
 			{
 				/* Check password */
-				if (Auth_Check(sptr, e->auth, password) == -1)
+				if (!Auth_Check(client, e->auth, password))
 					error = "CGI:IRC -- Invalid password";
 				else
 					return e; /* Found matching block, return straight away */
@@ -338,50 +337,51 @@ ConfigItem_webirc *Find_webirc(aClient *sptr, char *password, WEBIRCType type, c
 #define WEBIRC_STRINGLEN  (sizeof(WEBIRC_STRING)-1)
 
 /* Does the CGI:IRC host spoofing work */
-int dowebirc(aClient *cptr, char *ip, char *host, char *options)
+void dowebirc(Client *client, char *ip, char *host, char *options)
 {
 	char scratch[64];
-	char *sockhost;
-	int secure = 0;
 
-	if (IsWEBIRC(cptr))
-		return exit_client(cptr, cptr, &me, "Double CGI:IRC request (already identified)");
+	if (IsWEBIRC(client))
+	{
+		exit_client(client, NULL, "Double CGI:IRC request (already identified)");
+		return;
+	}
 
 	if (host && !strcmp(ip, host))
 		host = NULL; /* host did not resolve, make it NULL */
 
-	/* STEP 1: Update cptr->local->ip
+	/* STEP 1: Update client->local->ip
 	   inet_pton() returns 1 on success, 0 on bad input, -1 on bad AF */
 	if ((inet_pton(AF_INET, ip, scratch) != 1) &&
 	    (inet_pton(AF_INET6, ip, scratch) != 1))
 	{
 		/* then we have an invalid IP */
-		return exit_client(cptr, cptr, &me, "Invalid IP address");
+		exit_client(client, NULL, "Invalid IP address");
+		return;
 	}
 
 	/* STEP 2: Update GetIP() */
-	safefree(cptr->ip);
-	cptr->ip = strdup(ip);
+	safe_strdup(client->ip, ip);
 		
-	/* STEP 3: Update cptr->local->hostp */
+	/* STEP 3: Update client->local->hostp */
 	/* (free old) */
-	if (cptr->local->hostp)
+	if (client->local->hostp)
 	{
-		unreal_free_hostent(cptr->local->hostp);
-		cptr->local->hostp = NULL;
+		unreal_free_hostent(client->local->hostp);
+		client->local->hostp = NULL;
 	}
 	/* (create new) */
 	if (host && verify_hostname(host))
-		cptr->local->hostp = unreal_create_hostent(host, cptr->ip);
+		client->local->hostp = unreal_create_hostent(host, client->ip);
 
 	/* STEP 4: Update sockhost
 	   Make sure that if this any IPv4 address is _not_ prefixed with
 	   "::ffff:" by using Inet_ia2p().
 	 */
 	// Hmm I ignored above warning. May be bad during transition period.
-	strlcpy(cptr->local->sockhost, cptr->ip, sizeof(cptr->local->sockhost));
+	strlcpy(client->local->sockhost, client->ip, sizeof(client->local->sockhost));
 
-	SetWEBIRC(cptr);
+	SetWEBIRC(client);
 
 	if (options)
 	{
@@ -391,37 +391,36 @@ int dowebirc(aClient *cptr, char *ip, char *host, char *options)
 			p2 = strchr(name, '=');
 			if (p2)
 				*p2 = '\0';
-			if (!strcmp(name, "secure") && IsSecure(cptr))
+			if (!strcmp(name, "secure") && IsSecure(client))
 			{
 				/* The entire [client]--[webirc gw]--[server] chain is secure */
-				SetWEBIRCSecure(cptr);
+				SetWEBIRCSecure(client);
 			}
 		}
 	}
 
 	/* blacklist_start_check() */
 	if (RCallbacks[CALLBACKTYPE_BLACKLIST_CHECK] != NULL)
-		RCallbacks[CALLBACKTYPE_BLACKLIST_CHECK]->func.intfunc(cptr);
+		RCallbacks[CALLBACKTYPE_BLACKLIST_CHECK]->func.intfunc(client);
 
 	/* Check (g)zlines right now; these are normally checked upon accept(),
 	 * but since we know the IP only now after PASS/WEBIRC, we have to check
 	 * here again...
 	 */
-	return check_banned(cptr);
+	check_banned(client, 0);
 }
 
 /* WEBIRC <pass> "cgiirc" <hostname> <ip> [:option1 [option2...]]*/
-CMD_FUNC(m_webirc)
+CMD_FUNC(cmd_webirc)
 {
 	char *ip, *host, *password, *options;
-	size_t ourlen;
 	ConfigItem_webirc *e;
 	char *error = NULL;
 
 	if ((parc < 5) || BadPtr(parv[4]))
 	{
-		sendto_one(cptr, err_str(ERR_NEEDMOREPARAMS), me.name, "*", "WEBIRC");
-		return -1;
+		sendnumeric(client, ERR_NEEDMOREPARAMS, "WEBIRC");
+		return;
 	}
 
 	password = parv[1];
@@ -430,27 +429,29 @@ CMD_FUNC(m_webirc)
 	options = parv[5]; /* can be NULL */
 
 	/* Check if allowed host */
-	e = Find_webirc(sptr, password, WEBIRC_WEBIRC, &error);
+	e = find_webirc(client, password, WEBIRC_WEBIRC, &error);
 	if (!e)
-		return exit_client(cptr, sptr, &me, error);
+	{
+		exit_client(client, NULL, error);
+		return;
+	}
 
 	/* And do our job.. */
-	return dowebirc(cptr, ip, host, options);
+	dowebirc(client, ip, host, options);
 }
 
-
-int webirc_check_init(aClient *cptr, char *sockn, size_t size)
+int webirc_check_init(Client *client, char *sockn, size_t size)
 {
-	if (IsWEBIRC(cptr))
+	if (IsWEBIRC(client))
 	{
-		strlcpy(sockn, GetIP(cptr), size); /* use already set value */
-		return 0;
+		strlcpy(sockn, GetIP(client), size); /* use already set value */
+		return HOOK_DENY;
 	}
 	
-	return 1; /* nothing to do */
+	return HOOK_CONTINUE; /* nothing to do */
 }
 
-int webirc_local_pass(aClient *sptr, char *password)
+int webirc_local_pass(Client *client, char *password)
 {
 	if (!strncmp(password, WEBIRC_STRING, WEBIRC_STRINGLEN))
 	{
@@ -458,7 +459,7 @@ int webirc_local_pass(aClient *sptr, char *password)
 		ConfigItem_webirc *e;
 		char *error = NULL;
 
-		e = Find_webirc(sptr, NULL, WEBIRC_PASS, &error);
+		e = find_webirc(client, NULL, WEBIRC_PASS, &error);
 		if (e)
 		{
 			/* Ok now we got that sorted out, proceed:
@@ -469,25 +470,29 @@ int webirc_local_pass(aClient *sptr, char *password)
 			ip = password + WEBIRC_STRINGLEN;
 			host = strchr(ip, '_');
 			if (!host)
-				return exit_client(sptr, sptr, &me, "Invalid CGI:IRC IP received");
+			{
+				exit_client(client, NULL, "Invalid CGI:IRC IP received");
+				return HOOK_DENY;
+			}
 			*host++ = '\0';
 		
-			return dowebirc(sptr, ip, host, NULL);
+			dowebirc(client, ip, host, NULL);
+			return HOOK_DENY;
 		}
 		/* fallthrough if not in webirc block.. */
 	}
 
-	return 0; /* not webirc */
+	return HOOK_CONTINUE; /* not webirc */
 }
 
 /** Called from register_user() right after setting user +z */
-int webirc_secure_connect(aClient *acptr)
+int webirc_secure_connect(Client *client)
 {
 	/* Remove secure mode (-z) if the WEBIRC gateway did not ensure
 	 * us that their [client]--[webirc gateway] connection is also
 	 * secure (eg: using https)
 	 */
-	if (IsWEBIRC(acptr) && IsSecureConnect(acptr) && !IsWEBIRCSecure(acptr))
-		acptr->umodes &= ~UMODE_SECURE;
+	if (IsWEBIRC(client) && IsSecureConnect(client) && !IsWEBIRCSecure(client))
+		client->umodes &= ~UMODE_SECURE;
 	return 0;
 }
