@@ -7,12 +7,11 @@
  
 #include "unrealircd.h"
 
-extern ConfigFile *config_load(char *filename);
 extern void config_free(ConfigFile *cfptr);
 
 char configfiletmp[512];
 
-struct _upgrade
+struct Upgrade
 {
 	char *locop_host;
 	char *oper_host;
@@ -23,10 +22,10 @@ struct _upgrade
 	int host_on_oper_up;
 };
 
-struct _upgrade upgrade;
+struct Upgrade upgrade;
 
-typedef struct flagmapping FlagMapping;
-struct flagmapping
+typedef struct FlagMapping FlagMapping;
+struct FlagMapping
 {
 	char shortflag;
 	char *longflag;
@@ -99,7 +98,7 @@ void modify_file(int start, char *ins, int stop)
 		die();
 	}
 
-	rdbuf = MyMallocEx(start);
+	rdbuf = safe_alloc(start);
 	
 	if ((n = fread(rdbuf, 1, start, fdi)) != start)
 	{
@@ -109,7 +108,7 @@ void modify_file(int start, char *ins, int stop)
 	
 	fwrite(rdbuf, 1, start, fdo);
 	
-	safefree(rdbuf);
+	safe_free(rdbuf);
 
 	if (ins)
 		fwrite(ins, 1, strlen(ins), fdo); /* insert this piece */
@@ -121,7 +120,7 @@ void modify_file(int start, char *ins, int stop)
 	}
 	
 	// read the remaining stuff
-	rdbuf = MyMallocEx(CFGBUFSIZE);
+	rdbuf = safe_alloc(CFGBUFSIZE);
 	
 	while(1)
 	{
@@ -155,7 +154,7 @@ end:
 	fclose(fdi);
 	fclose(fdo);
 	
-	safefree(rdbuf);
+	safe_free(rdbuf);
 	// todo: handle write errors and such..
 
 	unlink(configfiletmp);
@@ -207,53 +206,9 @@ void replace_section(ConfigEntry *ce, char *buf)
 
 static char buf[8192];
 
-int updconf_addquotes_r(char *i, char *o, size_t len)
-{
-	if (len == 0)
-		return 0;
-	
-	len--; /* reserve room for nul byte */
-
-	if (len == 0)
-	{
-		*o = '\0';
-		return 0;
-	}
-	
-	for (; *i; i++)
-	{
-		if ((*i == '"') || (*i == '\\')) /* only " and \ need to be quoted */
-		{
-			if (len < 2)
-				break;
-			*o++ = '\\';
-			*o++ = *i;
-			len -= 2;
-		} else
-		{
-			if (len == 0)
-				break;
-			*o++ = *i;
-			len--;
-		}
-	}
-	*o = '\0';
-	
-	return 1;
-}	
-
-char *updconf_addquotes(char *str)
-{
-	static char qbuf[2048];
-	
-	*qbuf = '\0';
-	updconf_addquotes_r(str, qbuf, sizeof(qbuf));
-	return qbuf;
-}
-
 int upgrade_me_block(ConfigEntry *ce)
 {
-	ConfigEntry *cep, *cepp;
+	ConfigEntry *cep;
 	char *name = NULL;
 	char *info = NULL;
 	int numeric = 0;
@@ -323,7 +278,6 @@ int upgrade_link_block(ConfigEntry *ce)
 	char *ciphers = NULL;
 	char *password_receive_authmethod = NULL;
 	int need_incoming = 0, need_outgoing = 0;
-	int upg_passwd_warning = 0;
 
 	/* ripped from test_link */
 	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
@@ -493,7 +447,7 @@ int upgrade_link_block(ConfigEntry *ce)
 #define MAXFROMENTRIES 100
 int upgrade_from_subblock(ConfigEntry *ce)
 {
-	ConfigEntry *cep, *cepp;
+	ConfigEntry *cep;
 	char *list[MAXFROMENTRIES];
 	int listcnt = 0;
 
@@ -595,13 +549,12 @@ int upgrade_loadmodule(ConfigEntry *ce)
 int upgrade_include(ConfigEntry *ce)
 {
 	char *file = ce->ce_vardata;
-	char tmp[512], *p, *newfile;
 	static int badwords_upgraded_already = 0;
 	
 	if (!file)
 		return 0;
 
-	if (!strstr(file, "help/") && !_match("help*.conf", file))
+	if (!strstr(file, "help/") && match_simple("help*.conf", file))
 	{
 		snprintf(buf, sizeof(buf), "include \"help/%s\";\n", file);
 		replace_section(ce, buf);
@@ -617,7 +570,7 @@ int upgrade_include(ConfigEntry *ce)
 		return 1;
 	}
 
-	if (!_match("badwords.*.conf", file))
+	if (match_simple("badwords.*.conf", file))
 	{
 		if (badwords_upgraded_already)
 		{
@@ -727,13 +680,13 @@ int upgrade_spamfilter_block(ConfigEntry *ce)
 	                           "\ttarget %s;\n"
 	                           "\taction %s;\n",
 	                           match_type,
-	                           updconf_addquotes(regex),
+	                           unreal_add_quotes(regex),
 	                           targets,
 	                           action);
 
 	/* optional: reason */
 	if (reason)
-		snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "\treason \"%s\";\n", updconf_addquotes(reason));
+		snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "\treason \"%s\";\n", unreal_add_quotes(reason));
 	
 	/* optional: ban-time */
 	if (ban_time)
@@ -1036,7 +989,7 @@ int upgrade_listen_block(ConfigEntry *ce)
 
 int upgrade_cgiirc_block(ConfigEntry *ce)
 {
-	ConfigEntry *cep, *cepp;
+	ConfigEntry *cep;
 	char *type = NULL;
 	char *username = NULL;
 	char *hostname = NULL;
@@ -1391,7 +1344,7 @@ void update_read_settings(char *cfgfile)
 	ConfigFile *cf = NULL;
 	ConfigEntry *ce = NULL, *cep, *cepp;
 
-	cf = config_load(cfgfile);
+	cf = config_load(cfgfile, NULL);
 	if (!cf)
 		return;
 		
@@ -1414,22 +1367,22 @@ void update_read_settings(char *cfgfile)
 						if (!cepp->ce_vardata)
 							continue;
 						if (!strcmp(cepp->ce_varname, "local")) {
-							safestrdup(upgrade.locop_host, cepp->ce_vardata);
+							safe_strdup(upgrade.locop_host, cepp->ce_vardata);
 						}
 						else if (!strcmp(cepp->ce_varname, "global")) {
-							safestrdup(upgrade.oper_host, cepp->ce_vardata);
+							safe_strdup(upgrade.oper_host, cepp->ce_vardata);
 						}
 						else if (!strcmp(cepp->ce_varname, "coadmin")) {
-							safestrdup(upgrade.coadmin_host, cepp->ce_vardata);
+							safe_strdup(upgrade.coadmin_host, cepp->ce_vardata);
 						}
 						else if (!strcmp(cepp->ce_varname, "admin")) {
-							safestrdup(upgrade.admin_host, cepp->ce_vardata);
+							safe_strdup(upgrade.admin_host, cepp->ce_vardata);
 						}
 						else if (!strcmp(cepp->ce_varname, "servicesadmin")) {
-							safestrdup(upgrade.sadmin_host, cepp->ce_vardata);
+							safe_strdup(upgrade.sadmin_host, cepp->ce_vardata);
 						}
 						else if (!strcmp(cepp->ce_varname, "netadmin")) {
-							safestrdup(upgrade.netadmin_host, cepp->ce_vardata);
+							safe_strdup(upgrade.netadmin_host, cepp->ce_vardata);
 						}
 						else if (!strcmp(cepp->ce_varname, "host-on-oper-up")) {
 							upgrade.host_on_oper_up = config_checkval(cepp->ce_vardata,CFG_YESNO);
@@ -1463,7 +1416,7 @@ again:
 		cf = NULL;
 	}
 
-	cf = config_load(configfiletmp);
+	cf = config_load(configfiletmp, NULL);
 	if (!cf)
 	{
 		config_error("could not load configuration file '%s'", configfile);
@@ -1597,17 +1550,17 @@ static int already_included(char *fname, ConfigFile *cf)
 
 static void add_include_list(char *fname, ConfigFile **cf)
 {
-	ConfigFile *n = MyMallocEx(sizeof(ConfigFile));
+	ConfigFile *n = safe_alloc(sizeof(ConfigFile));
 	
 //	config_status("INCLUDE: %s", fname);
-	n->cf_filename = strdup(fname);
+	safe_strdup(n->cf_filename, fname);
 	n->cf_next = *cf;
 	*cf = n;
 }
 
 void build_include_list_ex(char *fname, ConfigFile **cf_list)
 {
-	ConfigFile *cf, *cf2;
+	ConfigFile *cf;
 	ConfigEntry *ce;
 
 	if (strstr(fname, "://"))
@@ -1615,7 +1568,7 @@ void build_include_list_ex(char *fname, ConfigFile **cf_list)
 
 	add_include_list(fname, cf_list);
 
-	cf = config_load(fname);
+	cf = config_load(fname, NULL);
 	if (!cf)
 		return;
 
@@ -1624,9 +1577,9 @@ void build_include_list_ex(char *fname, ConfigFile **cf_list)
 		{
 			if ((ce->ce_vardata[0] != '/') && (ce->ce_vardata[0] != '\\') && strcmp(ce->ce_vardata, CPATH))
 			{
-				char *str = MyMallocEx(strlen(ce->ce_vardata) + strlen(CONFDIR) + 4);
+				char *str = safe_alloc(strlen(ce->ce_vardata) + strlen(CONFDIR) + 4);
 				sprintf(str, "%s/%s", CONFDIR, ce->ce_vardata);
-				MyFree(ce->ce_vardata);
+				safe_free(ce->ce_vardata);
 				ce->ce_vardata = str;
 			}
 			if (!already_included(ce->ce_vardata, *cf_list))
@@ -1648,11 +1601,34 @@ void update_conf(void)
 {
 	ConfigFile *files;
 	ConfigFile *cf;
-	ConfigEntry *ce, *cep, *cepp;
 	char *mainconf = configfile;
 	int upgraded_files = 0;
+	char answerbuf[128], *answer;
 
-	config_status("Attempting to upgrade '%s' (and all it's included files) from UnrealIRCd 3.2.x to UnrealIRCd 4.x...", configfile);
+	config_status("You have requested to upgrade your configuration files.");
+	config_status("If you are upgrading from 4.x to 5.x then DO NOT run this script. This script does NOT update config files from 4.x -> 5.x.");
+	config_status("UnrealIRCd 4.2.x configuration files should work OK on 5.x, with only some warnings printed when you boot the IRCd.");
+	config_status("See https://www.unrealircd.org/docs/Upgrading_from_4.x#Configuration_changes");
+	config_status("This upgrade-conf script is only useful if you are upgrading from 3.2.x.");
+	config_status("");
+#ifndef _WIN32
+	do
+	{
+		printf("Continue upgrading 3.2.x to 4.x configuration file format? (Y/N): ");
+		*answerbuf = '\0';
+		answer = fgets(answerbuf, sizeof(answerbuf), stdin);
+		if (answer && (toupper(*answer) == 'N'))
+		{
+			printf("Configuration unchanged.\n");
+			return;
+		}
+		if (answer && (toupper(*answer) == 'Y'))
+		{
+			break;
+		}
+		printf("Invalid response. Please enter either Y or N\n\n");
+	} while(1);
+#endif
 	
 	strlcpy(me.name, "<server>", sizeof(me.name));
 	memset(&upgrade, 0, sizeof(upgrade));
